/*
 *
 * Copyright (c) 1998, 1999 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

#include <string.h>
#include <bsp/bsp.h>
#include <bsp/cpu.h>
#include <bsp/platform.h>
#include "bsp_if.h"
#include <bsp/i82559.h>
#include "net.h"
#include "bsplog.h"
#include <bsp/pci_cfg.h>
#include <bsp/skbuff.h>

#define ALLOW_GDB_HOOK

/* local debug options */
//#define DC_SOFT_INTERRUPT       1
//#define OUTPUT_SA_CONTROL_DEBUG_INFO	1

//#define DC_DUMP_TX_INFO		1 	
//#define DC_FORCE_TO_10_MB	1
//#define SHOW_READ_EEPROM	1

//#define SHOW_TX_INFO        1
//#define SHOW_RX_INFO        1
//#define DUMP_RX_MODE_INFO	1

//#define INIFINITE_LOOP_FOR_TEST     1 
//#define SHOW_RX_INFO    1 
//#define USE_I82559_INTERRUPT    1
//#define SHOW_POLLING_READ_INFO     1

//#define SHOW_CURTX_INFO		1
//#define QUICK_FIX_TEST_CURTX	1
//#ifdef QUICK_FIX_TEST_CURTX
//    static int first_time_calling_polling_xmit=1;
//#endif //#ifdef QUICK_FIX_TEST_CURTX

struct pci_dev *pdev_i82559=NULL;

void dump_buf(char rbuff[], int nReadCnt);
void *malloc(int size);
void free(void *buff);

typedef unsigned char u8;
typedef unsigned char byte;
typedef unsigned short u16;
typedef short s16;
typedef int s32;
typedef int u32;


#if 0
#define DEBUG_ENET 32
#endif

#if DEBUG_ENET
static void DumpStatus(void);
#endif

void _setup_i82559_vec();   

static int congenb = 0;        /* Enable congestion control in the DP83840. */
static int txfifo = 8;        /* Tx FIFO threshold in 4 byte units, 0-15 */
static int rxfifo = 8;        /* Rx FIFO threshold, default 32 bytes. */
/* Tx/Rx DMA burst length, 0-127, 0 == no preemption, tx==128 -> disabled. */
static int txdmacount = 128;
static int rxdmacount = 0;

/* Set the copy breakpoint for the copy-only-tiny-buffer Rx method.
   Lower values use more memory, but are faster. */
static int rx_copybreak = 200;

/* Maximum events (Rx packets, etc.) to handle at each interrupt. */
static int max_interrupt_work = 200;


/* following staff are take from Linux Intel 82557 driver eepro100.c */

/*
                Theory of Operation

I. Board Compatibility

This device driver is designed for the Intel i82557 "Speedo3" chip, Intel's
single-chip fast Ethernet controller for PCI, as used on the Intel
EtherExpress Pro 100 adapter.

II. Board-specific settings

PCI bus devices are configured by the system at boot time, so no jumpers
need to be set on the board.  The system BIOS should be set to assign the
PCI INTA signal to an otherwise unused system IRQ line.  While it's
possible to share PCI interrupt lines, it negatively impacts performance and
only recent kernels support it.

III. Driver operation

IIIA. General
The Speedo3 is very similar to other Intel network chips, that is to say
"apparently designed on a different planet".  This chips retains the complex
Rx and Tx descriptors and multiple buffers pointers as previous chips, but
also has simplified Tx and Rx buffer modes.  This driver uses the "flexible"
Tx mode, but in a simplified lower-overhead manner: it associates only a
single buffer descriptor with each frame descriptor.

Despite the extra space overhead in each receive skbuff, the driver must use
the simplified Rx buffer mode to assure that only a single data buffer is
associated with each RxFD. The driver implements this by reserving space
for the Rx descriptor at the head of each Rx skbuff.

The Speedo-3 has receive and command unit base addresses that are added to
almost all descriptor pointers.  The driver sets these to zero, so that all
pointer fields are absolute addresses.

The System Control Block (SCB) of some previous Intel chips exists on the
chip in both PCI I/O and memory space.  This driver uses the I/O space
registers, but might switch to memory mapped mode to better support non-x86
processors.

IIIB. Transmit structure

The driver must use the complex Tx command+descriptor mode in order to
have a indirect pointer to the skbuff data section.  Each Tx command block
(TxCB) is associated with two immediately appended Tx Buffer Descriptor
(TxBD).  A fixed ring of these TxCB+TxBD pairs are kept as part of the
speedo_private data structure for each adapter instance.

The newer i82558 explicitly supports this structure, and can read the two
TxBDs in the same PCI burst as the TxCB.

This ring structure is used for all normal transmit packets, but the
transmit packet descriptors aren't long enough for most non-Tx commands such
as CmdConfigure.  This is complicated by the possibility that the chip has
already loaded the link address in the previous descriptor.  So for these
commands we convert the next free descriptor on the ring to a NoOp, and point
that descriptor's link to the complex command.

An additional complexity of these non-transmit commands are that they may be
added asynchronous to the normal transmit queue, so we disable interrupts
whenever the Tx descriptor ring is manipulated.

A notable aspect of these special configure commands is that they do
work with the normal Tx ring entry scavenge method.  The Tx ring scavenge
is done at interrupt time using the 'dirty_tx' index, and checking for the
command-complete bit.  While the setup frames may have the NoOp command on the
Tx ring marked as complete, but not have completed the setup command, this
is not a problem.  The tx_ring entry can be still safely reused, as the
tx_skbuff[] entry is always empty for config_cmd and mc_setup frames.

Commands may have bits set e.g. CmdSuspend in the command word to either
suspend or stop the transmit/command unit.  This driver always flags the last
command with CmdSuspend, erases the CmdSuspend in the previous command, and
then issues a CU_RESUME.

Note: Watch out for the potential race condition here: imagine
    erasing the previous suspend
        the chip processes the previous command
        the chip processes the final command, and suspends
    doing the CU_RESUME

        the chip processes the next-yet-valid post-final-command.
So blindly sending a CU_RESUME is only safe if we do it immediately after
after erasing the previous CmdSuspend, without the possibility of an
intervening delay.  Thus the resume command is always within the
interrupts-disabled region.  This is a timing dependence, but handling this
condition in a timing-independent way would considerably complicate the code.

Note: In previous generation Intel chips, restarting the command unit was a
notoriously slow process.  This is presumably no longer true.


IIIC. Receive structure

Because of the bus-master support on the Speedo3 this driver uses the new
SKBUFF_RX_COPYBREAK scheme, rather than a fixed intermediate receive buffer.
This scheme allocates full-sized skbuffs as receive buffers.  The value
SKBUFF_RX_COPYBREAK is used as the copying breakpoint: it is chosen to
trade-off the memory wasted by passing the full-sized skbuff to the queue
layer for all frames vs. the copying cost of copying a frame to a
correctly-sized skbuff.

For small frames the copying cost is negligible (esp. considering that we
are pre-loading the cache with immediately useful header information), so we
allocate a new, minimally-sized skbuff.  For large frames the copying cost
is non-trivial, and the larger copy might flush the cache of useful data, so
we pass up the skbuff the packet was received into.

IIID. Synchronization

The driver runs as two independent, single-threaded flows of control.  One
is the send-packet routine, which enforces single-threaded use by the
dev->tbusy flag.  The other thread is the interrupt handler, which is single
threaded by the hardware and other software.

The send packet thread has partial control over the Tx ring and 'dev->tbusy'
flag. It sets the tbusy flag whenever it's queuing a Tx packet. If the next
queue slot is empty, it clears the tbusy flag when finished otherwise it sets
the 'sp->tx_full' flag.

The interrupt handler has exclusive control over the Rx ring and records stats
from the Tx ring. (The Tx-done interrupt can't be selectively turned off, so
we can't avoid the interrupt overhead by having the Tx routine reap the Tx
stats.) After reaping the stats, it marks the queue entry as empty by setting
the 'base' to zero. Iff the 'sp->tx_full' flag is set, it clears both the
tx_full and tbusy flags.

IV. Notes

Thanks to Steve Williams of Intel for arranging the non-disclosure agreement
that stated that I could disclose the information.  But I still resent
having to sign an Intel NDA when I'm helping Intel sell their own product!
*/


/* 'options' is used to pass a transceiver override or full-duplex flag
 *  e.g. "options=16" for FD, "options=32" for 100mbps-only.
 */
static int options[] = {-1, -1, -1, -1, -1, -1, -1, -1};

/* A few values that may be tweaked. */
/* The ring sizes should be a power of two for efficiency. */

#define TX_RING_SIZE    16        /* Effectively 2 entries fewer. */
#define RX_RING_SIZE    16

/* Size of an pre-allocated Rx buffer: <Ethernet MTU> + slack.*/
#define PKT_BUF_SZ        1536

/* Time in jiffies before concluding the transmitter is hung. */
/* DC this is original  #define TX_TIMEOUT  ((800*HZ)/1000) */
#define TX_TIMEOUT_IN_MS  800 

#define IXP1200_PCI_IO_BASE     0x54000000  /* mmu off */

/* endian has no effect on long word (4 bytes) access */
#define inl(addr)  ((*(volatile unsigned long *)((addr + IXP1200_PCI_IO_BASE))))
#define outl(b,addr)  (*(volatile unsigned long *)((addr + IXP1200_PCI_IO_BASE)) = (b))

static unsigned char inb(unsigned int addr);
static unsigned short inw(unsigned int addr);
static void outb(unsigned char val_uchar, unsigned int addr);
static void outw(unsigned short val_ushort, unsigned int addr);

#define us_outw(b,addr)  (*(volatile unsigned short *)((addr + IXP1200_PCI_IO_BASE)) = (b))
#define us_outl(b,addr)  (*(volatile unsigned long *)((addr + IXP1200_PCI_IO_BASE)) = (b))

/* Typically this takes 0 ticks. */
static inline void wait_for_cmd_done(u32 cmd_ioaddr)
{
    int wait = 100;

    do   ;
    /* orignall using inb, for big endian test, try inw 
    while(inb(cmd_ioaddr) && --wait >= 0);
    */
    while(inb(cmd_ioaddr) && --wait >= 0);
}

enet_addr_t __local_enet_addr;
void flush_i82559_rcv_buf(void);


/* Offsets to the various registers.
   All accesses need not be longword aligned. */

enum SCBPort_cmds {
    PortReset=0, PortSelfTest=1, PortPartialReset=2, PortDump=3,
};

enum speedo_offsets {
    SCBStatus = 0, SCBCmd = 2,        /* Rx/Command Unit command and status. */
    SCBPointer = 4,                /* General purpose pointer. */
    SCBPort = 8,                /* Misc. commands and operands.  */
    SCBflash = 12, SCBeeprom = 14,    /* EEPROM and flash memory control. */
    SCBCtrlMDI = 16,            /* MDI interface control. */
    SCBEarlyRx = 20,            /* Early receive byte count. */
};

/* Commands that can be put in a command list entry. */
enum commands {
    CmdNOp = 0, CmdIASetup = 1, CmdConfigure = 2, CmdMulticastList = 3,
    CmdTx = 4, CmdTDR = 5, CmdDump = 6, CmdDiagnose = 7,
    CmdSuspend = 0x4000,            /* Suspend after completion. */
    CmdIntr = 0x2000,            /* Interrupt after completion. */
    CmdTxFlex = 0x0008,            /* Use "Flexible mode" for CmdTx command. */
};

#define CMDSUSPEND_32 0x40000000
#define NOT_CMDSUSPEND_32 0xBFFFFFFF
/* Do atomically if possible. */
#if defined(__i386__)
#define clear_suspend(cmd)   ((char *)(&(cmd)->cmd_status))[3] &= ~0x40
#elif defined(__alpha__)
#define clear_suspend(cmd)   clear_bit(30, &(cmd)->cmd_status)
#elif defined(__powerpc__) || defined(__sparc__) || (__BIG_ENDIAN)
#define clear_suspend(cmd)	clear_bit(6, &(cmd)->cmd_status)
#else
#warning Undefined architecture.
//#define clear_suspend(cmd)	(cmd)->cmd_status &= cpu_to_le32(~CMCmdSuspend)
//#define clear_suspend(cmd)	(cmd)->cmd_status &= cpu_to_le32(NOT_CMDSUSPEND_32)
#define clear_suspend(cmd)	(cmd)->cmd_status &= NOT_CMDSUSPEND_32
#endif

/* The SCB accepts the following controls for the Tx and Rx units: */
#define     CU_START        0x0010
#define     CU_RESUME        0x0020
#define     CU_STATSADDR    0x0040
#define     CU_SHOWSTATS    0x0050    /* Dump statistics counters. */
#define     CU_CMD_BASE    0x0060    /* Base address to add to add CU commands. */
#define     CU_DUMPSTATS    0x0070    /* Dump then reset stats counters. */
#define     RX_START    0x0001
#define     RX_RESUME    0x0002
#define     RX_ABORT    0x0004
#define     RX_ADDR_LOAD    0x0006
#define     RX_RESUMENR    0x0007
#define INT_MASK    0x0100
#define DRVR_INT    0x0200        /* Driver generated interrupt. */

/* The Speedo3 Rx and Tx frame/buffer descriptors. */
struct descriptor {        /* A generic descriptor. */
    s32 cmd_status;       /* all command and status field */
    u32 link;            /* struct descriptor *  */
    unsigned char params[0];
};

/* The Speedo3 Rx and Tx buffer descriptors. */
#ifdef __LITTLE_ENDIAN__
struct RxFD {                    /* Receive frame descriptor. */
    s32 status;
    u32 link;                /* struct RxFD * */
    u32 rx_buf_addr;            /* void * */
    u16 count;
    u16 size;
};
#else //#ifdef __LITTLE_ENDIAN__
struct RxFD {                    /* Receive frame descriptor. */
    s32 status;
    u32 link;                /* struct RxFD * */
    u32 rx_buf_addr;            /* void * */
    u16 size;
    u16 count;
};
#endif //#ifdef __LITTLE_ENDIAN__

/* Selected elements of the Tx/RxFD.status word. */
enum RxFD_bits {
    RxComplete=0x8000, RxOK=0x2000,
    RxErrCRC=0x0800, RxErrAlign=0x0400, RxErrTooBig=0x0200, RxErrSymbol=0x0010,
    RxEth2Type=0x0020, RxNoMatch=0x0004, RxNoIAMatch=0x0002,
    StatusComplete=0x8000,
};

struct TxFD {                    /* Transmit frame descriptor set. */
    s32 status;
    u32 link;                /* void * */
    u32 tx_desc_addr;            /* Always points to the tx_buf_addr element. */
    s32 count;                /* # of TBD (=1), Tx start thresh., etc. */
    /* This constitutes two "TBD" entries -- we only use one. */
    u32 tx_buf_addr0;            /* void *, frame to be transmitted.  */
    s32 tx_buf_size0;            /* Length of Tx frame. */
    u32 tx_buf_addr1;            /* void *, frame to be transmitted.  */
    s32 tx_buf_size1;            /* Length of Tx frame. */
};

struct enet_statistics {  
    u32 rx_errors;
        u32 rx_packets;
        u32 rx_bytes;
        u32 tx_errors;
    u32 tx_packets;
        u32 tx_bytes;
};

/* Elements of the dump_statistics block. This block must be lword aligned. */
struct speedo_stats {
    u32 tx_good_frames;
    u32 tx_coll16_errs;
    u32 tx_late_colls;
    u32 tx_underruns;
    u32 tx_lost_carrier;
    u32 tx_deferred;
    u32 tx_one_colls;
    u32 tx_multi_colls;
    u32 tx_total_colls;
    u32 rx_good_frames;
    u32 rx_crc_errs;
    u32 rx_align_errs;
    u32 rx_resource_errs;
    u32 rx_overrun_errs;
    u32 rx_colls_errs;
    u32 rx_runt_errs;
    u32 done_marker;
};

struct speedo_private {
    char devname[8];            /* Used only for kernel debugging. */
    const char *product_name;
    struct TxFD tx_ring[TX_RING_SIZE];   /* Commands (usually CmdTxPacket). */
    /* The saved address of a sent-in-place packet/buffer, for skfree(). */
    struct sk_buff* tx_skbuff[TX_RING_SIZE];
    struct descriptor  *last_cmd;    /* Last command sent. */

    /* Rx descriptor ring & addresses of receive-in-place skbuffs. */
    struct RxFD *rx_ringp[RX_RING_SIZE];
    struct sk_buff* rx_skbuff[RX_RING_SIZE];
    struct RxFD *last_rxf;            /* Last command sent. */
    struct enet_statistics stats;  
    struct speedo_stats lstats;
    long last_rx_time;            /* Last Rx, in jiffies, to handle Rx hang. */
    unsigned int cur_rx, cur_tx;        /* The next free ring entry */
    unsigned int dirty_rx, dirty_tx;    /* The ring entries to be free()ed. */
    int mc_setup_frm_len;            /* The length of an allocated.. */
    struct descriptor *mc_setup_frm;     /* ..multicast setup frame. */
    int mc_setup_busy;            /* Avoid double-use of setup frame. */
    char rx_mode;                /* Current PROMISC/ALLMULTI setting. */
    unsigned int tx_full:1;            /* The Tx queue is full. */
    unsigned int full_duplex:1;        /* Full-duplex operation requested. */
    unsigned int default_port:1;        /* Last dev->if_port value. */
    unsigned int rx_bug:1;            /* Work around receiver hang errata. */
    unsigned int rx_bug10:1;        /* Receiver might hang at 10mbps. */
    unsigned int rx_bug100:1;        /* Receiver might hang at 100mbps. */
    unsigned short phy[2];            /* PHY media interfaces available. */
};


/* a local device data structure */
struct local_device {
    unsigned int if_port;
    unsigned int tbusy;
    unsigned int interrupt;
    unsigned int start;
    u32 ioaddr;
    int irq;
    int trans_start;
};

struct local_device enet_dev;

/* The parameters for a CmdConfigure operation.
   There are so many options that it would be difficult to document each bit.
   We mostly use the default or recommended settings. */

const char i82557_config_cmd[22] = {
    22, 0x08, 0, 0,  0, 0x80, 0x32, 0x03,  1, /* 1=Use MII  0=Use AUI */
    0, 0x2E, 0,  0x60, 0,
    0xf2, 0x48,   0, 0x40, 0xf2, 0x80,    /* 0x40=Force full-duplex */
    0x3f, 0x05, };

//DC change array length to 24 for swapping in big endian, padding 0, 0
//DC const char i82558_config_cmd[22] = {
const char i82558_config_cmd[24] = {
    22, 0x08, 0, 1,  0, 0x80, 0x22, 0x03,  1, /* 1=Use MII  0=Use AUI */
    0, 0x2E, 0,  0x60, 0x08, 0x88,
    0x68, 0, 0x40, 0xf2, 0xBD,     /* 0xBD->0xFD=Force full-duplex */
//DC    0x31, 0x05, };
    0x31, 0x05, 0, 0};

/* PHY media interface chips. */

static const char *phys[] = {
    "None", "i82553-A/B", "i82553-C", "i82503",
    "DP83840", "80c240", "80c24", "i82555",
    "unknown-8", "unknown-9", "DP83840A", "unknown-11",
    "unknown-12", "unknown-13", "unknown-14", "unknown-15", };

enum phy_chips { NonSuchPhy=0, I82553AB, I82553C, I82503, DP83840, S80C240,
                S80C24, I82555, DP83840A=10, };
static const char is_mii[] = { 0, 1, 1, 0, 1, 1, 0, 1 };


/* The total I/O port extent of the board.
   The registers beyond 0x18 only exist on the i82558. */

#define SPEEDO3_TOTAL_SIZE 0x20

/* static int speedo_debug = 5; */
static int speedo_debug = 0;

/* Serial EEPROM section.
   A "bit" grungy, but we work our way through bit-by-bit :->. */

/*  EEPROM_Ctrl bits. */
#define EE_SHIFT_CLK    0x01    /* EEPROM shift clock. */
#define EE_CS            0x02    /* EEPROM chip select. */
#define EE_DATA_WRITE    0x04    /* EEPROM chip data in. */
#define EE_WRITE_0        0x01
#define EE_WRITE_1        0x05
#define EE_DATA_READ    0x08    /* EEPROM chip data out. */
#define EE_ENB            (0x4800 | EE_CS)

static void udelay(int delay);

/* Delay between EEPROM clock transitions.
  This will actually work with no delay on 33Mhz PCI.  */
#define eeprom_delay(nanosec)        udelay(1);

/* The EEPROM commands include the alway-set leading bit. */
#define EE_WRITE_CMD    (5 << addr_len)
#define EE_READ_CMD        (6 << addr_len)
#define EE_ERASE_CMD    (7 << addr_len)

static void speedo_found1(int busno, int deviceno, int funcno,
                         u32 ioaddr, int irq, int card_idx);
static int read_eeprom(u32 ioaddr, int location, int addr_len);
static int mdio_read(u32 ioaddr, int phy_id, int location);
static int mdio_write(u32 ioaddr, int phy_id, int location, int value);
static void speedo_open(void);
static void speedo_init_rx_ring(void);
static void speedo_tx_timeout(void);
static void set_rx_mode(void);

#ifdef honor_default_port
/* Optional driver feature to allow forcing the transceiver setting. Not recommended. */
static int mii_ctrl[8] = { 0x3300, 0x3100, 0x0000, 0x0100,
               0x2000, 0x2100, 0x0400, 0x3100};
#endif

/* A list of all installed Speedo devices, for removing the driver module. */
#define PCI_VENDOR_ID_INTEL  0x8086
#define PCI_DEVICE_ID_INTEL_82557   0x1229

static u16 eeprom[0x40];
struct speedo_private spdo;
static char enet_hw_addr[6];
char *spdo_start;
static u32 virt_to_bus(u32 ixp1200_SA_addr);
static u32 bus_to_virt(u32 pci_space_addr);
static int speedo_rx(void);

/* create 20 local buf for received data */
#define NUMB_LBUF    5
#define LENGTH_LBUF     1600
static char lbuf[NUMB_LBUF][LENGTH_LBUF];
static int  len_lbuf[NUMB_LBUF];
static int  lbuf_get_ptr=0;
static int  lbuf_put_ptr=0;
static int valid_lbuf_cnt=0;

static int display_i82559_info_only_once=1;

static void maintain_lbuf_put_ptr(void);
static void maintain_lbuf_get_ptr(void);

extern unsigned long get_ixp1200_system_time_in_ms(void);
extern int is_ixp1200_central_func;
extern unsigned int bo_kernel_load_offset;

void _init_i82559(int option); //DC i82559 initializaton 
void disable_i82559(); //DC disable i82559, prevent unexpected 
                       // garbage packets transmission

static void find_test_i82559(void);
static int cards_found = 0;
static void i559_init(enet_addr_t enet_addr); 

#ifdef USE_I82559_INTERRUPT        
static int speedo_start_xmit(struct sk_buff *skb);
#endif /* #ifdef USE_I82559_INTERRUPT */        


#ifndef USE_I82559_INTERRUPT        
static int polling_read_i82559(void);      
static int polling_xmit_i82559(struct sk_buff *skb);
#endif /* #ifndef USE_I82559_INTERRUPT */        

#ifndef __LITTLE_ENDIAN__
//for big endian, the contents of buf need to be swapped, so use two 
//local buffer to implements, the size of this buffer can be set to
//PKT_BUF_SZ (1536)
static char ch_buf_in[PKT_BUF_SZ]; 	//used in polling_xmit_i82559()
static char ch_buf_out[PKT_BUF_SZ];	//used in polling_xmit_i82559()
#endif //#ifndef __LITTLE_ENDIAN__
static char pr_buf[LENGTH_LBUF];	//used in polling_read_i82559()
#ifndef __LITTLE_ENDIAN__
static char pr_be_buf[LENGTH_LBUF];	//used in polling read_i82559()
#endif //#ifndef __LITTLE_ENDIAN__

static void find_test_i82559(void)
{
/* find and test i82559 */
//#define DC_TEST_IO 	1 
    static int pci_index = 0;
    int busno=0;     /* bus number */
    int deviceno=0;  /* device number */
    int funcno=0;   /* function number */
    unsigned char pci_latency;
    u32 ioaddr;
    int int_ioaddr;
    int irq;
    u16 pci_command=0, new_command=0;
#ifdef DC_TEST_IO
    /* i82559 general control register offset */
    #define GENERAL_CONTROL	0x1C 
    int i=0;
#endif  //#ifdef DC_TEST_IO


#if DEBUG_ENET >= 3
    BSPLOG(bsp_log("Initializing i82559...\n"));
#endif

    for (pci_index=0; pci_index < 8; pci_index++)
    {
        if (speedo_debug > 2)
            bsp_printf(" *** Looking for i82559 index %d\n", pci_index);

        pdev_i82559= pci_find_device(PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82559,
                        pci_index);

        if(pdev_i82559 == NULL) 
            break;

        /* Use [1] to memory map to CSR registers */
        pci_read_config_dword(pdev_i82559, PCI_BASE_ADDRESS_1, &int_ioaddr);
        ioaddr=(u32)int_ioaddr;
        pci_read_config_dword(pdev_i82559, PCI_INTERRUPT_LINE, &irq);
        /* Remove I/O space marker in bit 0. */
        ioaddr &= 0xFFFFFFFC;    /* ~3UL */;

        if (speedo_debug > 2)
            bsp_printf("Found Intel i82559 at I/O %08X, IRQ %08X.\n",
                        ioaddr, irq);

        /* Get and check the bus-master and latency values. */
        pci_read_config_word(pdev_i82559, PCI_COMMAND, &pci_command);
        new_command = pci_command | PCI_COMMAND_MASTER|PCI_COMMAND_IO;

        if (pci_command != new_command)
        {
            if (speedo_debug > 2)
                bsp_printf("  The PCI BIOS has not enabled this"
               " device!  Updating PCI command %4.4x->%4.4x.\n",
               pci_command, new_command);
            pci_write_config_word(pdev_i82559, PCI_COMMAND, new_command);
        }
        pci_read_config_byte(pdev_i82559, PCI_LATENCY_TIMER, &pci_latency);
        if (pci_latency < 32) {
            if (speedo_debug > 2)
                bsp_printf("PCI latency timer (CFLT) is unreasonably low at %d."
               "  Setting to 32 clocks.\n", pci_latency);
            pci_write_config_byte(pdev_i82559, PCI_LATENCY_TIMER, 32);

        } else if (speedo_debug > 2)
            bsp_printf("  PCI latency timer (CFLT) is %08X\n", pci_latency);

        speedo_found1(busno, deviceno, funcno, ioaddr, irq, cards_found);
        cards_found++;
    }

    if(speedo_debug) bsp_printf("  Total number of 82559 found %d\n",
                                  cards_found);
}    

static void i559_init(enet_addr_t enet_addr) 
{
    // call speedo_open of linux eepro100.c, it is also part of initialization
    if(cards_found)
    { 
        speedo_open();
    }
    else bsp_printf("No 82559 found.\n");

#if DEBUG_ENET >= 3
    BSPLOG(bsp_log(" 82559 initialization done.\n"));
    DumpStatus();
#endif
}


/*
 *  Send a packet out over the ethernet.  The packet is sitting at the
 *  beginning of the transmit buffer.  The routine returns when the
 *  packet has been successfully sent.
*/
static void i559_send(void *unused, const char *buf, int length)
{
  struct sk_buff *skb;
#ifndef __LITTLE_ENDIAN__
  int i=0, four_bytes_bound=0; 
  unsigned long int *uintl_in = (unsigned long int *)ch_buf_in;
  unsigned long int *uintl_out = (unsigned long int *)ch_buf_out;
#endif //#ifndef __LITTLE_ENDIAN__

#ifdef SHOW_TX_INFO
    bsp_printf("snd_len=%d, sb:\n", length);
    dump_buf(buf, length);
#endif /* #ifdef SHOW_TX_INFO */

    if (length < (ETH_MIN_PKTLEN+2))  //DC ETH_MIN_PKTLEN defined as 60 in net.h
                                      //DC since we have 2 bytes padding in
                                      //DC beginning of our eth_header_t, so
                                      //DC we need to add 2
        length = ETH_MIN_PKTLEN+2; /* and min. ethernet len */
    else if ((length-2) > PKT_BUF_SZ)
    return;
    
    memcpy(((eth_header_t *)buf)->source, __local_enet_addr,
       sizeof(__local_enet_addr));

#ifndef __LITTLE_ENDIAN__  //big endian
    if((length -2)%4) four_bytes_bound=((length -2)/4) + 1;
    else four_bytes_bound=(length -2)/4;

    memcpy(&ch_buf_in[0], &buf[2], (four_bytes_bound*4)); 
    for(i=0; i < four_bytes_bound; i++)   //do swaping
        uintl_out[i]=swap32(uintl_in[i]);
#ifdef SHOW_TX_INFO 	
    bsp_printf(" v2 - length-2=%d, four_bytes_boundi=%d\n", 
	(length-2), four_bytes_bound);
    bsp_printf(" following is ch_buf_in, total %d bytes:\n",
	four_bytes_bound*4);
    dump_buf(ch_buf_in, four_bytes_bound*4);
    bsp_printf(" following is ch_buf_out, total %d bytes:\n",
	four_bytes_bound*4);
    dump_buf(ch_buf_out, four_bytes_bound*4);
#endif //#ifdef DC_DUMP_TX_INFO 	

#endif //#ifndef __LITTLE_ENDIAN__

    skb = dev_alloc_skb(PKT_BUF_SZ);
    /* due to arm struct alignment problem, we have two extra bytes in the 
       beginning of eth_header take them away now */
    /* memcpy(skb_put(skb, length), buf, length); */
#ifdef __LITTLE_ENDIAN__
    memcpy(skb_put(skb, (length-2)), &buf[2], (length-2) );
#else // big endian
  //due to bytes swapped in ch_buf_out, we need to pass mutilple of 4buts 
  //to i82559 h/w, otherwise we will lose some valid bytes
    memcpy(skb_put(skb, (four_bytes_bound*4)), &ch_buf_out[0], 
           (four_bytes_bound*4) );
#endif //#ifdef __LITTLE_ENDIAN__
#ifdef USE_I82559_INTERRUPT        
    speedo_start_xmit(skb);
#else /* use polling */        
    polling_xmit_i82559(skb);
#endif /* #ifdef USE_I82559_INTERRUPT */        
    dev_free_skb(skb);
}


/*
 * Test for the arrival of a packet on the Ethernet interface.
 * If a packet is found, copy it into the buffer and return its length.
 * If no packet, then return 0.
*/
int i559_receive(void *unused, char *buf, int maxlen)
{
    int rtn_len=0;
    char *src_buf;
    int length_lbuf=0;
        
#ifdef USE_I82559_INTERRUPT         
    if(valid_lbuf_cnt) /* valid frames there */
    {
    /* DC in order to fix the struct align problem of ARM, we need to 
      add two extra bytes in the beginning of received frame */
       length_lbuf=len_lbuf[lbuf_get_ptr];
       rtn_len=maxlen;
       if(length_lbuf < (rtn_len -2) ) rtn_len=length_lbuf +2;
       src_buf=&lbuf[lbuf_get_ptr][0];
       buf[0]=buf[1]=0;
       memcpy(&buf[2], src_buf, (rtn_len -2) );
       maintain_lbuf_get_ptr();
    }

#else /* not use interrupt,polling i82559 */        
    polling_read_i82559();
 
    if(valid_lbuf_cnt) /*if there are valid frames there */
    /* then return what we have, ip_match will consume them */
    {
    /* DC in order to fix the struct align problem of ARM, we need to 
      add two extra bytes in the beginning of received frame 
    */
       length_lbuf=len_lbuf[lbuf_get_ptr];
       rtn_len=maxlen;
       if(length_lbuf < (rtn_len -2) ) rtn_len=length_lbuf +2;
       src_buf=&lbuf[lbuf_get_ptr][0];
       buf[0]=buf[1]=0;
       memcpy(&buf[2], src_buf, (rtn_len -2) );
       maintain_lbuf_get_ptr();
    }
    else 
       rtn_len=0;
#endif /* #ifdef USE_I82559_INTERRUPT */        

#ifdef SHOW_RX_INFO     
    if(rtn_len)
    {
        bsp_printf("rv_len=%d, rv: \n", rtn_len);
        dump_buf(buf, rtn_len);
    }
#endif  /* #ifdef SHOW_RX_INFO */ 
    return rtn_len;
}

static int i559_irq_handler(int irq_nr, void *regs)
{
    extern int gdb_interrupt_check(void);
    /* The interrupt handler does all of the Rx thread work and cleans up
       after the Tx thread. */
    u32 ioaddr=enet_dev.ioaddr;
    int boguscnt = max_interrupt_work;
    unsigned int dirty_tx=0;
    int entry = 0;
    unsigned short status = 0;

#ifndef final_version
    enet_dev.interrupt = 1;
#endif
    
    do {
        status = inw(ioaddr + SCBStatus);
        /* Acknowledge all of the current interrupt sources ASAP. */
        outw(status & 0xfc00, ioaddr + SCBStatus);

        if (speedo_debug > 2)
            bsp_printf("intst=%#4.4x.\n", status); 

        if ((status & 0xfc00) == 0)
            break;

        if (status & 0x4000)     /* Packet received. */
            speedo_rx();

#ifdef ALLOW_GDB_HOOK
        {
            if (gdb_interrupt_check())
            {
                bsp_printf("gdb_interrupt_check Got ^C.\n");
                return 0;
            }
        }
#endif // ALLOW_GDB_HOOK

        if (status & 0x1000)
        {
            if ((status & 0x003c) == 0x0028) /* No more Rx buffers. */
                outb(RX_RESUMENR, ioaddr + SCBCmd);  
            else if ((status & 0x003c) == 0x0008)
            { /* No resources (why?!) */
                /* No idea of what went wrong.  Restart the receiver. */
                outl(virt_to_bus((u32)(spdo.rx_ringp[spdo.cur_rx % RX_RING_SIZE])),
                ioaddr + SCBPointer);
                //DC outw(RX_START, ioaddr + SCBCmd);
                outb(RX_START, ioaddr + SCBCmd);  //DC new driver
            }
            spdo.stats.rx_errors++;
        }

        /* User interrupt, Command/Tx unit interrupt or CU not active. */
        if (status & 0xA400) {
            dirty_tx = spdo.dirty_tx;
            while (spdo.cur_tx - dirty_tx > 0)
            {
                entry = dirty_tx % TX_RING_SIZE;
                status = le32_to_cpu(spdo.tx_ring[entry].status);

                if (speedo_debug > 5)
                        bsp_printf("scavenge candidate %d status %4.4x.\n",
                                    entry, status);
                if ((status & StatusComplete) == 0)
                    break;            /* It still hasn't been processed. */
    
                    /* Free the original skb. */
                if (spdo.tx_skbuff[entry])
                {
                    spdo.stats.tx_packets++; /* Count only user packets. */
                        /* Count transmitted bytes */
                    spdo.stats.tx_bytes += spdo.tx_skbuff[entry]->len;
                    dev_free_skb(spdo.tx_skbuff[entry]);
                    spdo.tx_skbuff[entry] = 0;
                } else if ((status&0x70000) == CmdNOp << 16)
                    spdo.mc_setup_busy = 0;
                dirty_tx++;
            }

#ifndef final_version
            if (spdo.cur_tx - dirty_tx > TX_RING_SIZE)
            {
                if (speedo_debug > 2)
                  bsp_printf("out-of-sync dirty pointer, %d vs. %d, full=%d.\n",
                               dirty_tx, spdo.cur_tx, spdo.tx_full);
                dirty_tx += TX_RING_SIZE;
            }
#endif
            if (spdo.tx_full && enet_dev.tbusy
                && dirty_tx > spdo.cur_tx - TX_RING_SIZE + 2)
            {
                    /* The ring is no longer full, clear tbusy. */
                spdo.tx_full = 0;
                    /* DC clear_bit(0, (void*)&enet_dev.tbusy); */
                enet_dev.tbusy=0;
            }
    
            spdo.dirty_tx = dirty_tx;
        }

        if (--boguscnt < 0) {
            bsp_printf("Too much work at interrupt, status=0x%4.4x.\n",
                       status);
            /* Clear all interrupt sources. */
            outl(0xfc00, ioaddr + SCBStatus);
            break;
        }
    } while (1);

    if (speedo_debug > 3)
        bsp_printf("exit int, stat=%#4.4x.\n", inw(ioaddr + SCBStatus));

    enet_dev.interrupt = 0;

    return 1;
}

static int i559_control(void *unused, int func, ...)
{
    static bsp_vec_t eth_vec;
    int          retval = 0;
    va_list      ap;

    va_start (ap, func);

    switch (func) {
      case COMMCTL_INSTALL_DBG_ISR:
          eth_vec.handler = i559_irq_handler;
          eth_vec.next = NULL;

          bsp_install_vec(BSP_VEC_INTERRUPT, IXP1200_PCI_INT_VEC,
              BSP_VEC_REPLACE, &eth_vec);
          bsp_enable_irq(IXP1200_PCI_INT_VEC);
#ifdef USE_I82559_INTERRUPT         
          *IXP1200_PCI_IRQ_ENABLE_SET |= IXP1200_PCI_IRQ_PIL_SET;
          bsp_printf("in i82559.c DBG_SR install enet int_vec %d\n", IXP1200_PCI_INT_VEC); 
#endif /* #ifdef USE_I82559_INTERRUPT */        
          break;

      case COMMCTL_REMOVE_DBG_ISR:
          bsp_disable_irq(IXP1200_PCI_INT_VEC);
          bsp_remove_vec(BSP_VEC_INTERRUPT, IXP1200_PCI_INT_VEC, &eth_vec);
          break;

      case COMMCTL_IRQ_DISABLE:
          retval = bsp_disable_irq(IXP1200_PCI_INT_VEC);
          break;

      case COMMCTL_IRQ_ENABLE:
          bsp_enable_irq(IXP1200_PCI_INT_VEC); 
          break;

      default:
          retval = -1;
          break;
    }

    va_end(ap);
    return retval;
}

static void dummy_putc(void *unused, char ch)
{
    if(speedo_debug > 2)
    bsp_printf("putc not supported for an enet link driver\n");
}

static int dummy_getc(void *unused)
{
    if(speedo_debug > 2)
    bsp_printf("getc not supported for an enet link driver\n");
}

static struct bsp_comm_procs i559_procs = {
    NULL,
    i559_send, i559_receive,
    dummy_putc, dummy_getc,
    i559_control
};

struct bsp_comm_procs * __enet_link_init(enet_addr_t enet_addr)
{
    i559_init(enet_addr);
    return &i559_procs;
}

#if DEBUG_ENET
static void
DumpStatus(void)
{
    BSPLOG(bsp_log("scb: command[0x%x] status[0x%x]\n",
        scb->command, scb->status));
}
#endif

static void speedo_found1(int busno, int deviceno, int funcno, u32 ioaddr,
        int irq, int card_idx)
{
    u16 sum = 0;
    int option;
    int i=0, j=0;
    int addr_len = 0;
    const char *connectors[] = {" RJ45", " BNC", " AUI", " MII"};

#ifndef SKIP_SYSTEM_SELF_TEST
    /* The self-test results must be paragraph aligned. */
    static s32 str[6];
    static *volatile self_test_results;
    int boguscnt = 16000;    /* Timeout for set-test. */
#endif /* SKIP_SYSTEM_SELF_TEST */

    option = options[card_idx];
  
    /* Read the station address EEPROM before doing the reset.
    Perhaps this should even be done before accepting the device,
    then we wouldn't have a device name with which to report the error. */

    addr_len = read_eeprom(ioaddr, 0, 6) == 0xffff ? 8 : 6;

    if (speedo_debug > 2) bsp_printf("addr_len=%d \n", addr_len);

    for (j = 0, i = 0; i < 0x40; i++) {
        u16 value = read_eeprom(ioaddr, i, addr_len);
        eeprom[i] = value;
        sum += value;
        if (i < 3) {
            enet_hw_addr[j++] = (u8)(value & 0xFF);
            enet_hw_addr[j++] = (u8)((value >> 8) & 0xFF);
        }
    }
    
    for(j=0; j < 6; j++)
        __local_enet_addr[j]=enet_hw_addr[j];

    if (sum != 0xBABA){
         if(speedo_debug) bsp_printf("Card %d: Invalid EEPROM checksum %#4.4x, "
                     "check settings before activating this device!\n",
                         card_idx, sum);
    }

    /* Reset the chip: stop Tx and Rx processes and clear counters.
    This takes less than 10usec and will easily finish before the next
    action. */

    outl(PortReset, ioaddr + SCBPort);

    if(speedo_debug) {
        bsp_printf("Ethernet MAC address: ");
        for (i = 0; i < 5; i++)
            bsp_printf("%2.2X:", enet_hw_addr[i]);
        bsp_printf("%2.2X", enet_hw_addr[5]);
        bsp_printf(", IO address=%8.8x, IRQ=%X\n", ioaddr, (irq & 0xFF) );
    }

    if ( (eeprom[3] & 0x03) && (speedo_debug))
        bsp_printf( "Receiver lock-up bug exists -- enabling"
               " work-around.\n");

    if(speedo_debug)
        bsp_printf("Board assembly %4.4x%2.2x-%3.3d, Physical"
                " connectors present:",
                eeprom[8], eeprom[9]>>8, eeprom[9] & 0xff);
    
    for (i = 0; i < 4; i++)
        if (eeprom[5] & (1 << i))
        {
            if(speedo_debug)
                bsp_printf(connectors[i]);
        }

    if(speedo_debug)
       bsp_printf("\n  Primary interface chip %s PHY #%d.\n",
               phys[(eeprom[6]>>8)&15], eeprom[6] & 0x1f);

    if (eeprom[7] & 0x0700)
    {
        if(speedo_debug) bsp_printf("Secondary interface chip %s.\n",
             phys[(eeprom[7]>>8)&7]);
    }

    if (((eeprom[6]>>8) & 0x3f) == DP83840
        ||  ((eeprom[6]>>8) & 0x3f) == DP83840A)
    {
        int mdi_reg23 = mdio_read(ioaddr, eeprom[6] & 0x1f, 23) | 0x0422;
        if (congenb)
            mdi_reg23 |= 0x0100;
        bsp_printf("DP83840 specific setup, setting register 23 to %4.4x.\n",
                    mdi_reg23);
        mdio_write(ioaddr, eeprom[6] & 0x1f, 23, mdi_reg23);
    }

#ifndef SKIP_SYSTEM_SELF_TEST
    /* OK, this is pure kernel bloat.  I don't like it when other drivers
    waste non-pageable kernel space to emit similar messages, but I need
    them for bug reports. */
    if (((eeprom[6]>>8) & 0x3f) == DP83840
        ||  ((eeprom[6]>>8) & 0x3f) == DP83840A) {
        int mdi_reg23 = mdio_read(ioaddr, eeprom[6] & 0x1f, 23) | 0x0422;
        if (congenb)
          mdi_reg23 |= 0x0100;
        bsp_printf("  DP83840 specific setup, setting register 23 to %4.4x.\n",
           mdi_reg23);
        mdio_write(ioaddr, eeprom[6] & 0x1f, 23, mdi_reg23);
    }

    if ((option >= 0) && (option & 0x70)) {
        bsp_printf("  Forcing %dMbs %s-duplex operation.\n",
           (option & 0x20 ? 100 : 10),
           (option & 0x10 ? "full" : "half"));
        mdio_write(ioaddr, eeprom[6] & 0x1f, 0,
               ((option & 0x20) ? 0x2000 : 0) |     /* 100mbps? */
               ((option & 0x10) ? 0x0100 : 0)); /* Full duplex? */
    }


#ifdef DC_FORCE_TO_10_MB
        bsp_printf("DC-test  Forcing to 10 Mbs and full duplex operation.\n");
        mdio_write(ioaddr, eeprom[6] & 0x1f, 0, 0x0100);
#endif //#ifdef DC_FORCE_TO_10_MB

    /* Perform a system self-test. */
    self_test_results = (s32*) ((((long) str) + 15) & ~0xf);
    self_test_results[0] = 0;
    self_test_results[1] = -1;
    outl(virt_to_bus((u32)self_test_results) | PortSelfTest, ioaddr + SCBPort); 

    do {
        udelay(10);
    } while (self_test_results[1] == -1  &&  --boguscnt >= 0);

    if (boguscnt < 0) {     /* Test optimized out. */
        if(speedo_debug) {
            bsp_printf("Self test failed, status %8.8x:\n",
                        self_test_results[1]); 
            bsp_printf("Failure to initialize the i82557.\n");
            bsp_printf("Verify that the card is a bus-master capable slot.\n");
        }
    }
    else
    {
        /* following information show user the i82559 information */
        if(display_i82559_info_only_once)
        {
            display_i82559_info_only_once=0;
	    bsp_printf("Intel 82559 found, MAC address: ");
	    for (i = 0; i < 5; i++)
	        bsp_printf("%2.2X:", enet_hw_addr[i]);
	    bsp_printf("%2.2X", enet_hw_addr[5]);
	    bsp_printf(", IO address=%8.8x, IRQ=%X\n", ioaddr, (irq & 0xFF) );
#ifdef __LITTLE_ENDIAN__
	    bsp_printf("IXP1200 is running in little endian mode.\n");
#else //#ifdef __LITTLE_ENDIAN__
	    bsp_printf("IXP1200 is running in big endian mode.\n");
#endif //#ifdef __LITTLE_ENDIAN__
        }

        if (speedo_debug) {
            bsp_printf("General self-test: %s.\n",
               self_test_results[1] & 0x1000 ? "failed" : "passed");
            bsp_printf("Serial sub-system self-test: %s.\n",
               self_test_results[1] & 0x0020 ? "failed" : "passed");
            bsp_printf("Internal registers self-test: %s.\n",
               self_test_results[1] & 0x0008 ? "failed" : "passed");
            bsp_printf("ROM checksum self-test: %s (%#8.8x).\n",
               self_test_results[1] & 0x0004 ? "failed" : "passed",
               self_test_results[0]);
        }
    }
#endif  /* SKIP_SYSTEM_SELF_TEST */
    outl(PortReset, ioaddr + SCBPort);

    memset(&spdo, 0, sizeof(struct speedo_private));
    enet_dev.ioaddr=ioaddr;
    enet_dev.irq=irq;
    spdo.full_duplex=1;  /*DC assumption */
    spdo.default_port = 1; /*DC assumption */
    spdo.phy[0] = eeprom[6];
    spdo.phy[1] = eeprom[7];
    spdo.rx_bug = (eeprom[3] & 0x03) == 3 ? 0 : 1;

    if (spdo.rx_bug)
        bsp_printf("Receiver lock-up workaround activated.\n");

    return;
}


static int read_eeprom(u32 ioaddr, int location, int addr_len)
{
    unsigned short retval = 0;
    u32 ee_addr = ioaddr + SCBeeprom;
    int read_cmd = location | EE_READ_CMD;
    int i;
#ifdef SHOW_READ_EEPROM
    unsigned short tmp_s1=0, tmp_s2=0;
#endif /* #ifdef SHOW_READ_EEPROM */

    outw(EE_ENB & ~EE_CS, ee_addr);
#ifdef SHOW_READ_EEPROM
bsp_printf("outw EE_ENB & ~EE_CS=%x, conv=%x, addr=%x\n",
	EE_ENB & ~EE_CS, cpu_to_le16(EE_ENB & ~EE_CS), ee_addr);
#endif /* #ifdef SHOW_READ_EEPROM */

    outw(EE_ENB, ee_addr);
#ifdef SHOW_READ_EEPROM 
bsp_printf("outw EE_ENB=%x, conv=%x, addr=%x\n", EE_ENB, cpu_to_le16(EE_ENB), ee_addr);
bsp_printf("read status reg=%x, addr=%x\n",
      inw(ioaddr), ioaddr);

bsp_printf("read general status reg=%x, addr=%x\n",
      inb(ioaddr+0x1d), ioaddr+0x1d);
#endif /* #ifdef SHOW_READ_EEPROM */

    /* Shift the read command bits out. */
    for (i = 12; i >= 0; i--) {
    	short dataval = (read_cmd & (1 << i)) ? EE_DATA_WRITE : 0;
    	outw(EE_ENB | dataval, ee_addr);
    	eeprom_delay(100);
    	outw(EE_ENB | dataval | EE_SHIFT_CLK, ee_addr);
    	eeprom_delay(150);
    }

    outw(EE_ENB, ee_addr);
    for (i = 15; i >= 0; i--) {
    	outw(EE_ENB | EE_SHIFT_CLK, ee_addr);
    	eeprom_delay(100);
#ifdef SHOW_READ_EEPROM
    	tmp_s1=inw(ee_addr);
    	tmp_s2= ((tmp_s1 & EE_DATA_READ) ? 1 : 0);
    	retval = (retval << 1) | tmp_s2;
    	bsp_printf("i=%d: s1=%4.4X, s2=%4.4X, rv=%4.4X \n",
		i, tmp_s1, tmp_s2, retval);
#else /* #ifdef SHOW_READ_EEPROM */
    	retval = (retval << 1) | ((inw(ee_addr) & EE_DATA_READ) ? 1 : 0);
#endif /* #ifdef SHOW_READ_EEPROM */
    	outw(EE_ENB, ee_addr);
    	eeprom_delay(100);
    }


    /* Terminate the EEPROM access. */
    outw(EE_ENB & ~EE_CS, ee_addr);
#ifdef SHOW_READ_EEPROM
    bsp_printf("EEPROM=%4.4X \n",  retval);
#endif /* #ifdef SHOW_READ_EEPROM */
    return retval;
}


static int mdio_read(int ioaddr, int phy_id, int location)
{
    int val, boguscnt = 64*10;        /* <64 usec. to complete, typ 27 ticks */
    outl(0x08000000 | (location<<16) | (phy_id<<21), ioaddr + SCBCtrlMDI);
    do {
        val = inl(ioaddr + SCBCtrlMDI);
    if (--boguscnt < 0) {
            if(speedo_debug)
        bsp_printf("mdio_read() timed out with val = %8.8x.\n", val);
    }
    } while (! (val & 0x10000000));

    return val & 0xffff;
}


static int mdio_write(u32 ioaddr, int phy_id, int location, int value)
{
    int val, boguscnt = 64*10;    /* <64 usec. to complete, typ 27 ticks */
    outl(0x04000000 | (location<<16) | (phy_id<<21) | value, ioaddr
    + SCBCtrlMDI);
    
    do {
        val = inl(ioaddr + SCBCtrlMDI);
    if (--boguscnt < 0) {
            bsp_printf("mdio_write() timed out with val = %8.8x.\n", val);
    }

    } while (! (val & 0x10000000));

    return val & 0xffff;
}

#define DO_FOR_LOOP_DELAY

static void udelay(int delay)
{
#ifdef DO_FOR_LOOP_DELAY
// This version provides 1 microsecond delay approx per
// 1 unit of delay argument on IXP 1200, running at 166 Mhz
     int  nInd;

     for (nInd = 0; nInd < delay; nInd++)
        ;
#else
    /* DC this software delay seems work */
    int i= 5 * delay;
    int j=0, n=3;

    while(i--)
    {
    j=i*n +3;
    }
#endif
}


static u32 virt_to_bus(u32 ixp1200_SA_addr)
{
    int pci_space_addr=0xFFFFFFFF;

    /* for cygmon with mmu off, since our SDRAM starting address is 0xC0000000,
       so first check whether the incoming ixp1200_SA_address is in SDRAM range, 
       if not, give a warning, otherwise based on *IXP1200_DRAM_BAR to get 
       adress in PCI space
    */
    if( ixp1200_SA_addr >= bo_kernel_load_offset)
     bsp_printf(" address %8.8X over range\n", ixp1200_SA_addr);
     	
    if( (ixp1200_SA_addr >= IXP1200_SDRAM_SA_PHY_START) &&
    (ixp1200_SA_addr < (IXP1200_SDRAM_SA_PHY_START + IXP1200EB_ACTUAL_SDRAM_SIZE)) )
    {
     pci_space_addr= (ixp1200_SA_addr - IXP1200_SDRAM_SA_PHY_START) + 
             (*IXP1200_PCI_DRAM_BAR & 0xFFFFFFF0);  /* mask out lower 4 bits 
                           of *IXP1200_PCI_DRAM_BAR, these 4 bits are not part of BAR */
    }
    else
    {
     bsp_printf(" address %8.8X not in SDRAM\n", ixp1200_SA_addr);
    }
    return pci_space_addr;
}


static  u32 bus_to_virt(u32 pci_space_addr)
{
    u32 ixp1200_SA_addr=0xFFFFFFFF;

    /* for cygmon with mmu off, since our SDRAM starting address is 0xC0000000,  
    so after coverting 
    */
    ixp1200_SA_addr= (pci_space_addr - (*IXP1200_PCI_DRAM_BAR & 0xFFFFFFF0)) + 
    IXP1200_SDRAM_SA_PHY_START;  /* mask out lower 4 bits of *IXP1200_PCI_DRAM_BAR,
                                                    these 4 bits are not part of BAR */
   if( (ixp1200_SA_addr < IXP1200_SDRAM_SA_PHY_START) &&
            (ixp1200_SA_addr >= (IXP1200_SDRAM_SA_PHY_START + IXP1200EB_ACTUAL_SDRAM_SIZE)) )
    {
        bsp_printf(" address %8.8X not in SDRAM\n", ixp1200_SA_addr);
    }
    return ixp1200_SA_addr;
}

static void speedo_open(void)
{
    u32 ioaddr = enet_dev.ioaddr;
    u8 *setup_frm = (u8 *)&(spdo.tx_ring[0].tx_desc_addr);
    int i=0;
#ifndef __LITTLE_ENDIAN__
    char be_setup_frm[8]={0,0,0,0,0,0,0,0};
#endif //#ifndef __LITTLE_ENDIAN__

#ifdef DC_SOFT_INTERRUPT
    unsigned short sw_status=0; 
    int j=0, dummy=0;
#endif /* #ifdef DC_SOFT_INTERRUPT  */

#ifdef DC_DUMP_TX_INFO
    s32 xx_status=0;
#endif //#ifdef DC_DUMP_TX_INFO 	

    flush_i82559_rcv_buf(); /* flush local receive buffer*/

#ifdef notdef
    /* We could reset the chip, but should not need to. */
    /* In fact we MUST NOT, unless we also re-do the init */
    outl(PortReset, ioaddr + SCBPort);
    udelay(10);
#endif

#ifdef oh_no_you_dont_unless_you_honour_the_options_passed_in_to_us
    /* Retrigger negotiation to reset previous errors. */
    if ((spdo.phy[0] & 0x8000) == 0) {
        int phy_addr = spdo.phy[0] & 0x1f ;
    /* Use 0x3300 for restarting NWay, other values to force xcvr:
       0x0000 10-HD
       0x0100 10-FD
       0x2000 100-HD
       0x2100 100-FD
    */
#ifdef honor_default_port
        mdio_write(ioaddr, phy_addr, 0, mii_ctrl[spdo.default_port & 7]);
#else
    mdio_write(ioaddr, phy_addr, 0, 0x3300);
#endif
    }
#endif

    /* Load the statistics block address. */
    wait_for_cmd_done(ioaddr + SCBCmd);
    outl(virt_to_bus((u32)(&spdo.lstats)), ioaddr + SCBPointer);
    outw(INT_MASK | CU_STATSADDR, ioaddr + SCBCmd);
    spdo.lstats.done_marker = 0;

    speedo_init_rx_ring();

    wait_for_cmd_done(ioaddr + SCBCmd);
    outl(0, ioaddr + SCBPointer);
    outw(INT_MASK | RX_ADDR_LOAD, ioaddr + SCBCmd);

    /* Todo: verify that we must wait for previous command completion. */
    wait_for_cmd_done(ioaddr + SCBCmd);
    outl(virt_to_bus((u32)(spdo.rx_ringp[0])), ioaddr + SCBPointer);
    outw(INT_MASK | RX_START, ioaddr + SCBCmd);

    /* Fill the first command with our physical address. */
    /* Avoid a bug(?!) here by marking the command already completed. */
    spdo.tx_ring[0].status = cpu_to_le32(((CmdSuspend | CmdIASetup) << 16) | 0xa000);
    spdo.tx_ring[0].link = cpu_to_le32(virt_to_bus((u32)(&(spdo.tx_ring[1]))));
#ifdef __LITTLE_ENDIAN__
    for(i=0; i < 6; i++) 
        setup_frm[i]=enet_hw_addr[i];
#else //#ifdef __LITTLE_ENDIAN__
    for(i=0; i < 6; i++) 
        be_setup_frm[i]=enet_hw_addr[i];
    for(i=0; i < 2; i++) 
    {
        setup_frm[(i*4)]=be_setup_frm[(i*4) + 3];
        setup_frm[(i*4) + 1]=be_setup_frm[(i*4) + 2];
        setup_frm[(i*4) + 2]=be_setup_frm[(i*4) + 1];
        setup_frm[(i*4) + 3]=be_setup_frm[(i*4)];
    }
#endif //#ifdef __LITTLE_ENDIAN__
#ifdef DC_DUMP_TX_INFO 	
    xx_status=((CmdSuspend | CmdIASetup) << 16);
    bsp_printf(" xx_status=%8.8x\n", xx_status);
    spdo_start=&spdo;
    bsp_printf(" in speedo_open_0, following is spdo struct\n");
    dump_buf(spdo_start, sizeof(struct speedo_private));
#endif //#ifdef DC_DUMP_TX_INFO 	
#ifdef SHOW_CURTX_INFO
    bsp_printf(" in speedo_open_0, spdo.cur_tx=%d\n", spdo.cur_tx);
    spdo_start=&spdo;
    bsp_printf(" following is spdo struct\n");
    dump_buf(spdo_start, sizeof(struct speedo_private));
#endif //#ifdef SHOW_CURTX_INFO


    spdo.last_cmd = (struct descriptor *)&spdo.tx_ring[0];
    spdo.cur_tx = 1;
    spdo.dirty_tx = 0;
    spdo.tx_full = 0;

    wait_for_cmd_done(ioaddr + SCBCmd);
    outl(0, ioaddr + SCBPointer);
    outw(INT_MASK | CU_CMD_BASE, ioaddr + SCBCmd);

    enet_dev.if_port = spdo.default_port;

    enet_dev.tbusy = 0;
    enet_dev.interrupt = 0;
    enet_dev.start = 1;

    /* Start the chip's Tx process and unmask interrupts. */
    /* Todo: verify that we must wait for previous command completion. */
    wait_for_cmd_done(ioaddr + SCBCmd);
    outl(virt_to_bus((u32)(&spdo.tx_ring[0])), ioaddr + SCBPointer);  
    outw(CU_START, ioaddr + SCBCmd);

    /* Setup the chip and configure the multicast list. */
    spdo.mc_setup_frm = NULL;
    spdo.mc_setup_frm_len = 0;
    spdo.mc_setup_busy = 0;
    spdo.rx_mode = -1;           /* Invalid -> always reset the mode. */
    set_rx_mode();
#ifdef DUMP_RX_MODE_INFO
    wait_for_cmd_done(ioaddr + SCBCmd);
    bsp_printf("Af set_rx_mode() in speedo_open(), status=%4.4x.\n", inw(ioaddr + SCBStatus));
#endif	//#ifdef DUMP_RX_MODE_INFO

#ifdef SHOW_CURTX_INFO
    bsp_printf("Af set_rx_mode() in speedo_open(), spdo.cur_tx=%d\n", spdo.cur_tx);
    spdo_start=&spdo;
    bsp_printf(" following is spdo struct\n");
    dump_buf(spdo_start, sizeof(struct speedo_private));
#endif //#ifdef SHOW_CURTX_INFO

    if (speedo_debug > 2) {
       bsp_printf("Done speedo_open(), status %8.8x.\n", inw(ioaddr + SCBStatus));
    }
    //DC following CU_DUMPSTATS command seems corrupt spdo.cur_tx in big endian
    // mode, and seem not necessary, so comment it out    
    //   wait_for_cmd_done(ioaddr + SCBCmd);
    //DC     outb(CU_DUMPSTATS, ioaddr + SCBCmd); 

#ifdef SHOW_CURTX_INFO
    bsp_printf("Af CU_DUMPSTATS in speedo_open(), spdo.cur_tx=%d\n", spdo.cur_tx);
    spdo_start=&spdo;
    bsp_printf(" following is spdo struct\n");
    dump_buf(spdo_start, sizeof(struct speedo_private));
#endif //#ifdef SHOW_CURTX_INFO
    
#ifdef DC_DUMP_TX_INFO 	
    spdo_start=&spdo;
    bsp_printf(" in speedo_open_1, following is spdo struct\n");
    dump_buf(spdo_start, sizeof(struct speedo_private));
#endif //#ifdef DC_DUMP_TX_INFO 	
    /*
     * Request the IRQ last, after we have set up all data structures.
     * It would be bad to get an interrupt before we're ready.
    */

    /* No need to wait for the command unit to accept here. */
    if ((spdo.phy[0] & 0x8000) == 0)
        mdio_read(ioaddr, spdo.phy[0] & 0x1f, 0);

    /* Set the timer.  The timer serves a dual purpose:
       1) to monitor the media interface (e.g. link beat) and perhaps switch
       to an alternate media type
       2) to monitor Rx activity, and restart the Rx process if the receiver
        hangs. */
    /* DC not care timer for polling mode now
    init_timer(&spdo.timer);
    spdo.timer.expires = RUN_AT((24*HZ)/10); */            /* 2.4 sec. */
    /*DC not care timer for polling mode now
    spdo.timer.data = (unsigned long)dev;
    spdo.timer.function = &speedo_timer;  */               /* timer handler */
    /*DC not care timer for polling mode now add_timer(&spdo.timer);  */
   
#ifdef DC_SOFT_INTERRUPT 
    for(i=0; i < 5; i++)
    {
       bsp_printf("befoe set SW_INT, status=%4.4X, cmd=%4.4X\n",
           inw(ioaddr + SCBStatus), inw(ioaddr + SCBCmd) );
       outw(0x0200, ioaddr +SCBCmd);   //req. SW int
       bsp_printf("imm af set SW_INT, status=%4.4X, cmd=%4.4X\n", 
           inw(ioaddr + SCBStatus), inw(ioaddr + SCBCmd) );
       for(j=0; j < 0xFFFF; j++)
       {
          dummy=i*j +67;
       }
       sw_status=inw(ioaddr + SCBStatus);
       bsp_printf("a while af set SW_INT, status=%4.4X, cmd=%4.4X\n", 
           inw(ioaddr + SCBStatus), inw(ioaddr + SCBCmd) );
       outw((sw_status & 0xfc00), ioaddr +SCBStatus); 
       bsp_printf("imm af reset SW_INT, status=%4.4X, cmd=%4.4X\n", 
           inw(ioaddr + SCBStatus), inw(ioaddr + SCBCmd) );
       for(j=0; j < 0xFFFF; j++)
       {
          dummy=i*j +67;
       }
       bsp_printf("a while af reset SW_INT, status=%4.4X, cmd=%4.4X\n", 
           inw(ioaddr + SCBStatus), inw(ioaddr + SCBCmd) );
    }
#endif /* #ifdef DC_SOFT_INTERRUPT  */
#ifndef __LITTLE_ENDIAN__
//    bsp_printf("Exit speedo_open(), SA_Control = %08X\n",
//               *IXP1200_PCI_SA_CONTROL);
#endif /* #ifdef OUTPUT_SA_CONTROL_DEBUG_INFO */

#ifdef SHOW_CURTX_INFO
    bsp_printf("Exit speedo_open(), spdo.cur_tx=%d\n", spdo.cur_tx);
    spdo_start=&spdo;
    bsp_printf(" following is spdo struct\n");
    dump_buf(spdo_start, sizeof(struct speedo_private));
#endif //#ifdef SHOW_CURTX_INFO

#ifdef QUICK_FIX_TEST_CURTX
//    spdo.cur_tx=2; //force 
#endif //#ifdef QUICK_FIX_TEST_CURTX

    return;
}


/* Initialize the Rx and Tx rings, along with various 'dev' bits. */
static void speedo_init_rx_ring(void)
{
    struct RxFD *rxf, *last_rxf = NULL;
    int i;
    struct sk_buff *skb;

    spdo.cur_rx = 0;
    for (i = 0; i < RX_RING_SIZE; i++) {
    skb = dev_alloc_skb(PKT_BUF_SZ + sizeof(struct RxFD));
    spdo.rx_skbuff[i] = skb;
    if (skb == NULL)
        break;        /* OK.  Just initially short of Rx bufs. */

    rxf = (struct RxFD *)skb->tail;
        spdo.rx_ringp[i] = rxf;
    skb_reserve(skb, sizeof(struct RxFD));
    if (last_rxf)
        last_rxf->link = cpu_to_le32(virt_to_bus((u32)rxf));
    last_rxf = rxf;
    rxf->status = cpu_to_le32(0x00000001);         /* '1' is flag value only. */
    rxf->link = 0;                    /* None yet. */
    /* This field unused by i82557, we use it as a consistency check. */
#ifdef final_version
    rxf->rx_buf_addr = 0xffffffff;
#else
    rxf->rx_buf_addr = cpu_to_le32(virt_to_bus((u32)(skb->tail)));
#endif
    rxf->count = 0;
    rxf->size = cpu_to_le16(PKT_BUF_SZ);
    }
    spdo.dirty_rx = (unsigned int)(i - RX_RING_SIZE);
    /* Mark the last entry as end-of-list. */
    last_rxf->status = cpu_to_le32(0xC0000002);        /* '2' is flag value only. */
    spdo.last_rxf = last_rxf;
}


static void speedo_tx_timeout(void)
{
    long ioaddr = enet_dev.ioaddr;

    bsp_printf("Transmit timed out: status %4.4x %4.4x at %d/%d command %8.8x.\n",
    inw(ioaddr + SCBStatus), inw(ioaddr + SCBCmd),
    spdo.dirty_tx, spdo.cur_tx,
    le32_to_cpu(spdo.tx_ring[spdo.dirty_tx % TX_RING_SIZE].status));
    if ((inw(ioaddr + SCBStatus) & 0x00C0) != 0x0080) {
    if(speedo_debug)
            bsp_printf("Trying to restart the transmitter...\n");
    outl(virt_to_bus((u32)(&spdo.tx_ring[spdo.dirty_tx % TX_RING_SIZE])),
         ioaddr + SCBPointer);
    outw(CU_START, ioaddr + SCBCmd);
    } else {
    outw(DRVR_INT, ioaddr + SCBCmd);
    }
#ifdef oh_no_you_dont_unless_you_honour_the_options_passed_in_to_us
    /* Reset the MII transceiver, suggested by Fred Young @ scalable.com. */
    if ((spdo.phy[0] & 0x8000) == 0) {
        int phy_addr = spdo.phy[0] & 0x1f;
        mdio_write(ioaddr, phy_addr, 0, 0x0400);
        mdio_write(ioaddr, phy_addr, 1, 0x0000);
        mdio_write(ioaddr, phy_addr, 4, 0x0000);
        mdio_write(ioaddr, phy_addr, 0, 0x8000);
#ifdef honor_default_port
       mdio_write(ioaddr, phy_addr, 0, mii_ctrl[enet_dev.default_port & 7]);
#endif
    }
#endif
    spdo.stats.tx_errors++;
    return;
}

#ifdef USE_I82559_INTERRUPT         
static int speedo_start_xmit(struct sk_buff *skb)
{
    u32 ioaddr = enet_dev.ioaddr;
    int entry;
    unsigned int  tmp_tbusy=0;
    int tickssofar=0;
#ifdef SHOW_TX_INFO
    int tx_length=0; 
    int ixn=0; 
#endif /* #ifdef SHOW_TX_INFO */

    /* Block a timer-based transmit from overlapping.  This could better be
       done with atomic_swap(1, enet_dev.tbusy), but set_bit() works as well.
       If this ever occurs the queue layer is doing something evil! */
    
    tmp_tbusy=enet_dev.tbusy;
    enet_dev.tbusy=1;
    if (tmp_tbusy != 0) {
        /* DC not care timer for polling mode now */
        /* tickssofar = jiffies - enet_dev.trans_start; */
        bsp_printf("try to Tx, but enet_dev.busy=1\n");
    
        if (tickssofar < TX_TIMEOUT_IN_MS - 2)
            return 1;
        if (tickssofar < TX_TIMEOUT_IN_MS) {
            /* Reap sent packets from the full Tx queue. */
            outw(DRVR_INT, ioaddr + SCBCmd);
            return 1;
        }
        speedo_tx_timeout();
        return 1;
    }

    /* Caution: the write order is important here, set the base address
    with the "ownership" bits last. */

    /* Prevent interrupts from changing the Tx ring from underneath us. */
    /* Calculate the Tx descriptor entry. */
        entry = spdo.cur_tx++ % TX_RING_SIZE;

    spdo.tx_skbuff[entry] = skb;
    /* Todo: be a little more clever about setting the interrupt bit. */
    spdo.tx_ring[entry].status =
    cpu_to_le32((CmdSuspend | CmdTx | CmdTxFlex) << 16);
    spdo.tx_ring[entry].link =
    cpu_to_le32(virt_to_bus((u32)(&spdo.tx_ring[spdo.cur_tx % TX_RING_SIZE])));
    spdo.tx_ring[entry].tx_desc_addr =
    cpu_to_le32(virt_to_bus((u32)(&spdo.tx_ring[entry].tx_buf_addr0)));
    /* The data region is always in one buffer descriptor, Tx FIFO
       threshold of 256. */
    spdo.tx_ring[entry].count = cpu_to_le32(0x01208000);
    spdo.tx_ring[entry].tx_buf_addr0 = cpu_to_le32(virt_to_bus((u32)skb->data));
    spdo.tx_ring[entry].tx_buf_size0 = cpu_to_le32(skb->len);
    /* Todo: perhaps leave the interrupt bit set if the Tx queue is more
     than half full.  Argument against: we should be receiving packets
     and scavenging the queue.  Argument for: if so, it shouldn't
     matter.
    */ spdo.last_cmd->command &= cpu_to_le16(~(CmdSuspend | CmdIntr));
    spdo.last_cmd = (struct descriptor *)&spdo.tx_ring[entry];

    /* Trigger the command unit resume. */
    wait_for_cmd_done(ioaddr + SCBCmd);
    outb(CU_RESUME, ioaddr + SCBCmd);
#ifdef SHOW_TX_INFO
    tx_length=spdo.tx_ring[entry].tx_buf_size0;   
    bsp_printf("txlen=%d,tb:\n", tx_length);
    dump_buf(skb->data, tx_length);
#endif /* #ifdef SHOW_TX_INFO */

    /* Leave room for set_rx_mode() to fill two entries. */
    if (spdo.cur_tx - spdo.dirty_tx > TX_RING_SIZE - 3)
        spdo.tx_full = 1;
    else
        /* clear_bit(0, (void*)&enet_dev.tbusy); */
        enet_dev.tbusy=0;

    return 0;
}
#endif /* #ifdef USE_I82559_INTERRUPT */        

/* Set or clear the multicast filter for this adaptor.
   This is very ugly with Intel chips -- we usually have to execute an
   entire configuration command, plus process a multicast command.
   This is complicated.  We must put a large configuration command and
   an arbitrarily-sized multicast command in the transmit list.
   To minimize the disruption -- the previous command might have already
   loaded the link -- we convert the current command block, normally a Tx
   command, into a no-op and link it to the new command.
*/
static void set_rx_mode(void)
{
    u32 ioaddr = enet_dev.ioaddr;
    struct descriptor *last_cmd;
    char new_rx_mode;
    int entry;
    u8 *config_cmd_data;
#ifndef __LITTLE_ENDIAN__
    u8 *be_config_cmd_data;
    int i=0;
    u8 tmp_cfg_cmd_data[24];
#endif //#ifndef __LITTLE_ENDIAN__

#ifdef DC_DUMP_TX_INFO
    bsp_printf(" enter set_rx_mode spdo.cur_tx=%d\n", spdo.cur_tx);
#endif //#ifdef DC_DUMP_TX_INFO

    // set rx_mode=0 will increase the ethernet speed 
    new_rx_mode = 0;	//unicast mode   

#ifdef DUMP_RX_MODE_INFO
    bsp_printf(" in set_rx_mode req. new_rx_mode=%d, org_rx_mode=%d\n",
        new_rx_mode,  spdo.rx_mode);
#endif //#ifdef DUMP_RX_MODE_INFO

    if (spdo.cur_tx - spdo.dirty_tx >= TX_RING_SIZE - 1) {
    /* The Tx ring is full -- don't add anything!  Presumably the new mode
        is in config_cmd_data and will be added anyway. 
        */
        bsp_printf("*** set_rx_mode abnormal exit - 1\n");
        spdo.rx_mode = -1;
    return;
    }

    if (new_rx_mode != spdo.rx_mode) {
        entry = spdo.cur_tx++ % TX_RING_SIZE;
        last_cmd = spdo.last_cmd;
        spdo.last_cmd = (struct descriptor *)&spdo.tx_ring[entry];

        spdo.tx_skbuff[entry] = 0;           /* Redundant. */
        spdo.tx_ring[entry].status = cpu_to_le32((CmdSuspend | CmdConfigure) << 16);
        spdo.tx_ring[entry].link =
        cpu_to_le32(virt_to_bus((u32)(&spdo.tx_ring[(entry + 1) % TX_RING_SIZE])));
#ifdef __LITTLE_ENDIAN__
        config_cmd_data = (void *)&spdo.tx_ring[entry].tx_desc_addr;
#else //#ifdef __LITTLE_ENDIAN__
        config_cmd_data = &tmp_cfg_cmd_data[0];
        be_config_cmd_data = (void *)&spdo.tx_ring[entry].tx_desc_addr;
#endif #ifdef __LITTLE_ENDIAN__
        /* Construct a full CmdConfig frame. */
        memcpy(config_cmd_data, i82558_config_cmd, sizeof(i82558_config_cmd));
        config_cmd_data[1] = (txfifo << 4) | rxfifo;
        config_cmd_data[4] = rxdmacount;
        config_cmd_data[5] = txdmacount + 0x80;
        config_cmd_data[15] |= (new_rx_mode & 2) ? 1 : 0;
        config_cmd_data[19] |= spdo.full_duplex ? 0x40 : 0;
        config_cmd_data[21] = (new_rx_mode & 1) ? 0x0D : 0x05;
        if (spdo.phy[0] & 0x8000) {          /* Use the AUI port instead. */
            config_cmd_data[15] |= 0x80;
            config_cmd_data[8] = 0;
        }
#ifndef __LITTLE_ENDIAN__  
//big endian, need to swap within 4 bytes bound, before put into tx_desc_addr
    for(i=0; i < 6; i++)
    {
 	be_config_cmd_data[(i*4)]=config_cmd_data[(i*4) + 3];
 	be_config_cmd_data[(i*4) + 1]=config_cmd_data[(i*4) + 2];
 	be_config_cmd_data[(i*4) + 2]=config_cmd_data[(i*4) + 1];
 	be_config_cmd_data[(i*4) + 3]=config_cmd_data[(i*4)];
    }
#endif //#ifndef __LITTLE_ENDIAN__
        /* Trigger the command unit resume. */
        /* no endia now last_cmd->command &= cpu_to_le16(~CmdSuspend); */
        /* last_cmd->command &= ~CmdSuspend; */

        wait_for_cmd_done(ioaddr + SCBCmd);
        clear_suspend(last_cmd);
        outb(CU_RESUME, ioaddr + SCBCmd);

    }

    spdo.rx_mode = new_rx_mode;
#ifdef DUMP_RX_MODE_INFO
    bsp_printf(" exit set_rx_mode req. new_rx_mode=%d, org_rx_mode=%d\n",
        new_rx_mode,  spdo.rx_mode);
#endif //#ifdef DUMP_RX_MODE_INFO

#ifdef DC_DUMP_TX_INFO
    bsp_printf(" exit set_rx_mode spdo.cur_tx=%d\n", spdo.cur_tx);
#endif //#ifdef DC_DUMP_TX_INFO
}

static int speedo_rx(void)
{
    int rtn_len=0;
    char *dest_buf;
    int entry = spdo.cur_rx % RX_RING_SIZE;
    int status;
    int rx_work_limit = spdo.dirty_rx + RX_RING_SIZE - spdo.cur_rx;
    int pkt_len=0;

    struct sk_buff *skb;
    void *temp;

    /* If we own the next entry, it's a new packet. Send it up. */
    while (spdo.rx_ringp[entry] != NULL &&
    (status = le32_to_cpu(spdo.rx_ringp[entry]->status)) & RxComplete) {

    if (--rx_work_limit < 0)
        break;
#ifdef DEBUG_SPEEDO_RX
        bsp_printf("speedo_rx() status %8.8x len %d.\n", status,
            le16_to_cpu(spdo.rx_ringp[entry]->count) & 0x3fff);
#endif
    if ((status & (RxErrTooBig|RxOK)) != RxOK) {
        if (status & RxErrTooBig)
                bsp_printf("Ethernet frame overran the Rx buffer, "
                       "status %8.8x!\n",  status);
        else if ( ! (status & 0x2000)) {
        /* There was a fatal error.  This *should* be impossible. */
                spdo.stats.rx_errors++;
                bsp_printf("Anomalous event in speedo_rx(), "
                       "status %8.8x.\n", status);
        }
    } else {
            pkt_len = le16_to_cpu(spdo.rx_ringp[entry]->count) & 0x3fff;
            rtn_len=pkt_len;
            /* if (rtn_len > maxlen) rtn_len = maxlen; */
        /* Check if the packet is long enough to just accept without
           copying to a properly sized skbuff. */
        if (pkt_len < rx_copybreak
        && (skb = dev_alloc_skb(pkt_len + 2)) != 0) {
        skb_reserve(skb, 2);    /* Align IP on 16 byte boundaries */
        /* Packet is in one chunk -- we can copy + cksum. */
                memcpy(skb_put(skb, pkt_len),
                    (char *)(bus_to_virt(le32_to_cpu(spdo.rx_ringp[entry]->rx_buf_addr))), 
                    pkt_len);
        /* DC folowing for copy pkt to buf */
                if(valid_lbuf_cnt < NUMB_LBUF) /* still have room*/
                {
                    if(rtn_len > LENGTH_LBUF) rtn_len=LENGTH_LBUF;
                    dest_buf=&lbuf[lbuf_put_ptr][0];
                    memcpy(dest_buf, (char *)(bus_to_virt(le32_to_cpu(spdo.rx_ringp[entry]->rx_buf_addr))), rtn_len);
                    len_lbuf[lbuf_put_ptr]=rtn_len;
                    maintain_lbuf_put_ptr();
#ifdef SHOW_RX_INFO 
                    bsp_printf("rb1: \n");
                    bsp_printf("\n"); 
                    dump_buf(dest_buf, rtn_len);
#endif  /* #ifdef SHOW_RX_INFO */
        }
                else
                {
                    rtn_len=0;
                    bsp_printf("LBUF runout!!!!!!");    
                }
        } else {
        /* Pass up the already-filled skbuff. */
                skb = spdo.rx_skbuff[entry];
        if (skb == NULL) {
                    bsp_printf("Inconsistent Rx descriptor chain.\n");
            break;
        }
                spdo.rx_skbuff[entry] = NULL;
        temp = skb_put(skb, pkt_len);
                /* DC folowing for copy pkt to buf */
                if(valid_lbuf_cnt < NUMB_LBUF) /* still have room*/
                {
                    if(rtn_len > LENGTH_LBUF) rtn_len=LENGTH_LBUF;
                    dest_buf=&lbuf[lbuf_put_ptr][0];
                    memcpy(dest_buf, (char *)(bus_to_virt(le32_to_cpu(spdo.rx_ringp[entry]->rx_buf_addr))), rtn_len);
                    len_lbuf[lbuf_put_ptr]=rtn_len;
                    maintain_lbuf_put_ptr();
#ifdef SHOW_RX_INFO 
                    bsp_printf("rb3:\n ");
                    dump_buf(dest_buf, rtn_len);
#endif  /* #ifdef SHOW_RX_INFO */ 
                }
                else
                {
                    rtn_len=0;
                    bsp_printf("LBUF runout!!!!!!");    
                }

                spdo.rx_ringp[entry] = NULL;
        }
            spdo.stats.rx_packets++;
            spdo.stats.rx_bytes += pkt_len; /* Count received bytes */
    }
        entry = (++spdo.cur_rx) % RX_RING_SIZE;
    }

    /* Refill the Rx ring buffers. */
    for (; spdo.dirty_rx < spdo.cur_rx; spdo.dirty_rx++) {
        struct RxFD *rxf;
        entry = spdo.dirty_rx % RX_RING_SIZE;
        if (spdo.rx_skbuff[entry] == NULL) {
        struct sk_buff *skb;
        /* Get a fresh skbuff to replace the consumed one. */
        skb = dev_alloc_skb(PKT_BUF_SZ + sizeof(struct RxFD));
            spdo.rx_skbuff[entry] = skb;
        if (skb == NULL) {
                spdo.rx_ringp[entry] = NULL;
        break;            /* Better luck next time!  */
        }
            rxf = spdo.rx_ringp[entry] = (struct RxFD *)skb->tail;
            skb_reserve(skb, sizeof(struct RxFD));
            rxf->rx_buf_addr = cpu_to_le32(virt_to_bus((u32)skb->tail));
    } else {
            rxf = spdo.rx_ringp[entry];
    }
    rxf->status = cpu_to_le32(0xC0000001);     /* '1' for driver use only. */
    rxf->link = 0;            /* None yet. */
    rxf->count = 0;
    rxf->size = cpu_to_le16(PKT_BUF_SZ);
        spdo.last_rxf->link = cpu_to_le32(virt_to_bus((u32)rxf));
        spdo.last_rxf->status &= cpu_to_le32(~0xC0000000);
        spdo.last_rxf = rxf;
    }

    return rtn_len;
}

void flush_i82559_rcv_buf(void)
{
    int i=0;
    for(i=0; i < NUMB_LBUF; i++) len_lbuf[i]=0;
    lbuf_get_ptr=0;  /* reset */
    lbuf_put_ptr=0;  /* reset */
    valid_lbuf_cnt=0;  /* reset */
}

static void maintain_lbuf_put_ptr(void)
{
    lbuf_put_ptr++;
    if(lbuf_put_ptr == NUMB_LBUF) lbuf_put_ptr=0; /* circular buffer */
    valid_lbuf_cnt++;
}

static void maintain_lbuf_get_ptr(void)
{
    lbuf_get_ptr++;
    if(lbuf_get_ptr == NUMB_LBUF) lbuf_get_ptr=0; /* circular buf */
    valid_lbuf_cnt--;
}

#ifndef USE_I82559_INTERRUPT        
static int  polling_read_i82559(void)      
{
    u32 ioaddr=enet_dev.ioaddr;
    int rtn_len=0;
    int entry = spdo.cur_rx % RX_RING_SIZE;
    int status;
    /* int rx_work_limit = spdo.dirty_rx + RX_RING_SIZE - spdo.cur_rx; */
    int pkt_len=0;
    unsigned short int stat_word=0;
    struct sk_buff *skb;
    void *temp;
#ifndef __LITTLE_ENDIAN__
    int i=0, four_bytes_bound=0;
    unsigned long int *uintl_buf = (unsigned long int *)pr_buf;
    unsigned long int *uintl_be_buf = (unsigned long int *)pr_be_buf;
#endif //#ifndef __LITTLE_ENDIAN__
    char *dest_buf;

    stat_word = inw(ioaddr + SCBStatus);
    /* Acknowledge all of the current interrupt sources ASAP. */
    outw(stat_word & 0xfc00, ioaddr + SCBStatus);
#ifdef SHOW_POLLING_READ_INFO
    bsp_printf(" entering polling_read_i82559, stat=%4.4x, entry=%d, status=%8.8x\n", 
       stat_word, entry, le32_to_cpu(spdo.rx_ringp[entry]->status)); 
#endif /* #ifdef SHOW_POLLING_READ_INFO */

    if(stat_word & 0x4000) {   /* packet received */

        /* If we own the next entry, it's a new packet. Send it up. */
        while (spdo.rx_ringp[entry] != NULL &&
	    (status = le32_to_cpu(spdo.rx_ringp[entry]->status)) & 
             RxComplete) { 
#ifdef SHOW_POLLING_READ_INFO 
            bsp_printf(" in polling_read_i82559: status=%8.8X\n", status); 
#endif /* #ifdef SHOW_POLLING_READ_INFO */
	    if (speedo_debug > 4)
                bsp_printf("speedo_rx() status %8.8x len %d.\n", status,
                       le16_to_cpu(spdo.rx_ringp[entry]->count) & 0x3fff);
	    if ((status & (RxErrTooBig|RxOK)) != RxOK) {
	        if (status & RxErrTooBig)
                    bsp_printf("Ethernet frame overran the Rx buffer, "
                           "status %8.8x!\n",  status);
                else if ( ! (status & 0x2000)) {
		    /* There was a fatal error.  This *should* be impossible. */
                    spdo.stats.rx_errors++;
                    bsp_printf("Anomalous event in speedo_rx(), "
                        "status %8.8x.\n", status);
	        }
	    } else {
                pkt_len = le16_to_cpu(spdo.rx_ringp[entry]->count) & 0x3fff;
                rtn_len=pkt_len;
                /* if (rtn_len > maxlen) rtn_len = maxlen; */
	        /* Check if the packet is long enough to just accept without
		     copying to a properly sized skbuff. */
	        /* no skb needed if (pkt_len < rx_copybreak
		    && (skb = dev_alloc_skb(pkt_len + 2)) != 0) { */
	        if (pkt_len < rx_copybreak) {
		    /* Packet is in one chunk -- we can copy + cksum. */
                    /* not necessary copy to skb 
		    memcpy(skb_put(skb, pkt_len),
		         (char *)(bus_to_virt(le32_to_cpu(spdo.rx_ringp[entry]->rx_buf_addr))), 
                         pkt_len); */
		    /* DC folowing for copy pkt to pr_buf */
#ifdef __LITTLE_ENDIAN__
                    memcpy(pr_buf, (char *)(bus_to_virt(le32_to_cpu(spdo.rx_ringp[entry]->rx_buf_addr))), rtn_len);
#ifdef DUMP_RX_MODE_INFO
                    bsp_printf("buf: len=%d\n", rtn_len);
                    dump_buf(pr_buf, rtn_len);
#endif //#ifdef DUMP_RX_MODE_INFO
                    /* only take frames destinated for this MAC */
                    if( (pr_buf[0] == enet_hw_addr[0]) &&
                        (pr_buf[1] == enet_hw_addr[1]) &&
                        (pr_buf[2] == enet_hw_addr[2]) &&
                        (pr_buf[3] == enet_hw_addr[3]) &&
                        (pr_buf[4] == enet_hw_addr[4]) &&
                        (pr_buf[5] == enet_hw_addr[5]) )
                    {  
#else //#ifdef __LITTLE_ENDIAN__
                    //DC in big endian mode, the rx buf needed to be swapped
                    //before passed to upper protocol, and we need get more
                    //than rtn_len bytes to complete the swap, if rtn_len 
                    //is not multiple of four
                    if(rtn_len%4) four_bytes_bound=(rtn_len/4) + 1;
                    else four_bytes_bound=rtn_len/4;

                    memcpy(pr_be_buf, (char *)(bus_to_virt(le32_to_cpu(spdo.rx_ringp[entry]->rx_buf_addr))), (four_bytes_bound * 4));
#ifdef DUMP_RX_MODE_INFO
                    bsp_printf("be_buf: len=%d, be_copy_len=%d\n", 
                        rtn_len, (four_bytes_bound*4));
                    dump_buf(pr_be_buf, (four_bytes_bound*4));
#endif //#ifdef DUMP_RX_MODE_INFO
                    /* only take frames destinated for this MAC */
                    if( (pr_be_buf[3] == enet_hw_addr[0]) &&
                        (pr_be_buf[2] == enet_hw_addr[1]) &&
                        (pr_be_buf[1] == enet_hw_addr[2]) &&
                        (pr_be_buf[0] == enet_hw_addr[3]) &&
                        (pr_be_buf[7] == enet_hw_addr[4]) &&
                        (pr_be_buf[6] == enet_hw_addr[5]) )
                    {  

                        for(i=0; i < four_bytes_bound; i++)   //do swaping
                            uintl_buf[i]=swap32(uintl_be_buf[i]);
#endif //#ifdef __LITTLE_ENDIAN__
                        if(valid_lbuf_cnt < NUMB_LBUF) /* still have room*/
                        {
                            if(rtn_len > LENGTH_LBUF) rtn_len=LENGTH_LBUF;
                            dest_buf=&lbuf[lbuf_put_ptr][0];
                            memcpy(dest_buf, pr_buf, rtn_len);
                            len_lbuf[lbuf_put_ptr]=rtn_len;
                            maintain_lbuf_put_ptr();
#ifdef SHOW_RX_INFO 
                            bsp_printf("rb1: \n");
                            dump_buf(dest_buf, rtn_len);
#endif  /* #ifdef SHOW_RX_INFO */
	                }
                        else
                        {
                            flush_i82559_rcv_buf(); /* flush stale info */
                            rtn_len=0;
                            bsp_printf("LBUF runout! ");	
                        }
                    }
	       } else {
		    /* Pass up the already-filled skbuff. */
                    skb = spdo.rx_skbuff[entry];
		    /* if (skb == NULL) {
                         bsp_printf("Inconsistent Rx descriptor chain.\n");
		         break;
	            }
                    */
                    spdo.rx_skbuff[entry] = NULL;
		    temp = skb_put(skb, pkt_len);
		    /* DC folowing for copy pkt to buf */
#ifdef __LITTLE_ENDIAN__
                    memcpy(pr_buf, (char *)(bus_to_virt(le32_to_cpu(spdo.rx_ringp[entry]->rx_buf_addr))), rtn_len);
#ifdef DUMP_RX_MODE_INFO
                    bsp_printf("buf: len=%d\n", rtn_len);
                    dump_buf(pr_buf, rtn_len);
#endif //#ifdef DUMP_RX_MODE_INFO
                    /* only take frames destinated for this MAC */
                    if( (pr_buf[0] == enet_hw_addr[0]) &&
                        (pr_buf[1] == enet_hw_addr[1]) &&
                        (pr_buf[2] == enet_hw_addr[2]) &&
                        (pr_buf[3] == enet_hw_addr[3]) &&
                        (pr_buf[4] == enet_hw_addr[4]) &&
                        (pr_buf[5] == enet_hw_addr[5]) )
                    {  
#else //#ifdef __LITTLE_ENDIAN__
                    //DC in big endian mode, the rx buf needed to be swapped
                    //before passed to upper protocol, and we need get more
                    //than rtn_len bytes to complete the swap, if rtn_len 
                    //is not multiple of four
                    if(rtn_len%4) four_bytes_bound=(rtn_len/4) + 1;
                    else four_bytes_bound=rtn_len/4;

                    memcpy(pr_be_buf, (char *)(bus_to_virt(le32_to_cpu(spdo.rx_ringp[entry]->rx_buf_addr))), (four_bytes_bound * 4));
#ifdef DUMP_RX_MODE_INFO
                    bsp_printf("be_buf: len=%d, be_copy_len=%d\n", 
                        rtn_len, (four_bytes_bound*4));
                    dump_buf(pr_be_buf, (four_bytes_bound*4));
#endif //#ifdef DUMP_RX_MODE_INFO
                    /* only take frames destinated for this MAC */
                    if( (pr_be_buf[3] == enet_hw_addr[0]) &&
                        (pr_be_buf[2] == enet_hw_addr[1]) &&
                        (pr_be_buf[1] == enet_hw_addr[2]) &&
                        (pr_be_buf[0] == enet_hw_addr[3]) &&
                        (pr_be_buf[7] == enet_hw_addr[4]) &&
                        (pr_be_buf[6] == enet_hw_addr[5]) )
                    {  
                        for(i=0; i < four_bytes_bound; i++)   //do swaping
                            uintl_buf[i]=swap32(uintl_be_buf[i]);
#endif //#ifdef __LITTLE_ENDIAN__
                        if(valid_lbuf_cnt < NUMB_LBUF) /* still have room*/
                        {
                            if(rtn_len > LENGTH_LBUF) rtn_len=LENGTH_LBUF;
                            dest_buf=&lbuf[lbuf_put_ptr][0];
                            memcpy(dest_buf, pr_buf, rtn_len);
                            len_lbuf[lbuf_put_ptr]=rtn_len;
                            maintain_lbuf_put_ptr();
#ifdef SHOW_RX_INFO 
                            bsp_printf("rb1: \n");
                            bsp_printf("\n"); 
                            dump_buf(dest_buf, rtn_len);
#endif  /* #ifdef SHOW_RX_INFO */
	                }
                        else
                        {
                            flush_i82559_rcv_buf(); /* flush stale info */
                            rtn_len=0;
                            bsp_printf("LBUF runout! ");	
                        }
                    }
	            dev_free_skb(skb);  
                    spdo.rx_ringp[entry] = NULL;
	        }
                spdo.stats.rx_packets++;
                spdo.stats.rx_bytes += pkt_len; /* Count received bytes */
	    }
            entry = (++spdo.cur_rx) % RX_RING_SIZE;
        }
	
        /* Refill the Rx ring buffers. */
        for (; spdo.dirty_rx < spdo.cur_rx; spdo.dirty_rx++) {
	    struct RxFD *rxf;
            entry = spdo.dirty_rx % RX_RING_SIZE;
            if (spdo.rx_skbuff[entry] == NULL) {
	        struct sk_buff *skb;
	        /* Get a fresh skbuff to replace the consumed one. */
	        skb = dev_alloc_skb(PKT_BUF_SZ + sizeof(struct RxFD));
                spdo.rx_skbuff[entry] = skb;
	        if (skb == NULL) {
                     spdo.rx_ringp[entry] = NULL;
		     break;			/* Better luck next time!  */
	        }  
                rxf = spdo.rx_ringp[entry] = (struct RxFD *)skb->tail;
	        skb_reserve(skb, sizeof(struct RxFD));
	        rxf->rx_buf_addr = cpu_to_le32(virt_to_bus((u32)skb->tail));
	    } else {
                rxf = spdo.rx_ringp[entry];
	    }

            /* '1' for driver use only. */
	    rxf->status = cpu_to_le32(0xC0000001); 
	    rxf->link = 0;	/* None yet. */
	    rxf->count = 0;
	    rxf->size = cpu_to_le16(PKT_BUF_SZ);
            spdo.last_rxf->link = cpu_to_le32(virt_to_bus((u32)rxf));
            spdo.last_rxf->status &= cpu_to_le32(~0xC0000000);
            spdo.last_rxf = rxf;
	}
    }

    if (stat_word & 0x1000) {
	if ((stat_word & 0x003c) == 0x0028) /* No more Rx buffers. */
	    //DC outw(RX_RESUMENR, ioaddr + SCBCmd);
	    outb(RX_RESUMENR, ioaddr + SCBCmd);  //DC new driver 
        else if ((stat_word & 0x003c) == 0x0008) { /* No resources (why?!) */
	    /* No idea of what went wrong.  Restart the receiver. */
	    outl(virt_to_bus((u32)(spdo.rx_ringp[spdo.cur_rx % RX_RING_SIZE])),
            ioaddr + SCBPointer);
	    //DC outw(RX_START, ioaddr + SCBCmd);
	    outb(RX_START, ioaddr + SCBCmd);  //DC new driver
	}
        spdo.stats.rx_errors++;
#ifdef SHOW_POLLING_READ_INFO
        bsp_printf(" exit polling_read_i82559: lens=%d\n", rtn_len); 
#endif /* #ifdef SHOW_POLLING_READ_INFO */

        return rtn_len;   
    }

#ifdef SHOW_POLLING_READ_INFO 
    bsp_printf(" exit polling_read_i82559: lens=%d\n", rtn_len); 
#endif /* #ifdef SHOW_POLLING_READ_INFO */

    return rtn_len;
}

static int polling_xmit_i82559(struct sk_buff *skb)
{
    u32 ioaddr = enet_dev.ioaddr;
    int entry;
    unsigned int  tmp_tbusy=0;
    int tickssofar=0;
    unsigned short status = 0;
    int i=0, idummy_x=0, idummy_y=0;
    unsigned int dirty_tx=0;
    struct descriptor *last_cmd;
#ifdef DC_DUMP_TX_INFO 	
    int j=0;
#endif //#ifdef DC_DUMP_TX_INFO 	


    /* Block a timer-based transmit from overlapping.  This could better be
       done with atomic_swap(1, enet_dev.tbusy), but set_bit() works as well.
       If this ever occurs the queue layer is doing something evil! 
    */
    tmp_tbusy=enet_dev.tbusy;
    enet_dev.tbusy=1;
    if (tmp_tbusy != 0) {
        bsp_printf("try to Tx, but enet_dev.busy=1\n");

	if (tickssofar < TX_TIMEOUT_IN_MS - 2)
	    return 1;
	if (tickssofar < TX_TIMEOUT_IN_MS) {
	    /* Reap sent packets from the full Tx queue. */
	    outw(DRVR_INT, ioaddr + SCBCmd);
	    return 1;
	}
        speedo_tx_timeout();
	return 1;
    }

    /* Caution: the write order is important here, set the base address
	   with the "ownership" bits last. 
    */

    /* Calculate the Tx descriptor entry. */
    entry = spdo.cur_tx++ % TX_RING_SIZE;
#ifdef DC_DUMP_TX_INFO 	
    bsp_printf("entry=%d\n", entry);
#endif //#ifdef DC_DUMP_TX_INFO 	

    spdo.tx_skbuff[entry] = skb;
#ifdef DC_DUMP_TX_INFO 	
    bsp_printf("tx_skbuff=%8.8x\n", spdo.tx_skbuff[entry]);
#endif //#ifdef DC_DUMP_TX_INFO 	
    /* Todo: be a little more clever about setting the interrupt bit. */
    spdo.tx_ring[entry].status =
	cpu_to_le32((CmdSuspend | CmdTx | CmdTxFlex) << 16);
#ifdef DC_DUMP_TX_INFO 	
    bsp_printf("re_stat: swapped=%8.8x, unswapped=%8.8x\n",
        spdo.tx_ring[entry].status, cpu_to_le32(spdo.tx_ring[entry].status));
#endif //#ifdef DC_DUMP_TX_INFO 	
    spdo.tx_ring[entry].link =
        cpu_to_le32(virt_to_bus((u32)(&spdo.tx_ring[spdo.cur_tx % TX_RING_SIZE])));
#ifdef DC_DUMP_TX_INFO 	
    bsp_printf("re_link: swapped=%8.8x, unswapped=%8.8x\n",
        spdo.tx_ring[entry].link, cpu_to_le32(spdo.tx_ring[entry].link));
#endif //#ifdef DC_DUMP_TX_INFO 	
    spdo.tx_ring[entry].tx_desc_addr =
        cpu_to_le32(virt_to_bus((u32)(&spdo.tx_ring[entry].tx_buf_addr0)));
#ifdef DC_DUMP_TX_INFO 	
    bsp_printf("tx_addr: swapped=%8.8x, unswapped=%8.8x\n",
        spdo.tx_ring[entry].tx_desc_addr, cpu_to_le32(spdo.tx_ring[entry].tx_desc_addr));
#endif //#ifdef DC_DUMP_TX_INFO 	
    /* The data region is always in one buffer descriptor, Tx FIFO
		   threshold of 256. */
    spdo.tx_ring[entry].count = cpu_to_le32(0x01208000);
#ifdef DC_DUMP_TX_INFO 	
    bsp_printf("re_count: swapped=%8.8x, unswapped=%8.8x\n",
        spdo.tx_ring[entry].count, cpu_to_le32(spdo.tx_ring[entry].count));
#endif //#ifdef DC_DUMP_TX_INFO 	
    spdo.tx_ring[entry].tx_buf_addr0 = cpu_to_le32(virt_to_bus((u32)skb->data));
#ifdef DC_DUMP_TX_INFO 	
    bsp_printf("tb_ad0: swapped=%8.8x, unswapped=%8.8x\n",
        spdo.tx_ring[entry].tx_buf_addr0, cpu_to_le32(spdo.tx_ring[entry].tx_buf_addr0));
#endif //#ifdef DC_DUMP_TX_INFO 	
    spdo.tx_ring[entry].tx_buf_size0 = cpu_to_le32(skb->len);
#ifdef DC_DUMP_TX_INFO 	
    bsp_printf("size0: swapped=%8.8x, unswapped=%8.8x\n",
        spdo.tx_ring[entry].tx_buf_size0, cpu_to_le32(spdo.tx_ring[entry].tx_buf_size0));
#endif //#ifdef DC_DUMP_TX_INFO 	
    /* Todo: perhaps leave the interrupt bit set if the Tx queue is more
	than half full.  Argument against: we should be receiving packets
	 and scavenging the queue.  Argument for: if so, it shouldn't
	 matter. 
    */
    last_cmd = spdo.last_cmd;
    spdo.last_cmd = (struct descriptor *)&spdo.tx_ring[entry];
#ifdef DC_DUMP_TX_INFO 	
    bsp_printf(" bf clear_suspend cmdcmd: swapped=%8.8x, unswapped=%8.8x\n",
       last_cmd->cmd_status, cpu_to_le32(last_cmd->cmd_status));
#endif //#ifdef DC_DUMP_TX_INFO 	
    clear_suspend(last_cmd);
#ifdef DC_DUMP_TX_INFO 	
    bsp_printf(" af clear_suspend cmdcmd: swapped=%8.8x, unswapped=%8.8x\n",
       last_cmd->cmd_status, cpu_to_le32(last_cmd->cmd_status));
#endif //#ifdef DC_DUMP_TX_INFO 	
#ifdef DC_DUMP_TX_INFO 	
    bsp_printf("lst_cmd=%8.8x\n", spdo.last_cmd);
#endif //#ifdef DC_DUMP_TX_INFO 	

    /* Leave room for set_rx_mode() to fill two entries. */
    if (spdo.cur_tx - spdo.dirty_tx > TX_RING_SIZE - 3)
        spdo.tx_full = 1;
    else
        enet_dev.tbusy=0;

    /* Trigger the command unit resume. */
    wait_for_cmd_done(ioaddr + SCBCmd);
    outb(CU_RESUME, ioaddr + SCBCmd);

#ifdef DC_DUMP_TX_INFO 	
    spdo_start=&spdo;
    bsp_printf(" following is spdo struct, total %d bytes:\n",
	 sizeof(struct speedo_private));

    dump_buf(spdo_start, sizeof(struct speedo_private));

    bsp_printf("pxao: status %4.4x %4.4x at d=%d/c=%d/e=%d d_s=%8.8x c_s=%8.8x e_s=%8.8x e-1_s=%8.8x\n",
        inw(ioaddr + SCBStatus), inw(ioaddr + SCBCmd),
        spdo.dirty_tx, spdo.cur_tx,entry,
        le32_to_cpu(spdo.tx_ring[spdo.dirty_tx % TX_RING_SIZE].status),
        le32_to_cpu(spdo.tx_ring[spdo.cur_tx % TX_RING_SIZE].status),
        le32_to_cpu(spdo.tx_ring[entry].status),
        le32_to_cpu(spdo.tx_ring[entry-1].status));
#endif //#ifdef DC_DUMP_TX_INFO 	
#ifdef SHOW_TX_INFO
    tx_length=cpu_to_le32(spdo.tx_ring[entry].tx_buf_size0);   
    bsp_printf("txlen=%d, tb:\n", tx_length);
    dump_buf(skb->data, tx_length);
#endif /* #ifdef SHOW_TX_INFO */

    /* polling status util it finished xmit */
    status = inw(ioaddr + SCBStatus);
#ifdef DC_DUMP_TX_INFO 	
    bsp_printf(" QX s=%4.4x.\n", status);  /* DC temp */
    j=0;
#endif //#ifdef DC_DUMP_TX_INFO 	
    while(!(status & 0xA000))
    { 
#ifdef DC_DUMP_TX_INFO 	
        j++;
        if(j > 0x0007FFFF)
        {
            speedo_tx_timeout();
	    return 1;
        }
#endif //#ifdef DC_DUMP_TX_INFO 	
	for(i=0; i <0xf; i++) idummy_y=idummy_x*3 +2;
	status = inw(ioaddr + SCBStatus);
    }

#ifdef DC_DUMP_TX_INFO 	
    bsp_printf(" PXf s=%4.4x.\n", status);  /* DC temp */
#endif //#ifdef DC_DUMP_TX_INFO 	

    /* finish xmit */
    dirty_tx = spdo.dirty_tx;
    while (spdo.cur_tx - dirty_tx > 0) {
	entry = dirty_tx % TX_RING_SIZE;
        status = le32_to_cpu(spdo.tx_ring[entry].status);

	if (speedo_debug > 5)
                    bsp_printf(" scavenge candidate %d status %4.4x.\n",
						   entry, status);
	if ((status & StatusComplete) == 0)
	    break;			/* It still hasn't been processed. */
	/* Free the original skb. */
        if (spdo.tx_skbuff[entry]) {
           spdo.stats.tx_packets++; /* Count only user packets. */
           spdo.stats.tx_bytes += spdo.tx_skbuff[entry]->len; /* Count transmitted bytes */
           spdo.tx_skbuff[entry] = 0;
	} 
	else if ((status&0x70000) == CmdNOp << 16)
            spdo.mc_setup_busy = 0;
	dirty_tx++;
    }

#ifndef final_version
    if (spdo.cur_tx - dirty_tx > TX_RING_SIZE) {
	bsp_printf("out-of-sync dirty pointer, %d vs. %d, full=%d.\n", 
	    dirty_tx, spdo.cur_tx, spdo.tx_full);
	dirty_tx += TX_RING_SIZE;
    }
#endif

    if (spdo.tx_full && enet_dev.tbusy
	&& dirty_tx > spdo.cur_tx - TX_RING_SIZE + 2) {
	/* The ring is no longer full, clear tbusy. */
        spdo.tx_full = 0;
        enet_dev.tbusy=0;
    }

    spdo.dirty_tx = dirty_tx;
    /* Clear all interrupt sources. */
    outw(0xfc00, ioaddr + SCBStatus);  //don't overwriiten 
		
    return 0;
}
#endif /* #ifndef USE_I82559_INTERRUPT */        

void _setup_i82559_vec() /* DC setup interrupt handler for i82559 */  
{
#ifdef USE_I82559_INTERRUPT        
    static bsp_vec_t eth_vec;
    /* static bsp_vec_t dummy_uart_vec; */
    
    eth_vec.handler = i559_irq_handler;
    eth_vec.next = NULL;

    /* if ixp1200 is not central func, no need to setup int vector */
    if (!is_ixp1200_central_func)
        return;

    bsp_install_vec(BSP_VEC_INTERRUPT, IXP1200_PCI_INT_VEC,
    BSP_VEC_REPLACE, &eth_vec);

    bsp_enable_irq(IXP1200_PCI_INT_VEC);
    *IXP1200_PCI_IRQ_ENABLE_SET |= IXP1200_PCI_IRQ_PIL_SET;
#endif /* #ifdef USE_I82559_INTERRUPT */        
}

void _init_i82559(int option) 
//option = 0 - just reinitialize, will not call _bsp_net_init()
{
    struct bsp_comm_procs *eprocs;

    /* if ixp1200 is not central func, no need to initialize i82559 */
    if (!is_ixp1200_central_func)
        return;

    /* first find i82559 and test it and put MAC address into __local_enet_addr */
    find_test_i82559();
    if(cards_found)
    {
    eprocs = __enet_link_init(__local_enet_addr);

    if (option)
    {
        if (eprocs) {
            if (!_bsp_net_init(eprocs))
            bsp_printf("Network initialization failed.\n");
        }
        else
            bsp_printf("Ethernet initialization failed.\n");
    }
#ifdef USE_I82559_INTERRUPT        

#ifdef INIFINITE_LOOP_FOR_TEST
        while(1)
        {
        }
#endif /* #ifdef INIFINITE_LOOP_FOR_TEST */

#endif /* #ifdef USE_I82559_INTERRUPT */       

    }  
}


int isprint(unsigned char ch)
{
    return (ch >= 0x20 && ch < 0x7f);
}

void dump_buf_16(char rbuff[], unsigned long foffs, int nReadCnt)
{
    int     nInd;
    
    bsp_printf("%08x", foffs);

    for (nInd = 0; nInd < nReadCnt; nInd++)
    {
    if ((nInd % 8) == 0)
        bsp_printf(" ");
        bsp_printf("%02x ", rbuff[nInd]);
    }

    for (; nInd < 16; nInd++)
    {
    bsp_printf("   ");
        if ((nInd % 8) == 0)
        bsp_printf(" ");
    }

    for (nInd = 0; nInd < nReadCnt; nInd++)
    {
    if (isprint(rbuff[nInd]))
            bsp_printf("%c", rbuff[nInd]);
        else
            bsp_printf(".");
    }
    bsp_printf("\n");
}


void dump_buf(char rbuff[], int nReadCnt)
{
    unsigned long foffs = 0;
    int nLen, ldlen;
    char *bp;

    bp = rbuff;

    for (nLen = 0; nLen < nReadCnt; nLen += 16, foffs += 16, bp += 16)
    {
        if ((nLen + 16) <= nReadCnt)
            ldlen = 16;
        else
            ldlen = nReadCnt - nLen;

        dump_buf_16(bp, foffs, 16);
    }
}

void disable_i82559()
{
    u32 ioaddr=enet_dev.ioaddr;
    int i;
    struct sk_buff *skb;
    //this i82559 will xmit some unexpected garbage packets after 
    //idle for certain time, so put it int software reset, when we
    //don't need it

    // if ixp1200 is not central func, no need to do this
    if (!is_ixp1200_central_func)
        return;
    outl(PortReset, ioaddr + SCBPort);
    udelay(10);

    //also free spdo.rx_skbuff allocated 
    for (i = 0; i < RX_RING_SIZE; i++)
    {
        skb=spdo.rx_skbuff[i];
        dev_free_skb(skb);
    }
}

#ifdef __LITTLE_ENDIAN__ 

static unsigned char inb(unsigned int addr)
{
    unsigned char val_uchar=0;
    val_uchar=*(volatile unsigned char *)((addr + IXP1200_PCI_IO_BASE));
    return val_uchar;
}

static unsigned short inw(unsigned int addr)
{
    unsigned short val_ushort=0;
    val_ushort=*(volatile unsigned short *)((addr + IXP1200_PCI_IO_BASE));
    return val_ushort;
}

static void outb(unsigned char val_uchar, unsigned int addr)
{
    *(volatile unsigned char *)((addr + IXP1200_PCI_IO_BASE))=val_uchar;
}

static void outw(unsigned short val_ushort, unsigned int addr)
{
    *(volatile unsigned short *)((addr + IXP1200_PCI_IO_BASE))=val_ushort;
}

#else //followng for big endian 
/* due to arm architechure, for big endian, we need to do 
   following  modification */
static unsigned int be_byte_addr[4]={3,2,1,0};
static unsigned int be_short_addr[4]={2,2,0,0};

static unsigned char inb(unsigned int addr)
{
    unsigned char val_uchar=0;
    unsigned int mod_addr=0;
    mod_addr=(addr/4)*4 + be_byte_addr[addr%4];
    val_uchar=*(volatile unsigned char *)((mod_addr + IXP1200_PCI_IO_BASE));
    return val_uchar;
}

static unsigned short inw(unsigned int addr)
{
    unsigned short val_ushort=0;
    unsigned int mod_addr=0;

    if( (addr%4 == 1) || (addr%4 == 3))
 	bsp_printf(" inw addr=%8.8X, not resonable!\n");
    mod_addr=(addr/4)*4 + be_short_addr[addr%4];
    val_ushort=*(volatile unsigned short *)((mod_addr + IXP1200_PCI_IO_BASE));
    return val_ushort;
}

static void outb(unsigned char val_uchar, unsigned int addr)
{
    unsigned int mod_addr=0;
    mod_addr=(addr/4)*4 + be_byte_addr[addr%4];
    *(volatile unsigned char *)((mod_addr + IXP1200_PCI_IO_BASE))=val_uchar;
}

static void outw(unsigned short val_ushort, unsigned int addr)
{
    unsigned int mod_addr=0;
    if( (addr%4 == 1) || (addr%4 == 3))
 	bsp_printf(" outw addr=%8.8X, not resonable!\n");
    mod_addr=(addr/4)*4 + be_short_addr[addr%4];
    *(volatile unsigned short *)((mod_addr + IXP1200_PCI_IO_BASE))=val_ushort;
}

#endif //#ifdef __LITTLE_ENDIAN__ 

