/*
 * ixp1200macros.S -- Header for macros on Intel(R) IXP1200 Evaluation Board
 *
 * Copyright (c) 1998, 1999 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * Intel is a Registered Trademark of Intel Corporation.
 * ARM is a Registered Trademark of Advanced RISC Machines Limited.
 * Other Brands and Trademarks are the property of their respective owners.
 */

	#include <bsp/mmumacros.S> 
/* macro to disable interrupts on powerup*/

.macro WLED  num
	LDR	R8, = 0x38508000
	MOV	R9, #(\num)
	STR	R9,[R8]
.endm


.macro WLED_REG  reg
	LDR	R8, = 0x38508000
	MOV	R9, \reg
	STR	R9,[R8]
.endm


//
// Coprocessor read of ID register
//
.macro RDCP15_ID reg_number
    MRC p15, 0, \reg_number, c0, c0 ,0
.endm

//
// Coprocessor read of Control register
//
.macro RDCP15_Control  reg_number
    MRC p15, 0, \reg_number, c1, c0 ,0
.endm

// 
// Coprocessor write of Control register
//
.macro WRCP15_Control  reg_number
    MCR p15, 0, \reg_number, c1, c0 ,0
.endm


//
// Coprocessor read of Translation Table Base reg.
// 
.macro RDCP15_TTBase reg_number
    MRC p15, 0, \reg_number, c2, c0 ,0
.endm


//
// Coprocessor write of Translation Table Base reg.
// 
.macro WRCP15_TTBase reg_number
    MCR p15, 0, \reg_number , c2, c0 ,0
.endm


//
// Coprocessor read of Domain Access Control reg.
// 
.macro RDCP15_DAControl reg_number
    MRC p15, 0, \reg_number, c3, c0 ,0
.endm


//
// Coprocessor write of Domain Access Control reg.
// 
.macro WRCP15_DAControl reg_number
    MCR p15, 0, \reg_number, c3, c0 ,0
.endm

//
// Coprocessor read of Fault Status register
// 
.macro RDCP15_FaultStatus reg_number
    MRC p15, 0, \reg_number, c5, c0 ,0
.endm


//
// Coprocessor write of Fault Status register
// 
.macro WRCP15_FaultStatus reg_number
    MCR p15, 0, \reg_number, c5, c0 ,0
.endm


//
// Coprocessor read of Fault Address register
// 
.macro RDCP15_FaultAddress reg_number
    MRC p15, 0, \reg_number, c6, c0 ,0
.endm


//
// Coprocessor write of Fault Address register
// 
.macro WRCP15_FaultAddress reg_number
    MCR p15, 0, \reg_number, c6, c0 ,0
.endm

//
// Coprocessor cache control
// Flush ICache + DCache
// 
.macro WRCP15_FlushIC_DC reg_number
    MCR p15, 0, \reg_number, c7, c7 ,0
.endm

//
// Coprocessor cache control
// Flush ICache
//
.macro WRCP15_FlushIC reg_number
    MCR p15, 0, \reg_number, c7, c5 ,0
.endm


//
// Coprocessor cache control
// Flush DCache
//
.macro WRCP15_FlushDC reg_number
    MCR p15, 0, \reg_number, c7, c6 ,0
.endm

//
// Coprocessor cache control
// Flush DCache entry
//
.macro WRCP15_CacheFlushDentry reg_number
    MCR p15, 0, \reg_number, c7, c6 ,1
.endm


//
// Coprocessor cache control
// Clean DCache entry
//
.macro WRCP15_CleanDCentry reg_number
    MCR p15, 0, \reg_number, c7, c10 ,1
.endm


//
// Coprocessor cache control
// Clean + Flush DCache entry
//
.macro WRCP15_Clean_FlushDCentry reg_number
    MCR p15, 0, \reg_number, c7, c14 ,1
.endm


//
// Coprocessor Drain Write Buffer
//
.macro WRCP15_DrainWriteBuffer reg_number
    MCR p15, 0, \reg_number, c7, c10 ,4
.endm


//
// Coprocessor TLB control
// Flush ITB + DTB
//
.macro WRCP15_FlushITB_DTB reg_number
    MCR p15, 0, \reg_number, c8, c7 ,0
.endm


//
// Coprocessor TLB control
// Flush ITB
//
.macro WRCP15_FlushITB reg_number
    MCR p15, 0, \reg_number, c8, c5 ,0
.endm

//
// Coprocessor TLB control
// Flush DTB
//
.macro WRCP15_FlushDTB reg_number
    MCR p15, 0, \reg_number, c8, c6 ,0
.endm


//
// Coprocessor TLB control
// Flush DTB entry
//
.macro WRCP15_FlushDTBentry reg_number
    MCR p15, 0, \reg_number, c8, c6 ,1
.endm


//
// Coprocessor test/clock/idle control
// Enable Clock Switching
//
.macro WRCP15_EnableClockSW reg_number
    MCR p15, 0, \reg_number, c15, c1 ,2
.endm


//
// Coprocessor test/clock/idle control
// Disable Clock Switching
//
.macro WRCP15_DisableClockSW reg_number
    MCR p15, 0, \reg_number, c15, c2 ,2
.endm


//
// Coprocessor test/clock/idle control
// Disable nMCLK output
//
.macro WRCP15_DisablenMCLK reg_number
    MCR p15, 0, \reg_number, c15, c4 ,2
.endm


//
// Coprocessor test/clock/idle control
// Wait for Interrupt
//
.macro WRCP15_WaitInt reg_number
    MCR p15, 0, \reg_number, c15, c8 ,2
.endm



//
// ------------------------------------------------------------
// Disable interrupts on powerup
// ------------------------------------------------------------
//
.macro DISABLE_INTS    reg1, reg2
        MOV     \reg2, #0
        LDR     \reg1, =IREG
        STR     \reg2, [\reg1]          /*@ disable micro engine interrupts*/
        LDR     \reg1, =UART_CR
        STR     \reg2, [\reg1]          /*@ disable UART interrupts*/
        LDR     \reg1, =(CSR_BASE + IRQ_ENABLE_CLEAR)
        MOV     \reg2, #ALL_BITS
        STR     \reg2, [\reg1]          /*@ disable rest of IRQ interrupts*/
        LDR     \reg1, =(CSR_BASE + FIQ_ENABLE_CLEAR)
        MOV     \reg2, #ALL_BITS
        STR     \reg2, [\reg1]          /*@ disable rest of FIQ interrupt*/
.endm

//
// ------------------------------------------------------------
// Macro to setup GPIOs on powerup.
// set up the direction of the GPIO pins connnected to the LEDs
// ------------------------------------------------------------
//
// .macro INIT_GPIOS      /*r0, r1*/
.macro INIT_GPIOS      reg1, reg2
        MOV     \reg2, #0xF           /*@ all pins output*/
        LDR     \reg1, =GPIO_EN       /*@ GPIO enable register*/
        STR     \reg2, [\reg1]          /*@ set the direction of bits*/
        MOV     \reg1, #0x0
// @        SET_LEDS r0, r1
.endm


//
//  ------------------------------------------------------------------
//  SETUPMMU
//  --------
//  Setup the memory map world for the target. This .macro is called by
//  INITMMU from ANGEL if required or by uHAL when running a
//  standalone program.
//  Startup is more painful on SA1200 than usual.
//  Consider a program built to run from ROM at 16MB (with RAM at 0),
//  but is actually in ROM at 0 (with RAM at high addresses). If we
//  try to build a new page table with ROM at EPROM_BASE & RAM at 0,
//  the program will use the low RAM alias for the page table which
//  doesn't exist yet. As soon as we try and use the new map the
//  program will have gone (to ROM_BASE), but we have not changed
//  the pc yet. So we have to build the desired page table in the real
//  RAM address range but overwrite the entry for the current address
//  to map back to ROM. Then, we can switch to the mapped version of
//  the ROM and restore the low memory page table entry (and then
//  flush the ITBs, DTBs and ICache).
// 
//  NOTE: This .macro needs to know which RAM Bank the page tables live
//        in, since it writes to them absolutely. If TTB_BASE is not
//        defined, the tables are assumed to be in the first RAM bank.
// 
//  Does your brain hurt yet? OK, picture time:
// 
//  Start         Physical Address    Virtual Address     Size
//  0x00000000    ROMBase             0x08000000          1Mb
//  0xC1800000    DRAMBase            0x00000000          8Mb
// 
//  Everything else is 1-1.
//  ------------------------------------------------------------------
//

// .macro SETUPMMU        @r0, r1, r2, r3, r6, r7
.macro SETUPMMU        tmp1, tmp2, tmp3, tmp4, tmp5, tabl

//
//  Setup the StrongARM MMU with a mapped page table:
//  set up the page tables - all for domain0, switched address map
//        RAM_SIZE / 1Mb sections for DRAM
//                - Read/Write all, cachable & bufferable
//        ROM_SIZE / 1MB sections for ROM
//                - Read all, cachable, nonbufferable
//        The rest of address space
//                - noncachable, nonbufferable

//  set the domain register: domain0 CLIENT, domain1-15 NO_ACCESS
//  write the translation base register
//  Flush ITB's and DTB's
//  Flush the Icache

//  First clear all TT entries as FAULT
// 
#if DO_LED
	WLED	40
#endif // DO_LED

        LDR     \tmp4, =TTB_BASE
        LDR     \tmp3, =0
        LDR     \tmp1, =(TTB_SIZE / 4)
1:
        STR     \tmp3, [\tmp4], #4
        SUBS    \tmp1, \tmp1, #1        /* decrement loop count*/
        BNE     1b

#if DO_LED
	WLED	41
#endif // DO_LED

        // --------------------------------------------------
        // Map DRAM non-prefetch area to address 0 and itself
        // --------------------------------------------------
        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =DRAMBase                @ physical address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ index to table entry
        ADD     \tmp4, \tmp4, \tmp2             @ address to table entry

        // Access permissions
        LDR     \tmp3, =NOCACHE_NOBUFFER_ACCESS @ No cache, no buffer
        ADD     \tmp5, \tmp1, \tmp3             @ table entry for upper RAM
        LDR     \tmp3, =CACHED_BUFFERED_ACCESS  @ cachable & bufferable
        ADD     \tmp1, \tmp1, \tmp3             @ table entry for lower RAM

        LDR     \tmp3, =DRAMSize                @ Size of first bank
        MOV     \tmp3, \tmp3, LSR #20           @ no. of 1MB segments
2:
        STR     \tmp1, [\tmp2], #4              @ virtual entry
        STR     \tmp5, [\tmp4], #4              @ real entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        ADD     \tmp5, \tmp5, #SZ_1M
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     2b

#if DO_LED
	WLED	42
#endif // DO_LED

        // -----------------------------------------------
        // Map DRAM portion for micro engine use to itself
        // -----------------------------------------------
        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =DRAMBaseForUEng         @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ index to table entry
        ADD     \tmp4, \tmp4, \tmp2             @ address to table entry

        // Access permissions
        LDR     \tmp3, =NOCACHE_NOBUFFER_ACCESS @ No cache, no buffer
        LDR     \tmp1, =DRAMBaseForUEng         @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =DRAMBaseForUEngSize
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
33:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     33b

#if DO_LED
	WLED	43
#endif // DO_LED

        // --------------------------------
        // Map DRAM prefetch area to itself
        // --------------------------------
        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =DRAMPrefetchBase        @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ index to table entry
        ADD     \tmp4, \tmp4, \tmp2             @ address to table entry

        @ Access permissions
        LDR     \tmp3, =NOCACHE_NOBUFFER_ACCESS @ No cache, no buffer
        LDR     \tmp1, =DRAMPrefetchBase        @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =DRAMPrefetchSize
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
3:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     3b

#if DO_LED
	WLED	44
#endif // DO_LED
        // -----------------------------------
        // Map DRAM cache flush area to itself
        // -----------------------------------

        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =DRAMFlushBase           @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ Index to the table
        ADD     \tmp4, \tmp4, \tmp2             @ address of entry

        // Access permissions
        LDR     \tmp3, =CACHED_NOBUFFER_ACCESS  @ Yes cache, no buffer
        LDR     \tmp1, =DRAMFlushBase           @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =DRAMFlushSize
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
4:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     4b

#if DO_LED
	WLED	45
#endif // DO_LED
        // -------------------------------
        // Map DRAM control area to itself
        // -------------------------------

        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =DRAMControlBase         @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ Index to the table
        ADD     \tmp4, \tmp4, \tmp2             @ address of entry

        // Access permissions
        LDR     \tmp3, =NOCACHE_NOBUFFER_ACCESS @ No cache, no buffer
        LDR     \tmp1, =DRAMControlBase         @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =DRAMControlSize
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
5:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     5b

#if DO_LED
	WLED	46
#endif // DO_LED
        // ------------------------------
        // Map AMBA Translation Unit area
        // ------------------------------
        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =AMBABase                @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ Index to the table
        ADD     \tmp4, \tmp4, \tmp2             @ address of entry

        // Access permissions
        LDR     \tmp3, =NOCACHE_NOBUFFER_ACCESS @ No cache, no buffer
        LDR     \tmp1, =AMBABase                @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =AMBASize
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
6:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     6b

#if DO_LED
	WLED	47
#endif // DO_LED
        // -----------------
        // Map ARM Core area
        // -----------------

        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =ARMCOREBase             @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ Index to the table
        ADD     \tmp4, \tmp4, \tmp2             @ address of entry

        // Access permissions
        LDR     \tmp3, =NOCACHE_NOBUFFER_ACCESS @ No cache, no buffer
        LDR     \tmp1, =ARMCOREBase             @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =ARMCORESize
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
7:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     7b

#if DO_LED
	WLED	48
#endif // DO_LED
        // --------------------
        // Map PCI Memory Space
        // --------------------

        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =PCIMemBase              @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ Index to the table
        ADD     \tmp4, \tmp4, \tmp2             @ address of entry

        // Access permissions
        LDR     \tmp3, =NOCACHE_NOBUFFER_ACCESS @ No cache, no buffer
        LDR     \tmp1, =PCIMemBase              @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =PCIMemSize
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
8:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     8b

#if DO_LED
	WLED	49
#endif // DO_LED

        // -----------------
        // Map PCI I/O Space
        // -----------------

        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =PCIIOBase               @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ Index to the table
        ADD     \tmp4, \tmp4, \tmp2             @ address of entry

        // Access permissions
        LDR     \tmp3, =NOCACHE_NOBUFFER_ACCESS @ No cache, no buffer
        LDR     \tmp1, =PCIIOBase               @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =PCIIOSize
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
9:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     9b

#if DO_LED
	WLED	50
#endif // DO_LED
        // ------------------------------------
        // Map PCI Type One Configuration Space
        // ------------------------------------

        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =PCIConfigTypeOneBase    @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ Index to the table
        ADD     \tmp4, \tmp4, \tmp2             @ address of entry

        // Access permissions
        LDR     \tmp3, =NOCACHE_NOBUFFER_ACCESS @ No cache, no buffer
        LDR     \tmp1, =PCIConfigTypeOneBase    @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =PCIConfigTypeOneSize
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
10:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     10b

#if DO_LED
	WLED	51
#endif // DO_LED
        // -------------------------------------
        // Map PCI Type Zero Configuration Space
        // -------------------------------------

        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =PCIConfigTypeZeroBase   @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ Index to the table
        ADD     \tmp4, \tmp4, \tmp2             @ address of entry

        // Access permissions
        LDR     \tmp3, =NOCACHE_NOBUFFER_ACCESS @ No cache, no buffer
        LDR     \tmp1, =PCIConfigTypeZeroBase   @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =PCIConfigTypeZeroSize
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
11:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     11b


#if DO_LED
	WLED	52
#endif // DO_LED
        // ---------------------------------
        // Map PCI Local Configuration Space
        // ---------------------------------

        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =PCILocalBase            @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ Index to the table
        ADD     \tmp4, \tmp4, \tmp2             @ address of entry

        // Access permissions
        LDR     \tmp3, =NOCACHE_NOBUFFER_ACCESS @ No cache, no buffer
        LDR     \tmp1, =PCILocalBase            @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =PCILocalSize
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
12:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     12b

        // -----------------------
        // Map SRAM & Control area
        // -----------------------

        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =SRAMBase                @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ Index to the table
        ADD     \tmp4, \tmp4, \tmp2             @ address of entry

        // Access permissions
        LDR     \tmp3, =NOCACHE_NOBUFFER_ACCESS @ No cache, no buffer
        LDR     \tmp1, =SRAMBase                @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =SRAMSize
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
13:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     13b

        // ------------
        // Map ROM area
        // ------------

        LDR     \tmp2, =TTB_BASE
        LDR     \tmp1, =ROM_BASE                @ virtual address
        MOV     \tmp4, \tmp1, LSR #(20-2)       @ index to table entry
        ADD     \tmp4, \tmp4, \tmp2             @ address to table entry

        // Access permissions
        LDR     \tmp3, =CACHED_NOBUFFER_ACCESS  @ No cache, no buffer
        LDR     \tmp1, =ROMBase                 @ physical address
        ADD     \tmp1, \tmp1, \tmp3             @ entry

        LDR     \tmp3, =ROM_SIZE
        MOV     \tmp3, \tmp3, LSR #20           @ number of 1MB segments
14:
        STR     \tmp1, [\tmp4], #4              @ virtual entry
        ADD     \tmp1, \tmp1, #SZ_1M            @ add section number field
        SUBS    \tmp3, \tmp3, #1                @ decrement loop count
        BNE     14b

#if DO_LED
	WLED	45
#endif // DO_LED
        // ------------------------------------------------------------------
        // No MMU active, and executing from ROM space
        // ------------------------------------------------------------------
        RDCP15_Control  \tmp1                   @ Read the MMU state
//    	MRC p15, 0, \tmp1, c1, c0 ,0
        LDR     \tmp2, =EnableMMU               @ MMU enable bit
        ORR     \tmp1, \tmp1, \tmp2             @ Set the Enable bit

        // After MMU active current execution locaton (PC value) will be
        // in DRAM space (0x00000000 - 0x00100000) and the physical
        // ROM position we are at now will be remapped to
        // (0x08000000 - 0x08100000) area.
        // So in order to continue our execution after MMU active, we
        // need to temporarily map the current section into DRAM area.
        // After MMU activation, we'll jump to 0x08000000 area and
        // recover the temporary section in DRAM area.

        LDR     \tmp5, =TTB_BASE
        MOV     \tmp4, pc, LSR #(20-2)          @ Current execution location
        ADD     \tmp4, \tmp4, \tmp5             @ RAM Page Table entry
        // Don't overwrite \tmp3 and \tmp4, so we can restore these values
        LDR     \tmp3, [\tmp4]                  @ save current section

#if DO_LED
	WLED 30
#endif // DO_LED
//  ROM_BASE load was commented out
//        LDR     \tmp2, =ROM_BASE:SHR:(20-2)     // we'll be there after MMU
        LDR     \tmp2, =ROM_BASE     // we'll be there after MMU
        ADD     \tmp2, \tmp2, \tmp5             @ ROM Page Table entry
        LDR     \tmp5, [\tmp2]

        // Overwrite RAM entry temporarily
        STR     \tmp5, [\tmp4]

        // ------------------------------------------------------------------
        // Turn on the MMU and flush everything
        // ------------------------------------------------------------------
@ORIG        LDR     \tmp2, =EnableMMU
        LDR     \tmp2, =EnableMMU
        LDR     \tmp5, =TTB_BASE   @TTB address is 2**14 aligned

        WRCP15_DAControl \tmp2          @Initialise Domain Access Control
//    	MRC p15, 0, \tmp2, c3, c0 ,0

        WRCP15_TTBase    \tmp5          @Initialise Translation Table Base reg.
//    	MCR p15, 0, \tmp5 , c2, c0 ,0

//  We disabled TRUN_ON_MMU
        WRCP15_Control  \tmp2           @ Turn on the MMU
//    	MCR p15, 0, \tmp2, c1, c0 ,0


#if DO_LED
	WLED 31
#endif // DO_LED

        // ------------------------------------------------------------------
        // Jump from DRAM area (0x00000000 - 0x00100000) to
        // ROM area (0x08000000 - 0x08100000).
        // ------------------------------------------------------------------
        MOV     \tmp5, pc
        ADD     \tmp5, \tmp5, #ROM_BASE         @ add offset to mapped ROM
        ADD     \tmp5, \tmp5, #0x8

        MOV     pc, \tmp5
        // Don't forget return address (if any)
        ADD     lr, lr, #ROM_BASE

        // ------------------------------------------------------------------
        // Restore the overwritten page table entry & flush everything
        // ------------------------------------------------------------------
        STR     \tmp3, [\tmp4]
        WRCP15_FlushITB_DTB     \tmp1   @ Flush ITB's + DTB's
//      MCR p15, 0, \tmp1, c8, c7 ,0

        WRCP15_FlushIC_DC       \tmp1   @ Flush the ICache and DCaches
//	MCR p15, 0, \tmp1, c7, c7 ,0
        NOP                             @ make sure that the pipe is empty
        NOP
        NOP
        NOP
#if DO_LED
	WLED 32
#endif // DO_LED
60:	b 60b

.endm

//
// ----------------------------------------------------
// Switch to big endian mode
// ----------------------------------------------------
//
.macro SET_BIG_ENDIAN
        RDCP15_Control  r0              // Read the MMU state
        ORR     r0, r0, #EnableBigEndian
        WRCP15_Control  r0   // Update control register

//        WLED_REG    r0
//100:    b 100b
.endm

//
// ----------------------------------------------------
//  SA_INITMMU
//  ----------
//  uHAL macro, calls the core SETUPMMU macro if the MMU is
//  enabled and then sets ICache, DCache & WBuffer as required.
//  On certain system where the target does not have a MMU, then this
//  macro is almost a NOP, since using the ICache & WBuffer on their
//  own is still a big gain.
// ----------------------------------------------------
//
// .macro SA_INITMMU       @r0, r1, r2, r3, r6, r7
.macro SA_INITMMU       tmp1, tmp2, tmp3, tmp4, tmp5, tmp6

// setup and enable MMU, with caches off

#if DO_LED
	WLED 14
#endif // DO_LED

#if DO_SETUPMMU
        SETUPMMU       \tmp1, \tmp2, \tmp3, \tmp4, \tmp5, \tmp6
#endif // DO_SETUPMMU

#if DO_LED
	WLED 15
#endif // DO_LED


        // enable caches
/*  We changed IC_ON to IC_ON_BIT and DC_ON to DC_ON_BIT
 *  to get rid off compiler warnings for multiple definitions
 */
        LDR     \tmp1, =(IC_ON_BIT + DC_ON_BIT + EnableMMU)

#IF {ENDIAN} = "big"
        ORR     \tmp1, \tmp1, #EnableBigEndian
        WLED    0x77
#ENDIF  /*@ BIG_ENDIAN*/

        WRCP15_Control  \tmp1   // Update control register
//    	MCR p15, 0, \tmp1, c1, c0 ,0

        // Flush Instruction Cache and Data Cache
        WRCP15_FlushITB_DTB     \tmp1   // Flush ITB's + DTB's
//      MCR p15, 0, \tmp1, c8, c7 ,0

        WRCP15_FlushIC_DC       \tmp1   // Flush the ICache and DCaches
//      MCR p15, 0, \tmp1, c7, c7 ,0

        WRCP15_DrainWriteBuffer \tmp1
//      MCR p15, 0, \tmp1, c7, c10 ,4

        NOP                            /* @ make sure that the pipe is empty*/
        NOP
        NOP
.endm


//
// -------------------------------------------------------------
// INIT_RAM
// -------------------------------------------------------------
// ANGEL and uHAL .macro to initialise memory on startup. See the calling
// .macros for reasons why this particular register set is used.
// -------------------------------------------------------------

// .macro INIT_RAM       /* r0, r1, r5*/
.macro INIT_RAM       reg1, reg2, reg3

//      By checking if the MMU is already active, memory will not be
//      initialized when running under Angel or from RAM.
//
        RDCP15_Control  \reg2              // Read the MMU state
        LDR     \reg3, =EnableMMU
        TST     \reg2, \reg3               // Already active?

#ifdef DO_LED
	WLED 	10
#endif // DO_LED

        BNE     42f                    /*@ Yes, so nothing to do*/

#ifdef DO_LED
	WLED 	11
#endif // DO_LED

        // Falling through, assume MMU disabled
        nop
//
// ------------------------------------------------------
// SRAM Controller Setup
// ------------------------------------------------------
//
        LDR     \reg1, =SRAM_CSR
        LDR     \reg2, =SRAM_CSR_VALUE
        STR     \reg2, [\reg1]

//DC test
//	WLED 	15
// 123:
//	b 123b
//DC 

        LDR     \reg1, =SRAM_BOOT_CONFIG
        LDR     \reg2, =SRAM_BOOT_CONFIG_VALUE
        STR     \reg2, [\reg1]

        LDR     \reg1, =SRAM_SLOWPORT_CONFIG
        LDR     \reg2, =SRAM_SLOWPORT_CONFIG_V
        STR     \reg2, [\reg1]

        LDR     \reg1, =SRAM_SLOW_CONFIG
        LDR     \reg2, =SRAM_SLOW_CONFIG_VALUE
        STR     \reg2, [\reg1]

//
// -------------------------------------------------------------
// SDRAM Control Setup
// -------------------------------------------------------------
//
        LDR \reg1,     =SDRAM_MEMCTL0
        LDR \reg2,     =SDRAM_MEMCTL0_VALUE
        STR \reg2,     [\reg1]

        LDR \reg1,     =SDRAM_MEMCTL1
        LDR \reg2,     =SDRAM_MEMCTL1_VALUE
        STR \reg2,     [\reg1]

        LDR \reg1,     =SDRAM_MEMINIT
        LDR \reg2,     =SDRAM_MEMINIT_VALUE
        STR \reg2,     [\reg1]

        LDR \reg1,     =SDRAM_CSR
        LDR \reg2,     =SDRAM_CSR_VALUE
        STR \reg2,     [\reg1]

// Memory autosizing code would go here..

#ifdef DO_LED
	WLED 	12
#endif // DO_LED

#if DO_INITMMU
        SA_INITMMU      \reg1, \reg2, r2, r3, r6, r7
#endif // DO_INITMMU
			
// set up the return arguments.

// SA_INITMMU is skipped and r5 is not setup
// to return DRAM_SIZE
42:	LDR	\reg3, =DRAM_SIZE

.endm


//
// ------------------------------------------------------------------
// STARTUPCODE
// -----------
// Provide code to deal with the main act of mapping the reset
// ROM away from zero (if required), and ensuring that the
// target I/O world is suitably clean before attempting to
// initialise the rest of Angel.
//
// NOTE:  do not use r4, that's $pos.
// ------------------------------------------------------------------
//

.macro STARTUPCODE       reg1, reg2, pos, ramsize

// leave $pos alone - already set - unless we have good reason
// to change it.  Here we don't.

// Bring everything out of reset except microengines.

    MOV             \reg1, #0x3F
    LDR             \reg2, =(CSR_BASE + PCI_RESET)
    STR             \reg1, [\reg2]

    // Flush Instruction Cache and Data Cache
    WRCP15_FlushITB_DTB     \reg1     // Flush ITB's + DTB's
    WRCP15_FlushIC_DC       \reg1     // Flush the ICache and DCaches
    WRCP15_DrainWriteBuffer \reg1
    NOP                             // make sure that the pipe is empty
    NOP
    NOP

    // target images can set their own clock rate, if required.
    // turn on clock switching
    WRCP15_EnableClockSW \reg1

    DISABLE_INTS    \reg1, \reg2

    INIT_RAM        \reg1, \reg2, \ramsize 

    // For unknow reason, $pos (r4) has the wrong value.
    // It dosen't keep the value __rom, which it fetched
    // at very beginning.

    LDR             \pos, =ROM_BASE

    INIT_GPIOS      \reg1, \reg2
.endm


