/*
 * platform.h -- Header for ixp1200 on Intel(R) IXP1200 Evaluation Board
 *
 * Copyright (c) 1998, 1999 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * Intel is a Registered Trademark of Intel Corporation.
 * ARM is a Registered Trademark of Advanced RISC Machines Limited.
 * Other Brands and Trademarks are the property of their respective owners.
 */

#ifndef __address_h
#define __address_h                     1

#define BIT0                            0x00000001
#define BIT1                            0x00000002
#define BIT2                            0x00000004
#define BIT3                            0x00000008
#define BIT4                            0x00000010
#define BIT5                            0x00000020
#define BIT6                            0x00000040
#define BIT7                            0x00000080
#define BIT8                            0x00000100
#define BIT9                            0x00000200
#define BIT10                           0x00000400
#define BIT11                           0x00000800
#define BIT12                           0x00001000
#define BIT13                           0x00002000
#define BIT14                           0x00004000
#define BIT15                           0x00008000
#define BIT16                           0x00010000
#define BIT17                           0x00020000
#define BIT18                           0x00040000
#define BIT19                           0x00080000
#define BIT20                           0x00100000
#define BIT21                           0x00200000
#define BIT22                           0x00400000
#define BIT23                           0x00800000
#define BIT24                           0x01000000
#define BIT25                           0x02000000
#define BIT26                           0x04000000
#define BIT27                           0x08000000
#define BIT28                           0x10000000
#define BIT29                           0x20000000
#define BIT30                           0x40000000
#define BIT31                           0x80000000

#define SZ_1K                           0x00000400
#define SZ_4K                           0x00001000
#define SZ_8K                           0x00002000
#define SZ_16K                          0x00004000
#define SZ_32K                          0x00008000
#define SZ_64K                          0x00010000
#define SZ_128K                         0x00020000
#define SZ_256K                         0x00040000
#define SZ_512K                         0x00080000

#define SZ_1M                           0x00100000
#define SZ_2M                           0x00200000
#define SZ_4M                           0x00400000
#define SZ_8M                           0x00800000
#define SZ_16M                          0x01000000
#define SZ_32M                          0x02000000
#define SZ_64M                          0x04000000
#define SZ_128M                         0x08000000
#define SZ_256M                         0x10000000
#define SZ_512M                         0x20000000

#define SZ_1G                           0x40000000

#define ALL_BITS                        0xFFFFFFFF

/* -------------------------------- */
/* SA1200 physical memory addresses */
/* -------------------------------- */

#define ROMBase                         0x00000000
#define ROMSize                         SZ_1M

#define SRAMBase                        0x10000000
//#define SRAMSize                        (SZ_512M + SZ_256M)
#define SRAMSize                        SZ_2M //DC

#define PCILocalBase                    0x42000000
#define PCILocalSize                    SZ_1M

#define PCIConfigTypeOneBase            0x52000000
#define PCIConfigTypeOneSize            SZ_32M

#define PCIConfigTypeZeroBase           0x53000000
#define PCIConfigTypeZeroSize           SZ_32M

#define PCIIOBase                       0x54000000
#define PCIIOSize                       SZ_1M

#define PCIMemBase                      0x60000000
#define PCIMemSize                      SZ_512M

#define ARMCOREBase                     0x90000000
#define ARMCORESize                     SZ_1M

#define AMBABase                        0xB0000000
#define AMBASize                        SZ_1M

#define DRAMBaseForUEng                 0xC0000000
#define DRAMBaseForUEngSize             0x01800000 /* 24Mb */

#define DRAMBase                        0xC1800000 /* Leave first 24Mb for ucode to use */
#define DRAMSize                        SZ_8M

#define DRAMPrefetchBase                0xD0000000
#define DRAMPrefetchSize                DRAMSize

#define DRAMFlushBase                   0xE0000000
#define DRAMFlushSize                   DRAMSize

#define DRAMControlBase                 0xFF000000
#define DRAMControlSize                 SZ_1M


/* ---------------------------- */
/* MMU Virtual SA1200 Addresses */
/* ---------------------------- */

#define DRAM_BASE                       0x00000000
#define DRAM_SIZE                       SZ_8M

#define ROM_BASE                        ROMBase
#define ROM_SIZE                        SZ_1M

#define SRAM_BASE                       SRAMBase
#define SRAM_SIZE                       SZ_8M


/* ================ */
/* SA1200 REGISTERS */
/* ================ */

/* -------------------------- */
/* PLL Configuration Register */
/* -------------------------- */
#define PLL_CFG                         0x90000C00

/* PLL_CFG Constants @ 3.6864 MHz */

#define PLL_CLK_29MHz                   0x00
#define PLL_CLK_36MHz                   0x01
#define PLL_CLK_44MHz                   0x02
#define PLL_CLK_51MHz                   0x03
#define PLL_CLK_58MHz                   0x04
#define PLL_CLK_66MHz                   0x05
#define PLL_CLK_73MHz                   0x06
#define PLL_CLK_81MHz                   0x07
#define PLL_CLK_88MHz                   0x08
#define PLL_CLK_95MHz                   0x09
#define PLL_CLK_103MHz                  0x0A
#define PLL_CLK_110MHz                  0x0B
#define PLL_CLK_117MHz                  0x0C
#define PLL_CLK_132MHz                  0x0D
#define PLL_CLK_147MHz                  0x0E
#define PLL_CLK_154MHz                  0x0F
#define PLL_CLK_162MHz                  0x10
#define PLL_CLK_176MHz                  0x11
#define PLL_CLK_191MHz                  0x12



/* ------------------------------ */
/* FBI Control & Status Registers */
/* ------------------------------ */
#define IREG                            0xB00401E0

/* IREG Contants */
#define UEFIQ_ENABLE                    BIT31
#define UEIRQ_ENABLE                    BIT30
#define CINTFIQ_ENABLE                  BIT29
#define CINTIRQ_ENABLE                  BIT28
#define CINT_BIT                        BIT27
#define BREAKPOINT_BIT                  BIT24

/* -------------------------- */
/* Master Interrupt Registers */
/* -------------------------- */
#define FIQ                             0x90001000
#define IRQ                             0x90001400

/* FIQ Status Constants */
#define F_SDRAM                         BIT7
#define F_RTC                           BIT6
#define F_SRAM                          BIT5
#define F_UENG                          BIT4
#define F_CINT                          BIT3
#define F_PCI                           BIT2

/* IRQ Status Constants */
#define I_SDRAM                         BIT7
#define I_UART                          BIT6
#define I_SRAM                          BIT5
#define I_UENG                          BIT4
#define I_CINT                          BIT3
#define I_PCI                           BIT2

/* ------------------------------ */
/* GPIO Enable and data Registers */
/* ------------------------------ */
#define GPIO_EN                         0x90001800
#define GPIO_DATA                       0x90001C00

/* ------------------------ */
/* Real Time Clock Register */
/* ------------------------ */
#define RTC_DIV                         0x90002000
#define RTC_TINT                        0x90002400
#define RTC_TVAL                        0x90002800
#define RTC_CNTR                        0x90002C00
#define RTC_ALM                         0x90003000

/* RTC_DIV Register Constant */
#define RTC_INT_RESET                   BIT18
#define RTC_INT_ENABLE                  BIT17
#define RTC_WRITE_ENABLE                BIT16

/* ---- */
/* UART */
/* ---- */
#define UART_SR                         0x90003400
#define UART_CR                         0x90003800
#define UART_DR                         0x90003C00

/* UART Status Register Constants */
#define UART_TX_FIFO_FULL               BIT7
#define UART_RX_FIFO_FULL               BIT6
#define UART_TX_FIFO_EMPTY              BIT5
#define UART_RX_READY                   BIT4
#define UART_RX_OVERRUN                 BIT3
#define UART_TX_READY                   BIT2
#define UART_FRAMING_ERR                BIT1
#define UART_PARITY_ERR                 BIT0

/* UART Control Register Contants */
#define UART_INT_CLEAR                  BIT9
#define UART_TX_INT_ENABLE              BIT8
#define UART_ENABLE_BIT                 BIT7
#define UART_DATA_BITS_8                (BIT6 | BIT5)
#define UART_RX_INT_ENABLE              BIT4
#define UART_STOP_BIT_1                 BIT3
#define UART_PARITY_ENABLE              BIT1
#define UART_BREAK                      BIT0

/* Clock frequency for UART */
#ifdef SA1200_A0
#define UART_CLOCK                      100000000/32
#else
#define UART_CLOCK                      3686400
#endif

#define BAUD_9600                       (((UART_CLOCK/16/9600)-1)   << 16)
#define BAUD_19200                      (((UART_CLOCK/16/19200)-1)  << 16)
#define BAUD_38400                      (((UART_CLOCK/16/38400)-1)  << 16)
#define BAUD_57600                      (((UART_CLOCK/16/57600)-1)  << 16)
#define BAUD_115200                     (((UART_CLOCK/16/115200)-1) << 16)

#define CLEAR_BAUD_RATE_FIELD           (0x3FF << 16)

/* UART Data Register with Error Status */
#define UART_DATA_FRMING_ERR            BIT10
#define UART_DATA_RX_OVERRUN            BIT9
#define UART_DATA_PARITY_ERR            BIT8

/* ---------------- */
/* PCI Control Unit */
/* ---------------- */
#define CSR_BASE                        0x42000000

#define PCI_VENDOR_ID                   0x00
#define PCI_DEVICE_ID                   0x02
#define PCI_COMMAND                     0x04
#define PCI_STATUS                      0x06
#define PCI_REV_ID                      0x08
#define PCI_CLASS_ID                    0x09
#define PCI_LATENCY                     0x0D
#define PCI_MEM_BAR                     0x10
#define PCI_CSR_BAR                     0x10
#define PCI_IO_BAR                      0x14
#define PCI_DRAM_BAR                    0x18
#define PCI_OUT_INT_STATUS              0x30
#define PCI_OUT_INT_MASK                0x34
#define PCI_INT_LINE                    0x3C
#define MAILBOX_0                       0x50
#define MAILBOX_1                       0x54
#define MAILBOX_2                       0x58
#define MAILBOX_3                       0x5C
#define DOORBELL                        0x60
#define DOORBELL_SETUP                  0x64
#define ROM_BYTE_WRITE                  0x68
#define CAP_PTR_EXT                     0x70
#define PWR_MGMT                        0x74
#define PCI_RESET                       0x7C
#define CHAN_1_BYTE_COUNT               0x80
#define CHAN_1_PCI_ADDR                 0x84
#define CHAN_1_DRAM_ADDR                0x88
#define CHAN_1_DESC_PTR                 0x8C
#define CHAN_1_CONTROL                  0x90
#define DMA_INF_MODE                    0x9C
#define CHAN_2_BYTE_COUNT               0xA0
#define CHAN_2_PCI_ADDR                 0xA4
#define CHAN_2_DRAM_ADDR                0xA8
#define CHAN_2_DESC_PTR                 0xAC
#define CHAN_2_CONTROL                  0xB0
#define CSR_BASE_ADDR_MASK              0xF8
#define CSR_BASE_ADDR_OFF               0xFC
#define DRAM_BASE_ADDR_MASK             0x100
#define DRAM_BASE_ADDR_OFF              0x104
#define ROM_BASE_ADDR_MASK              0x108
#define DRAM_TIMING                     0x10C
#define DRAM_ADDR_SIZE_0                0x110
#define DRAM_ADDR_SIZE_1                0x114
#define DRAM_ADDR_SIZE_2                0x118
#define DRAM_ADDR_SIZE_3                0x11C
#define I2O_IFH                         0x120
#define I2O_IPT                         0x124
#define I2O_OPH                         0x128
#define I2O_OFT                         0x12C
#define I2O_IFC                         0x130
#define I2O_OPC                         0x134
#define I2O_IPC                         0x138
#define SA_CONTROL                      0x13C
#define PCI_ADDR_EXT                    0x140
#define PREFETCH_RANGE                  0x144
#define PCI_ABITOR_STATUS               0x148
#define DBELL_PCI_MASK                  0x150
#define DBELL_SA_MASK                   0x154
#define IRQ_STATUS                      0x180
#define IRQ_RAW_STATUS                  0x184
#define IRQ_ENABLE                      0x188
#define IRQ_ENABLE_SET                  0x188
#define IRQ_ENABLE_CLEAR                0x18C
#define IRQ_SOFT                        0x190
#define FIQ_STATUS                      0x280
#define FIQ_RAW_STATUS                  0x284
#define FIQ_ENABLE                      0x288
#define FIQ_ENABLE_SET                  0x288
#define FIQ_ENABLE_CLEAR                0x28C
#define FIQ_SOFT                        0x290
#define TIMER_1_LOAD                    0x300
#define TIMER_1_VALUE                   0x304
#define TIMER_1_CONTROL                 0x308
#define TIMER_1_CLEAR                   0x30C
#define TIMER_2_LOAD                    0x320
#define TIMER_2_VALUE                   0x324
#define TIMER_2_CONTROL                 0x328
#define TIMER_2_CLEAR                   0x32C
#define TIMER_3_LOAD                    0x340
#define TIMER_3_VALUE                   0x344
#define TIMER_3_CONTROL                 0x348
#define TIMER_3_CLEAR                   0x34C
#define TIMER_4_LOAD                    0x360
#define TIMER_4_VALUE                   0x364
#define TIMER_4_CONTROL                 0x368
#define TIMER_4_CLEAR                   0x36C

/* ---------------------------- */
/* PCI_RESET Register Constants */
/* ---------------------------- */

#define UENG0_RST                       BIT1
#define UENG1_RST                       BIT2
#define UENG2_RST                       BIT3
#define UENG3_RST                       BIT4
#define UENG4_RST                       BIT5
#define UENG5_RST                       BIT6
#define FBI_RST                         BIT7
#define SDRAM_RST                       BIT8
#define SRAM_RST                        BIT9
#define PCI_RST                         BIT10
#define SACORE_RST                      BIT11
#define FXCMD_RST                       BIT12
#define EXT_RST                         BIT13

/* ------------------------------ */
/* ARM control register constants */
/* ------------------------------ */
#define INIT_COMPLETE                   BIT0
#define PCI_NRESET                      BIT9
#define PCI_CFN                         BIT31
#define ROM_TIMING                      0x0fff0000
#define ROM_SETUP                       0x04aa0000
#define SA_CTL_COMPLETE                 (INIT_COMPLETE | PCI_NRESET)

/* --------------- */
/* Interrupt Flags */
/* --------------- */
#define SOFT_INT                        BIT1
#define TIMER1_INT                      BIT4
#define TIMER2_INT                      BIT5
#define TIMER3_INT                      BIT6
#define TIMER4_INT                      BIT7
#define DOORBELL_INT                    BIT15
#define DMA1_INT                        BIT16
#define DMA2_INT                        BIT17
#define PCI_IRQ_L_INT                   BIT18
#define DMA1_NOT_BUSY_INT               BIT20
#define DMA2_NOT_BUSY_INT               BIT21
#define START_BIST_INT                  BIT22
#define SERR_INT                        BIT23
#define SDRAM_PARITY_INT                BIT24
#define I2O_INPOST_INT                  BIT25
#define DISCARD_TIMER_INT               BIT27
#define DATA_PARITY_INT                 BIT28
#define PCI_MASTER_ABORT_INT            BIT29
#define PCI_TARGET_ABORT_INT            BIT30
#define PCI_PARITY_INT                  BIT31

/* ------------------------------- */
/* Timer Control Register Constant */
/* ------------------------------- */
#define TIMER_ENABLE                    BIT7
#define TIMER_MODE                      BIT6

/*  timer stuff:
 *  On SA1200 there is an external clock which ticks at 3.68Mhz. This is wired
 *  into the DC21285 chip's timer #3. There are 24 bits of counter that we can
 *  set to divide this external timer down. Each external tick is 3.68*(10**-6)
 *  seconds long, 1mSEC needs counter of 3680 (every 3680 external ticks, we
 *  get one clock interrupt).
 */

#define OS_TIMER                        3

#ifdef SA1200_A0
#define mSEC_1                          100000000/1000
#else
#define mSEC_1                          162200000/1000
#endif

#define mSEC_5                          (mSEC_1 * 5)
#define mSEC_10                         (mSEC_1 * 10)
#define mSEC_100                        (mSEC_1 * 100)
#define SEC_1                           (mSEC_1 * 1000)
#define SEC_10                          (SEC_1  * 10)

/* ---------------------------------- */
/* Translation Lookaside Buffer Table */
/* ---------------------------------- */
#define TTB_SIZE                        SZ_32K

/* -------------- */
/* CoProcessor 15 */
/* -------------- */
/*  We changed IC_ON, DC_ON and WB_ON and added _BIT
 *  at the end to get rid off multiple definitions warning
 */
#define IC_ON_BIT                       BIT12
#define DC_ON_BIT                       BIT2
#define WB_ON_BIT                       BIT3


/* -------------- */
/* SRAM Registers */
/* -------------- */

#define SRAM_CSR                        0x38000000 /* SRAM CSR */
#define SRAM_SLOW_CONFIG                0x38000020 /* SRAM Flash/Mac cycle times */
#define SRAM_FLASH_CONFIG               0x38000024 /* Flash timing */
#define SRAM_MAC_CONFIG                 0x38000028 /* Mac timing */
#define SRAM_BOOT_CONFIG        	0x38000024
#define SRAM_SLOWPORT_CONFIG    	0x38000028

/* ------------------ */
/* SRAM CSR constants */
/* ------------------ */

#define SRAM_CFT                        BIT18   /* Comand FIFO Test Enable */
#define SRAM_MAC                        BIT17   /* MAC Configuration */
#define SRAM_FLWT                       BIT16   /* Flowthrough SRAM Enable */
#define SRAM_BKW                        BIT15   /* Bank Switch Wait Enable */
#define SRAM_8MF                        BIT14   /* 8MB Flash */
#define SRAM_4MF                        BIT13   /* 4MB Flash */
#define SRAM_2MF                        BIT12   /* 2MB Flash */
#define SRAM_8MSR                       BIT11   /* 8MB SRAM */
#define SRAM_4MSR                       BIT10   /* 4MB SRAM */
#define SRAM_2MSR                       BIT9    /* 2MB SRAM */
#define SRAM_1MSR                       BIT8    /* 1MB SRAM */
#define SRAM_RLAM                       BIT4    /* ASB Read Lock Achieved Mask */
#define SRAM_IRQ                        BIT3    /* IRQ Enable */
#define SRAM_FIQ                        BIT2    /* FIQ Enable */
#define SRAM_RLRS                       BIT1    /* ASB Read Lock Retry Achieved Status */
#define SRAM_RLS                        BIT0    /* ASB Read Lock Achieved Status */

#if 0
 IF :DEF: SA1200_A0
SRAM_CSR_VALUE          EQU         (SRAM_8MF + SRAM_8MSR)
SRAM_SLOW_CONFIG_VALUE  EQU         0x00000B0D
SRAM_BOOT_CONFIG_VALUE  EQU         0x090A0501
SRAM_SLOWPORT_CONFIG_V  EQU         0x0B0C0703
 ELSE
#endif
#define	SRAM_CSR_VALUE          (SRAM_8MF + SRAM_2MSR)
#define	SRAM_SLOW_CONFIG_VALUE  0x0000184A
#define	SRAM_BOOT_CONFIG_VALUE  0x18180400
#define	SRAM_SLOWPORT_CONFIG_V  0x3E440C06
/* --------------- */
/* SDRAM Registers */
/* --------------- */

#define SDRAM_CSR                       0xFF000000 /* SDRAM CSR */
#define SDRAM_MEMCTL0                   0xFF000004 /* SDRAM timing info */
#define SDRAM_MEMCTL1                   0xFF000008 /* SDRAM timing info */
#define SDRAM_MEMINIT                   0xFF00000C /* SDRAM initialization timing */

/* ------------------- */
/* SDRAM CSR Constants */
/* ------------------- */

#define SDRAM_EN_FTST                   BIT15   /* Enable FIFO RAM test */
#define SDRAM_EN_INT                    BIT10   /* Enable Parity Interrupt */
#define SDRAM_EN_IRQ                    BIT9    /* Select IRQ */
#define SDRAM_EN_FIQ                    BIT8    /* Select FIQ */
#define SDRAM_EN_CPE                    BIT6    /* Enable StrongARM Core parity error */
#define SDRAM_EN_PCIPE                  BIT5    /* Enable PCI parity error */
#define SDRAM_EN_UEPE                   BIT4    /* Enable Microengine parity error */
#define SDRAM_GENPAR                    BIT3    /* Enable parity generation */
#define SDRAM_EVENPAR                   BIT2    /* Generate even parity */
#define SDRAM_TF_BENDN                  BIT1    /* 1 = big endian mode */
#define SDRAM_INIT                      BIT0    /* Initialize SDRAM unit */
#define SDRAM_CSR_VALUE_BIT (SDRAM_INIT)

#if 0
 IF :DEF: SA1200_A0
SDRAM_CSR_VALUE         EQU         (SDRAM_INIT + SDRAM_EVENPAR)
SDRAM_MEMCTL0_VALUE     EQU         0x050082B9
SDRAM_MEMCTL1_VALUE     EQU         0x21282252
SDRAM_MEMINIT_VALUE     EQU         0x00000000
 ELSE
#endif
#define SDRAM_CSR_VALUE         0x00000001
#define SDRAM_MEMCTL0_VALUE     0x04F482C8
#define SDRAM_MEMCTL1_VALUE     0x12262252
#define SDRAM_MEMINIT_VALUE     0x00EF1FAF

/* interrupt levels */
#define INT_LVL_SOFT		1	/* soft interrupt */
#define INT_LVL_AMBARX		2	/* AMBA serial port Rx */
#define INT_LVL_AMBATX		3	/* AMBA serial port Tx */
#define INT_LVL_TIMER_1 	4	/* timer 1 */
#define INT_LVL_TIMER_2 	5	/* timer 2 */
#define INT_LVL_TIMER_3 	6	/* timer 3 */
#define INT_LVL_TIMER_4 	7	/* timer 4 */
#define	INT_LVL_IRQ_IN_0	8	/* external to 21285 */
#define	INT_LVL_IRQ_IN_1	9	/* external to 21285 */
#define	INT_LVL_IRQ_IN_2	10	/* external to 21285 */
#define	INT_LVL_IRQ_IN_3	11	/* external to 21285 */
#define INT_LVL_SPIO		12	/* Super I/O serial ports */
#define INT_LVL_DOORBELL	15	/* PCI doorbell -- Henry Qian 11/04/98 */
#define	INT_LVL_PCI		18	/* PCI external interrupts */

/*
 * IXP1200 PCI Registers.
 */
#define IXP1200_PCI_BASE                       0x42000000

#define IXP1200_PCI_REGISTER(x)                REG32_PTR(IXP1200_PCI_BASE + (x))

/*
 * PCI Control and Status Registers
 */
#define IXP1200_PCI_VEN_DEV_ID_o                0x000
#define IXP1200_PCI_CMD_STAT_o                  0x004
#define IXP1200_PCI_REV_CLASS_o                 0x008
#define IXP1200_PCI_CACHE_LAT_HB_o              0x00C
#define IXP1200_PCI_MEM_BAR_o                   0x010
#define IXP1200_PCI_IO_BAR_o                    0x014
#define IXP1200_PCI_DRAM_BAR_o                  0x018
#define IXP1200_PCI_BAR_RSV_1_o                 0x01C
#define IXP1200_PCI_BAR_RSV_2_o                 0x020
#define IXP1200_PCI_BAR_RSV_3_o                 0x024
#define IXP1200_PCI_CAP_PTR_o                   0x034
#define IXP1200_PCI_INT_LAT_o                   0x03C
#define IXP1200_PCI_CSR_BASE_ADDR_MASK_o        0x0F8
#define IXP1200_PCI_DRAM_BASE_ADDR_MASK_o       0x100
#define IXP1200_PCI_SA_CONTROL_o                0x13C
#define IXP1200_PCI_ADDR_EXT_o                  0x140
#define IXP1200_PCI_DBELL_PCI_MASK_o            0x150
#define IXP1200_PCI_DBELL_SA_MASK_o             0x154
#define IXP1200_PCI_IRQ_STATUS_o                0x180
#define IXP1200_PCI_IRQ_ENABLE_SET_o            0x188
#define IXP1200_PCI_IRQ_ENABLE_CLEAR_o          0x18C

#define IXP1200_PCI_VEN_DEV_ID                  IXP1200_PCI_REGISTER(IXP1200_PCI_VEN_DEV_ID_o)
#define IXP1200_PCI_CMD_STAT                    IXP1200_PCI_REGISTER(IXP1200_PCI_CMD_STAT_o)
#define IXP1200_PCI_REV_CLASS                   IXP1200_PCI_REGISTER(IXP1200_PCI_REV_CLASS_o)
#define IXP1200_PCI_CACHE_LAT_HB                IXP1200_PCI_REGISTER(IXP1200_PCI_CACHE_LAT_HB_o)
#define IXP1200_PCI_MEM_BAR                     IXP1200_PCI_REGISTER(IXP1200_PCI_MEM_BAR_o)
#define IXP1200_PCI_IO_BAR                      IXP1200_PCI_REGISTER(IXP1200_PCI_IO_BAR_o)
#define IXP1200_PCI_DRAM_BAR                    IXP1200_PCI_REGISTER(IXP1200_PCI_DRAM_BAR_o)
#define IXP1200_PCI_BAR_RSV_1                   IXP1200_PCI_REGISTER(IXP1200_PCI_BAR_RSV_1_o)
#define IXP1200_PCI_BAR_RSV_2                   IXP1200_PCI_REGISTER(IXP1200_PCI_BAR_RSV_2_o)
#define IXP1200_PCI_BAR_RSV_3                   IXP1200_PCI_REGISTER(IXP1200_PCI_BAR_RSV_3_o)
#define IXP1200_PCI_CAP_PTR                     IXP1200_PCI_REGISTER(IXP1200_PCI_CAP_PTR_o)
#define IXP1200_PCI_INT_LAT                     IXP1200_PCI_REGISTER(IXP1200_PCI_INT_LAT_o)
#define IXP1200_PCI_CSR_BASE_ADDR_MASK          IXP1200_PCI_REGISTER(IXP1200_PCI_CSR_BASE_ADDR_MASK_o)
#define IXP1200_PCI_DRAM_BASE_ADDR_MASK         IXP1200_PCI_REGISTER(IXP1200_PCI_DRAM_BASE_ADDR_MASK_o)
#define IXP1200_PCI_SA_CONTROL                  IXP1200_PCI_REGISTER(IXP1200_PCI_SA_CONTROL_o)
#define IXP1200_PCI_ADDR_EXT                    IXP1200_PCI_REGISTER(IXP1200_PCI_ADDR_EXT_o)
#define IXP1200_PCI_DBELL_PCI_MASK              IXP1200_PCI_REGISTER(IXP1200_PCI_DBELL_PCI_MASK_o)
#define IXP1200_PCI_DBELL_SA_MASK               IXP1200_PCI_REGISTER(IXP1200_PCI_DBELL_SA_MASK_o)
#define IXP1200_PCI_IRQ_STATUS                  IXP1200_PCI_REGISTER(IXP1200_PCI_IRQ_STATUS_o)
#define IXP1200_PCI_IRQ_ENABLE                  IXP1200_PCI_REGISTER(IXP1200_PCI_IRQ_ENABLE_o)
#define IXP1200_PCI_IRQ_ENABLE_SET              IXP1200_PCI_REGISTER(IXP1200_PCI_IRQ_ENABLE_SET_o)
#define IXP1200_PCI_IRQ_ENABLE_CLEAR            IXP1200_PCI_REGISTER(IXP1200_PCI_IRQ_ENABLE_CLEAR_o)

 /* constants for XP1200 PCI registers */
#define IXP1200_PCI_SA_CONTROL_PNR              BIT9  /* bit 9 */
#define IXP1200_PCI_SA_CONTROL_PCF0             BIT31 /* bit 31 */

#define IXP1200_SDRAM_SA_PHY_START              0xC0000000  /* SDRAM starting physical address */
#define IXP1200EB_ACTUAL_SDRAM_SIZE             0x02000000  /* 32 MB SDRAM on evaluation board */

#define IXP1200_PCI_IRQ_PIL_SET                 BIT18
#define IXP1200_PCI_IRQ_PIL_CLEAR               BIT18

#define IXP1200_IRQ_STATUSX    	REG32_PTR(0x90001400)

#define IXP1200_UART_CR    	REG32_PTR(0x90003800)

/* definition of system timer */
#define IXP1200_TIMER_BASE              0x42000000
#define IXP1200_TIMER_REGISTER(x)	REG32_PTR(IXP1200_TIMER_BASE + (x))

#define IXP1200_TIMER_1_LOAD_o                          0x300
#define IXP1200_TIMER_1_VALUE_o                         0x304
#define IXP1200_TIMER_1_CONTROL_o                       0x308
#define IXP1200_TIMER_1_CLEAR_o                         0x30C
#define IXP1200_TIMER_2_LOAD_o                          0x320 
#define IXP1200_TIMER_2_VALUE_o                         0x324
#define IXP1200_TIMER_2_CONTROL_o                       0x328
#define IXP1200_TIMER_2_CLEAR_o                         0x32C
#define IXP1200_TIMER_3_LOAD_o                          0x340
#define IXP1200_TIMER_3_VALUE_o                         0x344
#define IXP1200_TIMER_3_CONTROL_o                       0x348
#define IXP1200_TIMER_3_CLEAR_o                         0x34C
#define IXP1200_TIMER_4_LOAD_o                          0x360
#define IXP1200_TIMER_4_VALUE_o                         0x364
#define IXP1200_TIMER_4_CONTROL_o                       0x368
#define IXP1200_TIMER_4_CLEAR_o                         0x36C

#define IXP1200_TIMER_1_LOAD	IXP1200_TIMER_REGISTER(IXP1200_TIMER_1_LOAD_o)
#define IXP1200_TIMER_1_VALUE   IXP1200_TIMER_REGISTER(IXP1200_TIMER_1_VALUE_o)
#define IXP1200_TIMER_1_CONTROL IXP1200_TIMER_REGISTER(IXP1200_TIMER_1_CONTROL_o)
#define IXP1200_TIMER_1_CLEAR   IXP1200_TIMER_REGISTER(IXP1200_TIMER_1_CLEAR_o)
#define IXP1200_TIMER_2_LOAD    IXP1200_TIMER_REGISTER(IXP1200_TIMER_2_LOAD_o)
#define IXP1200_TIMER_2_VALUE   IXP1200_TIMER_REGISTER(IXP1200_TIMER_2_VALUE_o)
#define IXP1200_TIMER_2_CONTROL IXP1200_TIMER_REGISTER(IXP1200_TIMER_2_CONTROL_o)
#define IXP1200_TIMER_2_CLEAR   IXP1200_TIMER_REGISTER(IXP1200_TIMER_2_CLEAR_o)
#define IXP1200_TIMER_3_LOAD    IXP1200_TIMER_REGISTER(IXP1200_TIMER_3_LOAD_o)
#define IXP1200_TIMER_3_VALUE   IXP1200_TIMER_REGISTER(IXP1200_TIMER_3_VALUE_o)
#define IXP1200_TIMER_3_CONTROL IXP1200_TIMER_REGISTER(IXP1200_TIMER_3_CONTROL_o)
#define IXP1200_TIMER_3_CLEAR   IXP1200_TIMER_REGISTER(IXP1200_TIMER_3_CLEAR_o)
#define IXP1200_TIMER_4_LOAD    IXP1200_TIMER_REGISTER(IXP1200_TIMER_4_LOAD_o)
#define IXP1200_TIMER_4_VALUE   IXP1200_TIMER_REGISTER(IXP1200_TIMER_4_VALUE_o)
#define IXP1200_TIMER_4_CONTROL IXP1200_TIMER_REGISTER(IXP1200_TIMER_4_CONTROL_o)
#define IXP1200_TIMER_4_CLEAR   IXP1200_TIMER_REGISTER(IXP1200_TIMER_4_CLEAR_o)

#define IXP1200_PCI_IRQ_SYSTEM_TIMER_SET        BIT4
#define IXP1200_TIMER_CONTROL_ENABLE            BIT7
#define IXP1200_TIMER_CONTROL_PERIODIC          BIT6

#define IXP1200_PCI_IO_ACCESS_BASE   0x54000000 /* I/O cycle space, mmu off */
#define IXP1200_PCI_MEM_ACCESS_BASE  0x60000000 /* Mem cycle space, mmu off */

/* IXP1200EB_SDRAM_SIZE is the SDRAM size in IXP1200 eval. board */
#define IXP1200EB_SDRAM_SIZE    0x02000000 	/* 32 MB */
#define IXP1200EB_CSR_MEM_SIZE	0x00100000	/* 1 MB CSR memory space */
#define IXP1200EB_CSR_IO_SIZE	0x00000080	/* 128 bytes CSR I/O space */

#define IXP1200_PCI_TYPE_0_CONFIG_BASE 0x53000000
#define IXP1200_PCI_TYPE_1_CONFIG_BASE 0x52000000

#define IXP1200_PCI_INT_VEC	18 	/* pci_irq_1 */

#define IXP1200EB_SRAM_BASE	0x10000000	
#define IXP1200EB_SRAM_SIZE	0x00200000	//2 MB
#define  CYGMON_VECTORS_SA	(IXP1200EB_SRAM_BASE + IXP1200EB_SRAM_SIZE - 64)	
#define  CYGMON_ROM_SA		0x00140000

/*
 * IXP1200 Cygmon related flash ROM parameters
 * As discussed with James Guilford in Hudson, 512 bytes of flash ROM starting from
 * 0x8400 are reserved for cygmon's use, which belongs to part of Boot Parameters 
 * block (from 0x8000 to 0x9FFFF)
 */
#define ASSIGNED_MANUALLY	0  
#define ASSIGNED_BY_BOOTP	1

#define IXPFLASH_PARAM_BLOCK_START_ADDR	0x00008000
#define IXPFLASH_PARAM_BLOCK_SIZE	0x00004000	//in bytes
#define IXPFLASH_CYGMON_PARAM_OFFSET	0x0400 //flash rom parameters for 
                                               //cygmon starting from 0x8400
#define FLASH_VALID	0x5f7684a3

#define DEFAULT_LINUX_BOOT_COUNTDOWN	1


#define IXPFLASH_CYG_PAR_BASE                  (IXPFLASH_PARAM_BLOCK_START_ADDR	+ IXPFLASH_CYGMON_PARAM_OFFSET) 

#define IXPFLASH_CYG_PAR_REGISTER(x)           REG32_PTR(IXPFLASH_CYG_PAR_BASE + (x))


/*
 * Cygmon Related Parameters on Flash ROM
 */
#define FLASH_VALID_CHECK_o                     0x000
#define IP_ASSIGN_METHOD_o			0x004
#define LOCAL_MACHINE_IP_o			0x008
#define REMOTE_SERVER_IP_o			0x00C
#define LINUX_KERNEL_FILE_o                     0x010
#define LINUX_RAMDISK_FILE_o                    0x060
#define LINUX_CMD_LINE_o                        0x0B0
#define LINUX_BOOT_COUNTDOWN_o                  0x100
#define LINUX_KERNEL_LOAD_OFFSET_o              0x104

#define FLASH_VALID_CHECK			IXPFLASH_CYG_PAR_REGISTER(FLASH_VALID_CHECK_o)
#define IP_ASSIGN_METHOD			IXPFLASH_CYG_PAR_REGISTER(IP_ASSIGN_METHOD_o)
#define LOCAL_MACHINE_IP 			IXPFLASH_CYG_PAR_REGISTER(LOCAL_MACHINE_IP_o)
#define REMOTE_SERVER_IP			IXPFLASH_CYG_PAR_REGISTER(REMOTE_SERVER_IP_o)

#define LINUX_KERNEL_FILE			IXPFLASH_CYG_PAR_REGISTER(LINUX_KERNEL_FILE_o)
#define LINUX_RAMDISK_FILE			IXPFLASH_CYG_PAR_REGISTER(LINUX_RAMDISK_FILE_o)
#define LINUX_CMD_LINE          		IXPFLASH_CYG_PAR_REGISTER(LINUX_CMD_LINE_o)

#define LINUX_BOOT_COUNTDOWN			IXPFLASH_CYG_PAR_REGISTER(LINUX_BOOT_COUNTDOWN_o)

#define LINUX_KERNEL_LOAD_OFFSET		IXPFLASH_CYG_PAR_REGISTER(LINUX_KERNEL_LOAD_OFFSET_o)

//definition for laoding address for Linux kernel and ramdisk
//#define LINUXKERNELLOADADDRESS	0xC0008000	//release 2.00, 2.01 loc for kernel
//#define LINUXRAMDISKLOADADDRESS	0xC0800000	//release 2.00, 2.01 loc. for ramdisk

//#define LINUXKERNELLOADADDRESS	0xC1008000	//release 2.02 loc. for kernel - make debugger more stable
//#define LINUXRAMDISKLOADADDRESS	0xC1800000	//release 2.02 loc. for ramdisk - make debugger more stable

#define DEFAULT_LINUXKERNELLOADADDRESS	0xC0808000	//release 2.04 loc. for kernel - make debugger more stable
//#define LINUXRAMDISKLOADADDRESS	0xC1000000	//release 2.02 loc. for ramdisk - make debugger more stable

#define MINIMUM_LINUX_KERNEL_LOAD_OFFSET 0XC0200000

//choose a location for temp store linux boot address
#define TMP_STORE_AT_SDRAM_FOR_BOOT_LINUX  (SRAMBase + SZ_256K -16)

#endif /* __address_h */

/* end of file */

