/*
 * ixp1200eb.c -- Support for Intel(R) IXP1200 Evaluation Board
 *
 * Copyright (c) 2000, 2001 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * Intel is a Registered Trademark of Intel Corporation.
 * ARM is a Registered Trademark of Advanced RISC Machines Limited.
 * Other Brands and Trademarks are the property of their respective owners.
 */

#include __BOARD_HEADER__
#include <bsp_if.h>
#include <bsp/platform.h>
#include <bsp/pci_cfg.h>
#include "net.h"

/* #define OUTPUT_DEBUG_INFO	1 */

extern int is_ixp1200_central_func;

void write_led (unsigned char );

/*
 * Array of memory region descriptors. We just list RAM and FLASH.
 */
struct bsp_mem_info _bsp_memory_list[] =
{
/* NOTE: Original ebsa
*/
    { (void *)0x00000000, (void *)0x00000000, 0, 0, BSP_MEM_RAM }
/*    {(void *)SRAMBase,   (void *)SRAMBase,   0, SRAMSize,   BSP_MEM_RAM  },
*/
};

/*
 * Number of memory region descriptors.
 */
int _bsp_num_mem_regions = sizeof(_bsp_memory_list)/sizeof(_bsp_memory_list[0]);

extern struct bsp_comm_procs * __enet_link_init(enet_addr_t enet_addr); /*DC enet support */

/*
 * Toggle LED for debugging purposes.
 */
void flash_led(int n, int which)
{
    unsigned char *led_addr;

    led_addr= (unsigned char *) 0x38508000;
    *led_addr = (unsigned char) n;
}

/*
 * Early initialization of comm channels. Must not rely
 * on interrupts, yet. Interrupt operation can be enabled
 * in _bsp_board_init().
 */
void
_bsp_init_board_comm(void)
{
    extern void _bsp_init_ixp1200_comm(void);
    _bsp_init_ixp1200_comm();
}


/*
 * Set any board specific debug traps.
 */
void
_bsp_install_board_debug_traps(void)
{
}


/*
 * Install any board specific interrupt controllers.
 */
void
_bsp_install_board_irq_controllers(void)
{
}

/*
 *  Board specific BSP initialization.
 */
void
_bsp_board_init(void)
{
#ifdef OUTPUT_DEBUG_INFO
     /* write_led(0xC0); */
     bsp_printf(" Enter bsp_board_init\n");
#endif /* #ifdef OUTPUT_DEBUG_INFO */

    /*
     * Override default platform info.
     */
    _bsp_platform_info.board = "Intel(R) IXP1200 Evaluation Board";

    /*
     * Finish setup of RAM description. Early initialization put a
     * pointer to the top of RAM in _bsp_ram_info_ptr.
     */
    _bsp_memory_list[0].nbytes = (long)_bsp_ram_info_ptr;

    /* following for PCI configuration */

    check_central_func_init(); /* check whether IXP1200 configured as central
                                  function, if it is, do initialization */

    if (is_ixp1200_central_func)
    {
   	 /* start ixp1200 pci configuration */
  	 ixp1200_pci_config();
    }

#ifdef DEBUG_PRINT_INFO 
    if (is_ixp1200_central_func)
    {
	bsp_printf(" Display Address base for IXP1200:\n");
	bsp_printf(" BA0(10h)=%8.8X\n", *IXP1200_PCI_MEM_BASE);
	bsp_printf(" BA1(14h)=%8.8X\n", *IXP1200_PCI_IO_BAR);
	bsp_printf(" BA2(18h)=%8.8X\n", *IXP1200_PCI_DRAM_BAR);
	bsp_printf("Reach end of _bsp_board_init.\n");
    }
#endif   /* #ifdef DEBUG_PRINT_INFO */
}

/*
 * Initialize the mmu and Page Tables
 * The MMU is actually turned on by the caller of this function.
 */
/*void
ORIG*/
unsigned long *
_bsp_mmu_init(int sdram_size)
{
    unsigned long ttb_base = ((unsigned long)&page1);
    unsigned long i;
    /* unsigned long processor_id; */
    unsigned long dest_reg = 8;		// Destination register #

    return (unsigned long *)(SRAMBase + SRAMSize-0x1000);

    __mrc(ARM_CACHE_COPROCESSOR_NUM,
          ARM_COPROCESSOR_OPCODE_DONT_CARE,
          dest_reg,
          ARM_ID_REGISTER,
          ARM_COPROCESSOR_RM_DONT_CARE,
          ARM_COPROCESSOR_OPCODE_DONT_CARE);

    write_led ((dest_reg >> 0) & 0xff);

    write_led (0x16);

    return (unsigned long *)(SRAMBase + SRAMSize-0x1000);

    BSP_ASSERT((ttb_base & ARM_TRANSLATION_TABLE_MASK) == ttb_base);

    /*
     * Set the TTB register
     */
    __mcr(ARM_CACHE_COPROCESSOR_NUM,
          ARM_COPROCESSOR_OPCODE_DONT_CARE,
          ttb_base,
          ARM_TRANSLATION_TABLE_BASE_REGISTER,
          ARM_COPROCESSOR_RM_DONT_CARE,
          ARM_COPROCESSOR_OPCODE_DONT_CARE);

    /*
     * Set the Domain Access Control Register
     */
    i = ARM_ACCESS_TYPE_MANAGER(0)    |
        ARM_ACCESS_TYPE_NO_ACCESS(1)  |
        ARM_ACCESS_TYPE_NO_ACCESS(2)  |
        ARM_ACCESS_TYPE_NO_ACCESS(3)  |
        ARM_ACCESS_TYPE_NO_ACCESS(4)  |
        ARM_ACCESS_TYPE_NO_ACCESS(5)  |
        ARM_ACCESS_TYPE_NO_ACCESS(6)  |
        ARM_ACCESS_TYPE_NO_ACCESS(7)  |
        ARM_ACCESS_TYPE_NO_ACCESS(8)  |
        ARM_ACCESS_TYPE_NO_ACCESS(9)  |
        ARM_ACCESS_TYPE_NO_ACCESS(10) |
        ARM_ACCESS_TYPE_NO_ACCESS(11) |
        ARM_ACCESS_TYPE_NO_ACCESS(12) |
        ARM_ACCESS_TYPE_NO_ACCESS(13) |
        ARM_ACCESS_TYPE_NO_ACCESS(14) |
        ARM_ACCESS_TYPE_NO_ACCESS(15);
    __mcr(ARM_CACHE_COPROCESSOR_NUM,
          ARM_COPROCESSOR_OPCODE_DONT_CARE,
          i,
          ARM_DOMAIN_ACCESS_CONTROL_REGISTER,
          ARM_COPROCESSOR_RM_DONT_CARE,
          ARM_COPROCESSOR_OPCODE_DONT_CARE);

/*  Returned here because of a crash
 * if kermit compiled in
 */
    write_led (0x19);
    return (unsigned long *)(SRAMBase + SRAMSize-0x1000);
    /*
     * First clear all TT entries - ie Set them to Faulting
     */
    memset((void *)ttb_base, 0, ARM_FIRST_LEVEL_PAGE_TABLE_SIZE);

    /*
     * We only do direct mapping for the EBSA board. That is, all
     * virt_addr == phys_addr.
     */

    /*
     * Actual Base = 0x000(00000)
     * Virtual Base = 0x000(00000)
     * Size = lM
     * Flash
     */
        ARM_MMU_SECTION(ttb_base, 0x000, 0x000,
                        ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                        ARM_ACCESS_PERM_RW_RW);

    write_led (0x20);
    /*
     * Actual Base =  0x100(00000) - 0x107(FFFFF)
     * Virtual Base = 0x100(00000) - 0x107(FFFFF)
     * Size = 8M
     * SRAM
     */
    for (i = 0x100; i <= 0x107; i++) {
    	ARM_MMU_SECTION(ttb_base, i, i,
		    ARM_CACHEABLE, ARM_UNBUFFERABLE,
		    ARM_ACCESS_PERM_RW_RW);

    }
    write_led (0x21);
    /*
     * Actual Base = 0x420(00000) - 0x420(FFFFF)
     * Virtual Base = 0x420(00000) - 0x420(FFFFF)
     * Size = 1M
     * PCILOcalBase
     */
        ARM_MMU_SECTION(ttb_base, 0x420, 0x420,
                        ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                        ARM_ACCESS_PERM_RW_RW);


    write_led (0x22);
    /*
     * Actual Base = 0x520(00000) - 0x52F(FFFFF)
     * Virtual Base = 0x520(00000) - 0x52F(FFFFF)
     * Size = 1M
     * PCIConfigTypeOneBase
     */
	for (i=0x520;i<=0x52F;i++)
	{
        	ARM_MMU_SECTION(ttb_base, i, i,
                        ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                        ARM_ACCESS_PERM_RW_RW);
	}

    write_led (0x23);

    /*
     * Actual Base = 0x530(00000) - 0x53F(FFFFF)
     * Virtual Base = 0x530(00000) - 0x53F(FFFFF)
     * Size = 1M
     * PCIConfigTypeZeroBase
     */
	for (i=0x530;i<=0x53F;i++)
	{
        	ARM_MMU_SECTION(ttb_base, i, i,
                        ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                        ARM_ACCESS_PERM_RW_RW);
	}

    write_led (0x24);

    /*
     * Actual Base = 0x540(00000) - 0x540(FFFFF)
     * Virtual Base = 0x540(00000) - 0x540(FFFFF)
     * Size = 1M
     * PCIConfigIO
     */
        	ARM_MMU_SECTION(ttb_base, 0x540, 0x540,
                        ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                        ARM_ACCESS_PERM_RW_RW);

    write_led (0x25);

    /*
     * Actual Base = 0x600(00000) - 0x0x7FF(FFFFF)
     * Virtual Base = 0x600(00000) - 0x0x7FF(FFFFF)
     * Size = 512M
     * PCIMemory
     */
	for (i=0x600;i<=0x7FF;i++)
	{
        	ARM_MMU_SECTION(ttb_base, i, i,
                        ARM_UNCACHEABLE, ARM_BUFFERABLE,
                        ARM_ACCESS_PERM_RW_RW);
	}

    write_led (0x26);

    /*
     * Actual Base = 0x900(00000)
     * Virtual Base = 0x900(00000)
     * Size = 1M
     * ARM Core
     */
    ARM_MMU_SECTION(ttb_base, 0x900, 0x900,
                    ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                    ARM_ACCESS_PERM_RW_RW);

    write_led (0x27);
    /*
     * Actual Base = 0xB00(00000)-0xB00(FFFFF)
     * Virtual Base = 0xB00(00000)-0xB00(FFFFF)
     * Size = 1M
     * AMBA Base
     */
        ARM_MMU_SECTION(ttb_base, 0xB00, 0xB00,
                        ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                        ARM_ACCESS_PERM_RW_RW);

    write_led (0x28);
    /*
     * Actual Base = 0xA00(00000)-0xA00(04000)
     * Virtual Base = 0xA00(00000)-0xA00(04000)
     * Size = 16Kbytes
     * Cache Flush Area
     */
        ARM_MMU_SECTION(ttb_base, 0XA00, 0XA00,
                        ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                        ARM_ACCESS_PERM_RW_RW);
	write_led(0x29);

    /*
     * Actual Base = 0xC00(00000)-0xCFF(FFFFF)
     * Virtual Base = 0xC00(00000)-0xCFF(FFFFF)
     * Size = 256Mbytes
     * SDRAM Non-prefetch Memory
     */
    for (i = 0xC00; i <= 0xCFF; i++) {
        ARM_MMU_SECTION(ttb_base, i, i,
                        ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                        ARM_ACCESS_PERM_RW_RW);
    }

    /*
     * Actual Base = 0xD00(00000) - 0xDFF(FFFFF)
     * Virtual Base = 0xD00(00000) - 0xDFF(FFFFF)
     * Size = 256 Mbytes
     * SDRAM Prefetch Memory
     */
    for (i = 0xD00; i <= 0xDFF; i++) {
        ARM_MMU_SECTION(ttb_base, i, i,
                        ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                        ARM_ACCESS_PERM_RW_RW);
    }
    /*
     * Actual Base = 0xFF0(00000) - 0xFF0(00014)
     * Virtual Base = 0xFF0(00000) - 0xFF0(00004)
     * Size = 20 bytes
     * SDRAM Control Registers
     */
        ARM_MMU_SECTION(ttb_base, 0XFF0, 0XFF0,
                        ARM_UNCACHEABLE, ARM_UNBUFFERABLE,
                        ARM_ACCESS_PERM_RW_RW);
	write_led(0x16);

	/*Virtual and physical addr are mapped to be same */
    return (unsigned long *)(SRAMBase + SRAMSize-0x1000);
}


