/*
 * start.S -- ROM-based startup for ARM(R)
 *
 * Copyright (c) 1998, 1999 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * ARM is a Registered Trademark of Advanced RISC Machines Limited.
 * Other Brands and Trademarks are the property of their respective owners.
 */

//following #define COPYTO_AND_RUNIN_SRAM is necessary to copy cygmon image from
//flash ROM to SRAM and run in SRAM, so bootoptions can save some Linux auto-boot
//options in flash ROM 
#define COPYTO_AND_RUNIN_SRAM	1 

//following #define LOAD_LINUX is necessary for dwonload Linux kernel and
//ramdisk utilities to IXP1200 and boot Linux, but if it is defined, then 
//gdb will not work properly, since current cygmon's main function is loading
//and booting Linux, so we keep it defined.
#define LOAD_LINUX

//following #define IN_FLASH_WITH_BOOTMGR  will generate an cygmon image
//with ROM starting address specified in CYGMON_ROM_SA (currently 0x00140000)
//in ixp120eb/include/bsp/platform.h file, this cygmon image will be merged 
//into a biary file containing boot manager, flash utility, diagnostics,
//uCOS angel, and VxWork boot. The user can choose which program to run 
//through boot manager. Since we plane to ship cygmon with boot manager,
//do keep it defined. There is stiill some bug, so we need to reload
//SRAM_VECTORS before initialize_mon() in monior.c n cygmon directory.  
#define IN_FLASH_WITH_BOOTMGR	1

#include <bsp/cpu.h>
#include <bsp/platform.h>
#include <bsp/ixp1200macros.S>

        .file   "start.S"
	
	/*
	 * If USE_AIF_HEADER is defined, prepend an ARM(R) AIF header to the ROM image
	 * so that the Cygnus BSP can co-exist with ARM(R) bootloader. It will also be
	 * necessary to modify the link address as the bootloader will occupy the
	 * the default bsp link address.
	 */
#if defined(USE_AIF_HEADER)
	/* These two symbols are created in the linker script */
	.extern SYM_NAME(_ROSize)
	.extern SYM_NAME(_RWSize)
	
	.section ".aif_header","ax"
	nop
	nop
	nop
	.word 0
	nop
	.word SYM_NAME(_ROSize) - 0x80
	.word SYM_NAME(_RWSize)
	.word 0
	.word 0
	.word 0
	.word SYM_NAME(aif_hdr_end) + 0x40
	.p2align 7
SYM_NAME(aif_hdr_end):
#endif

        .text
FUNC_START start

#ifdef COPYTO_AND_RUNIN_SRAM 
        //in order to copy cygmone image to SRAM, set correct SRAM_CSR value first
        LDR     R8, = SRAM_CSR_VALUE    //SRAM_CSR_VALUE
  	LDR     R9, = SRAM_CSR		//SRAM_CSR
 	LDR     R1, = CYGMON_ROM_SA 	//cygmon image start address in flash ROM
 	ADD     R1, R1, #32
        //following code copy 256K image from flash ROM to begining of SRAM
	STR	R8, [R9]
  	LDR     R8, = CYGMON_ROM_SA	//cygmon image start address in flash ROM
        LDR     R7, = (CYGMON_ROM_SA + SZ_256K)	//assume cygmon image occupied upto 256KB
  	LDR     R9, = SRAM_BASE 	//SRAM start address
        CMP     R7, R8
        BLE     62
	LDR	R0, [R8]
	STR	R0, [R9]
        ADD     R8, R8, #4
        ADD     R9, R9, #4
        MOV     PC, R1
62:
        LDR     R1, = 0x0              //reset R6
        LDR     R7, = 0x0              //reset R7
#endif //#ifdef COPYTO_AND_RUNIN_SRAM
/* PVU Put exception vectors as the memory is mapped directly */
	b start2
#ifndef IN_FLASH_WITH_BOOTMGR
	ldr     pc, SYM_NAME(ROM_UNDEFINED_ISR)
	ldr     pc, SYM_NAME(ROM_SWI_ISR)
	ldr     pc, SYM_NAME(ROM_PREFETCH_ABORT_ISR)
	ldr     pc, SYM_NAME(ROM_DATA_ABORT_ISR)
	ldr     pc, SYM_NAME(ROM_ADDR_ERROR_ISR)
	ldr     pc, SYM_NAME(ROM_IRQ_ISR)
	ldr     pc, SYM_NAME(ROM_FIQ_ISR)
#endif //#ifndef IN_FLASH_WITH_BOOTMGR
FUNC_END start

FUNC_START start2

	/* 
	 * This nop seems necessary when using a PROMJet ROM emulator
	 * on IXP1200 boards. It does no harm, so leave it in
	 * for all boards.
	 */
	nop

#ifndef __LITTLE_ENDIAN__ 
        SET_BIG_ENDIAN             // Switch to big endian mode
#endif /* #ifndef __LITTLE_ENDIAN__ */
        /*
         * Careful.  There is no memory yet.
         */

// PLL initialization here

        ldr     r0, =PLL_CFG           /*get pointer to PLL control Reg*/
        ldr     r1, =PLL_CLK_162MHz    /*; get value for 88 MHz */
        str     r1, [r0]              /*; set speed */

        /*
         * Setup the initial exception stack
         */
/*PVU*/
#if DO_LED
        WLED     #0x0B	
#endif // DO_LED

/*PVU*/

        mrs	r14, cpsr		// get processor status
        bic	r14, r14, #ARM_PSR_MODE_MASK
        orr	r14, r14, #(ARM_PSR_IRQ | ARM_PSR_FIQ | ARM_PSR_MODE_SVC)
        msr	cpsr, r14

        //
        // state: SVC mode; IRQs disabled; FIQs disabled.
        // r14 = current PSR
        //  r4 = PowerOnStatus [input] (zero - hard; non-zero - soft).
        //  r5 = RAM size, or 0x00000000 if not yet known.
        //  r0, r1 used as temporary working space within STARTUPCODE
        //
        //        All other registers undefined.
        //        If present, MMU initialised to default off state.
        //
        // The following macro should perform any target specific I/O
        // disabling and setup. i.e. disable all interrupt sources, set
        // memory access timings, etc.

        MOV     r4, #0x00	// r4 used to indicate type of start
				// 0 = power up
        MOV     r5, #0x02	// set r5 to 0 unknown RAM size

        STARTUPCODE	r0, r1, r4, r5

#ifdef COPYTO_AND_RUNIN_SRAM 
        //due to something left by boot manager, need to clean following SRAM to
        //make cygmon run properly
 	LDR     R8, = (SRAM_BASE + SZ_256K)	//just after copied 256K cygmon image
7:
	LDR	R9, = (SRAM_BASE + SZ_512K)	//clean up to the top of stack 
	LDR	R0, = 0x00000000  
	STR	R0,[R8]
        ADD     R8, R8, #4
        CMP     R8, R9
        BLE     7b
#endif //#ifdef COPYTO_AND_RUNIN_SRAM 

#ifdef IN_FLASH_WITH_BOOTMGR
	//DC to keep 8 vectors, the first locaion is for RESET
        //exception, it really will be handled by bootmgr, call our
        //start function, so put NOP here
	LDR	R8, = (CYGMON_VECTORS_SA + 0x00)  
	LDR     R9, = 0xE1A00000	//NOP
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x04)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x08)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x0C)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x10)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x14)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x18)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x1C)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	//DC to keep 8 vectors, the locaion is for RESET
        //exception, it really will be handled by bootmgr, call our
        //start function, so put NOP here
	LDR	R8, = (CYGMON_VECTORS_SA + 0x20) //DC corresponding
	LDR     R9, = 0xE1A00000	//NOP
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x24)
	LDR     R9, SYM_NAME(ROM_UNDEFINED_ISR)
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x28)
	LDR     R9, SYM_NAME(ROM_SWI_ISR)
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x2C)
	LDR     R9, SYM_NAME(ROM_PREFETCH_ABORT_ISR)
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x30)
	LDR     R9, SYM_NAME(ROM_DATA_ABORT_ISR)
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x34)
	LDR     R9, SYM_NAME(ROM_ADDR_ERROR_ISR)
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x38)
	LDR     R9, SYM_NAME(ROM_IRQ_ISR)
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x3C)
	LDR     R9, SYM_NAME(ROM_FIQ_ISR)
	STR	R9,[R8]
#endif //#ifdef IN_FLASH_WITH_BOOTMGR

        ldr     sp, =SYM_NAME(_ex_stack)

        /*
         * Switch to System mode
         */
/* 
 * We want to remain in SERVICE mode to load linux
 */

#ifndef LOAD_LINUX
	ldr	r0,=(ARM_PSR_IRQ | ARM_PSR_FIQ | ARM_PSR_MODE_SYSTEM)
	msr	cpsr, r0

#if 0
        /*
         * Test if the lr works in system mode.
         * It appeared not to work on the AEB-1
         */
        ldr     lr, =(0x4)
        mov     r0, lr
        ldr     r1, =(0x4)
        cmps    r0, r1
0:      bne     0b
/*ORIG
        PORT_TOGGLE_DEBUG
*/
#endif // #if 0
#endif // LOAD_LINUX

        /*
         * We manually put the return address into the lr in case
         * _bsp_init_stack modifies the memory layout and invalidates
         * the current map
         */

#if DO_LED
        WLED	0x64
#endif // DO_LED

        ldr     lr, =SYM_NAME(c_start)
        b       SYM_NAME(_bsp_init_stack)

//
// We should never reach this point
//
        MOV     r0, #0x65
        LDR     r1, =0x38508000
        STR     r0, [r1]
1:	b	1b

FUNC_END start2

FUNC_START load_sram_vectors
#ifdef IN_FLASH_WITH_BOOTMGR
/* DC fillout nessary non-ROM vectors information starting from CYGMON_VECTORS_SA
   to CYGMON_VECTORS_SA + 0x37, total 14 entries */
   
	//DC to keep 8 vectors, the first locaion is for RESET
        //exception, it really will hadled by bootmgr, call our
        //start function, so put NOP here
	LDR	R8, = (CYGMON_VECTORS_SA + 0x00)  
	LDR     R9, = 0xE1A00000	//NOP
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x04)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x08)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x0C)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x10)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x14)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x18)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x1C)
	LDR     R9, = 0xE59FF018
	STR	R9,[R8]

	//DC to keep 8 vectors, the locaion is for RESET
        //exception, it really will hadled by bootmgr, call our
        //start function, so put NOP here
	LDR	R8, = (CYGMON_VECTORS_SA + 0x20) //DC corresponding
	LDR     R9, = 0xE1A00000	//NOP
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x24)
	LDR     R9, SYM_NAME(ROM_UNDEFINED_ISR)
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x28)
	LDR     R9, SYM_NAME(ROM_SWI_ISR)
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x2C)
	LDR     R9, SYM_NAME(ROM_PREFETCH_ABORT_ISR)
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x30)
	LDR     R9, SYM_NAME(ROM_DATA_ABORT_ISR)
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x34)
	LDR     R9, SYM_NAME(ROM_ADDR_ERROR_ISR)
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x38)
	LDR     R9, SYM_NAME(ROM_IRQ_ISR)
	STR	R9,[R8]

	LDR	R8, = (CYGMON_VECTORS_SA + 0x3C)
	LDR     R9, SYM_NAME(ROM_FIQ_ISR)
	STR	R9,[R8]
#endif //#ifdef IN_FLASH_WITH_BOOTMGR
	mov	pc, lr
FUNC_END load_sram_vectors

FUNC_START change_pc_to_tmp_store
        LDR     R8, = TMP_STORE_AT_SDRAM_FOR_BOOT_LINUX
        LDR	R0, [R8]
        MOV     PC, R0
FUNC_END change_pc_to_tmp_store
        
SYM_NAME(ROM_UNDEFINED_ISR):      .word   SYM_NAME(undefined_isr_asm)
SYM_NAME(ROM_SWI_ISR):            .word   SYM_NAME(swi_isr_asm)
SYM_NAME(ROM_PREFETCH_ABORT_ISR): .word   SYM_NAME(prefetch_abort_isr_asm)
SYM_NAME(ROM_DATA_ABORT_ISR):     .word   SYM_NAME(data_abort_isr_asm)
SYM_NAME(ROM_ADDR_ERROR_ISR):     .word   SYM_NAME(address_err_isr_asm)
SYM_NAME(ROM_IRQ_ISR):            .word   SYM_NAME(irq_isr_asm)
SYM_NAME(ROM_FIQ_ISR):            .word   SYM_NAME(fiq_isr_asm)

#ifdef IN_FLASH_WITH_BOOTMGR
SYM_NAME(SRAM_UNDEFINED_ISR):		.long (CYGMON_VECTORS_SA + 0x04) 
SYM_NAME(SRAM_SWI_ISR):                 .long (CYGMON_VECTORS_SA + 0x08)
SYM_NAME(SRAM_PREFETCH_ABORT_ISR):      .long (CYGMON_VECTORS_SA + 0x0C)
SYM_NAME(SRAM_DATA_ABORT_ISR):		.long (CYGMON_VECTORS_SA + 0x10)
SYM_NAME(SRAM_ADDR_ERROR_ISR):		.long (CYGMON_VECTORS_SA + 0x14)
SYM_NAME(SRAM_IRQ_ISR):                 .long (CYGMON_VECTORS_SA + 0x18)
SYM_NAME(SRAM_FIQ_ISR):                 .long (CYGMON_VECTORS_SA + 0x1C)
#endif //#ifdef IN_FLASH_WITH_BOOTMGR 
