/*
 * arp.c -- Polling ARP/RARP for Cygnus BSP
 *
 * Copyright (c) 1998 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors

 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */
#include <stdlib.h>
#include <bsp/bsp.h>
#include "bsp_if.h"
#include "net.h"

/* #define PRINTOUT_MS_TICKS_INFO	1 */  /* this bsp_print may relate */
                   /* to ARP_REQ timing, put it back when necessary */
/* #define PRINTOUT_ARPREQ_WAITING_INFO	1 */   /* this bsp_print may relate */
                   /* to ARP_REQ timing, put it back when necessary */

static struct {
    int      waiting;
    char     *eth;
    char     *ip;
} arp_req;

/*
 * Handle incoming ARP packets.
 */
void
__arp_handler(pktbuf_t *pkt)
{
  int foo = 0;
    arp_header_t *arp = pkt->arp_hdr;

    /*
     * Only handle ethernet hardware and IP protocol.
     */
#ifdef DEBUG_ARP
bsp_printf("__arp_handler, hw_type %x, proto=%x\n",
    arp->hw_type, arp->protocol);
bsp_printf("__arp_handler, htons hw_type %x, proto=%x\n",
    htons(ARP_HW_ETHER), htons(ETH_TYPE_IP));
#endif // DEBUG_ARP

    if (arp->hw_type == htons(ARP_HW_ETHER))
{
    if (arp->protocol == htons(ETH_TYPE_IP))
        foo = 1;
}

#ifdef DEBUG_ARP
bsp_printf("proto == ETH_TYPE_IP\n");
#endif // DEBUG_ARP

/*
    if ((arp->hw_type == (short) htons(ARP_HW_ETHER)) &&
	(arp->protocol == (short) htons(ETH_TYPE_IP)))
*/
     if (foo)
      {
	/*
	 * Handle requests for our ethernet address.
	 */
#ifdef DEBUG_ARP
bsp_printf("entered ARP_HW_ETHER target ip=%x local ip=\n",
          arp->target_ip, __local_ip_addr);
#endif // DEBUG_ARP

	if (!memcmp(arp->target_ip, __local_ip_addr, 4)) {
	    if (arp->opcode == htons(ARP_REQUEST)) {
#ifdef DEBUG_ARP
bsp_printf("ARP_REQUEST\n");
#endif // DEBUG_ARP
		/* format response. */
		arp->opcode = htons(ARP_REPLY);
		memcpy(arp->target_ip, arp->sender_ip,
		       sizeof(ip_addr_t));
#ifdef DEBUG_ARP
bsp_printf("ARP sender IP =%x\n", arp->sender_ip);
#endif // DEBUG_ARP
		memcpy(arp->target_enet, arp->sender_enet,
		       sizeof(enet_addr_t));
		memcpy(arp->sender_ip, __local_ip_addr,
		       sizeof(ip_addr_t));
		memcpy(arp->sender_enet, __local_enet_addr,
		       sizeof(enet_addr_t));
		pkt->pkt_bytes = sizeof(arp_header_t);
#ifdef DEBUG_ARP
bsp_printf("pkt->pkt_bytes=%x\n", pkt->pkt_bytes);
#endif // DEBUG_ARP
		__enet_send(pkt, arp->target_enet, ETH_TYPE_ARP);

	    } else if (arp->opcode == htons(ARP_REPLY) && arp_req.waiting) {
		if (!memcmp(arp_req.ip, arp->sender_ip, sizeof(ip_addr_t))) {
                    memcpy(arp_req.eth, arp->sender_enet, sizeof(enet_addr_t));
		    arp_req.waiting = 0;
		}
	    }
	}
    }
    __pktbuf_free(pkt);
}


/* 
 * Find the ethernet address of the machine with the given
 * ip address.
 * Return true and fills in 'eth_addr' if successful, false
 * if unsuccessful.
 */
int
__arp_request(ip_addr_t ip_addr, enet_addr_t eth_addr)
{
    pktbuf_t *pkt;
    arp_header_t *arp;
    unsigned long retry_start;
    enet_addr_t   bcast_addr;
    int           retry;

    /* just fail if can't get a buffer */
    if ((pkt = __pktbuf_alloc(ARP_PKT_SIZE)) == NULL)
	return 0;

    arp = pkt->arp_hdr;
    arp->opcode = htons(ARP_REQUEST);
    arp->hw_type = htons(ARP_HW_ETHER);
    arp->protocol = htons(0x800);
    arp->hw_len = sizeof(enet_addr_t);
    arp->proto_len = sizeof(ip_addr_t);

    memcpy(arp->sender_ip, __local_ip_addr, sizeof(ip_addr_t));
    memcpy(arp->sender_enet, __local_enet_addr, sizeof(enet_addr_t));
    memcpy(arp->target_ip, ip_addr, sizeof(ip_addr_t));

    bcast_addr[0] = 255;
    bcast_addr[1] = 255;
    bcast_addr[2] = 255;
    bcast_addr[3] = 255;
    bcast_addr[4] = 255;
    bcast_addr[5] = 255;

    arp_req.eth = eth_addr;
    arp_req.ip = ip_addr;
    arp_req.waiting = 1;

//DC modify bsp_ms_ticks() in timer.c to increase 1 
//every time called, from measurement, call it 13339 times 
//about cost 1 sec, 10 retry only cost about 10 sec, 
//should be tolerable 
    retry = 10;
    while (retry-- > 0) {

        /* send the packet */
	pkt->pkt_bytes = sizeof(arp_header_t);
        __enet_send(pkt, bcast_addr, ETH_TYPE_ARP);

        retry_start = MS_TICKS();

#ifdef PRINTOUT_MS_TICKS_INFO
	bsp_printf("before MS_TICKS\n");
#endif /* #ifdef PRINTOUT_MS_TICKS_INFO */

//DC modify bsp_ms_ticks() in timer.c to increase 1 
//every time called, from measurement, call it 13339 times 
//about cost 1 sec, 10 retry only cost about 10 sec, 
//should be tolerable 
	while ((MS_TICKS() - retry_start) < 13393) {

#ifdef PRINTOUT_MS_TICKS_INFO
	bsp_printf("MS_TICKS=%d, retry_start=%d\n", MS_TICKS(), retry_start);
#endif /* #ifdef PRINTOUT_MS_TICKS_INFO */

	    __enet_poll();

#ifdef PRINTOUT_ARPREQ_WAITING_INFO 
	bsp_printf("returned from __enet_poll, arp_req.waiting=%x\n",
            arp_req.waiting);
#endif /* #ifdef PRINTOUT_ARPREQ_WAITING_INFO */

	    if (!arp_req.waiting) {
		__pktbuf_free(pkt);
		return 1;
	    }
	}
    }
    __pktbuf_free(pkt);
#ifdef PRINTOUT_MS_TICKS_INFO
   bsp_printf("ARP_REQ fail, MS_TICKS=%d, retry_start=%d, retry=%d\n", 
       MS_TICKS(), retry_start, retry);
#endif /* #ifdef PRINTOUT_MS_TICKS_INFO */
    return 0;
}

