/*
 * bootp.c -- Polling BOOTP for Cygnus BSP
 *
 * Copyright (c) 1998 Cygnus Support
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */
#include <stdlib.h>
#include <bsp/bsp.h>
#include "bsp_if.h"
#include "net.h"

//#define SHOW_BOOTP_DEBUG_INFO	1

extern void ixp1200_sw_delay_msec(int msec);

#define SHOULD_BE_RANDOM  0x12345555

#ifdef __CPU_IXP1200__
extern unsigned int lip_bootp;
extern unsigned int rip_bootp;
#endif //#ifdef __CPU_IXP1200__
/* How many milliseconds to wait before retrying the request */
#define RETRY_TIME  2000


static void
bootp_handler(udp_socket_t *skt, char *buf, int len,
	      ip_route_t *src_route, word src_port)
{
    ip_addr_t bootpsvr_ip_addr;
    bootp_header_t *b;

    b = (bootp_header_t *)buf;

    if (b->opcode == BOOTP_OP_REPLY && 
	!memcmp(b->client_hw, __local_enet_addr, 6)) {
	memcpy(__local_ip_addr, b->your_ip, 4);
	memcpy(bootpsvr_ip_addr, b->server_ip, 4);
#ifdef __CPU_IXP1200__
        lip_bootp=((unsigned int)(__local_ip_addr[0] << 24) & 0xFF000000) | 
                  ((unsigned int)(__local_ip_addr[1] << 16) & 0x00FF0000) | 
                  ((unsigned int)(__local_ip_addr[2] << 8) & 0x0000FF00) | 
                  ((unsigned int)(__local_ip_addr[3]) & 0x000000FF) ; 
        rip_bootp=((unsigned int)(bootpsvr_ip_addr[0] << 24) & 0xFF000000) | 
                  ((unsigned int)(bootpsvr_ip_addr[1] << 16) & 0x00FF0000) | 
                  ((unsigned int)(bootpsvr_ip_addr[2] << 8) & 0x0000FF00) | 
                  ((unsigned int)(bootpsvr_ip_addr[3]) & 0x000000FF) ; 
#ifdef SHOW_BOOTP_DEBUG_INFO
        bsp_printf("BOOTP got lip=%d.%d.%d.%d\n",
            __local_ip_addr[0], __local_ip_addr[1],
            __local_ip_addr[2], __local_ip_addr[3]);
        bsp_printf("BOOTP got rip=%d.%d.%d.%d\n",
            bootpsvr_ip_addr[0], bootpsvr_ip_addr[1],
            bootpsvr_ip_addr[2], bootpsvr_ip_addr[3]);
#endif //#ifdef SHOW_BOOTP_DEBUG_INFO
#endif //#ifdef __CPU_IXP1200__
    }
}


/*
 * Find our IP address and copy to __local_ip_addr.
 * Return zero if successful, -1 if not.
 */
int
__bootp_find_local_ip(void)
{
    udp_socket_t udp_skt;
    bootp_header_t b;
    ip_route_t     r;
    int            retry;
//    unsigned long  start;
    int i=0;
 
#ifdef SHOW_BOOTP_DEBUG_INFO
        bsp_printf("__bootp_find_local_ip called.\n");
#endif //#ifdef SHOW_BOOTP_DEBUG_INFO
    memset(&b, 0, sizeof(b));

    b.opcode = BOOTP_OP_REQUEST;
    b.hw_type = ARP_HW_ETHER;
    b.hw_len = 6;
    b.trans_id = SHOULD_BE_RANDOM;

    __local_ip_addr[0] = 0;
    __local_ip_addr[1] = 0;
    __local_ip_addr[2] = 0;
    __local_ip_addr[3] = 0;

    memcpy(b.client_hw, __local_enet_addr, 6);
   
    /* fill out route for a broadcast */
    r.ip_addr[0] = 255;
    r.ip_addr[1] = 255;
    r.ip_addr[2] = 255;
    r.ip_addr[3] = 255;
    r.enet_addr[0] = 255;
    r.enet_addr[1] = 255;
    r.enet_addr[2] = 255;
    r.enet_addr[3] = 255;
    r.enet_addr[4] = 255;
    r.enet_addr[5] = 255;

    /* setup a socket listener for bootp replies */
    __udp_install_listener(&udp_skt, 68, bootp_handler);

    retry = 10;
    while (retry-- > 0) {
#ifdef SHOW_BOOTP_DEBUG_INFO
//    bsp_printf("retry=%d.\n", retry);
#endif //#ifdef SHOW_BOOTP_DEBUG_INFO

        //DC every try take about 2 sec
        b.seconds=htons((unsigned short)((9-retry)*2)); 
//DC for ver. A.1 chip, following is a pacth for big endian, also
//we increase the size of bootp struct by 4 for big endian
#ifdef __LITTLE_ENDIAN__ 
	__udp_send((char *)&b, (sizeof(b) -4), &r, 67, 68);
#else //#ifdef __LITTLE_ENDIAN__ 
	__udp_send((char *)&b, (sizeof(b) -2), &r, 67, 68);
#endif //#ifdef __LITTLE_ENDIAN__ 

        for(i=0; i < (RETRY_TIME/100); i++)  //every 100msec enet_poll once
	{
            ixp1200_sw_delay_msec(100);
	    __enet_poll();
	    if (__local_ip_addr[0] || __local_ip_addr[1] ||
		__local_ip_addr[2] || __local_ip_addr[3]) {
		/* success */
		__udp_remove_listener(68);
		return 0;
	    }
	} 
    }

    /* timed out */
    __udp_remove_listener(68);
    return -1;
}


