/* #define BSP_PRINTOUT_DEBUG_INFO	1  */
/* #define  DISPLAY_LOCAL_IP 	1 */

#define CYGMON
#define DEBUG_TFTP_SEND         0
#define DEBUG_TFTP_RCV          0

#ifdef CYGMON
#include <string.h>
#include <bsp/bsp.h>
#include "bsp_if.h"
#include "net.h"
#endif // CYGMON

#define CONF_TIMEOUT_MINOR_BASE	10
#define CONF_TIMEOUT_MAJOR_BASE	500
#define CONF_TIMEOUT_MULT	5/4
#define CONF_TIMEOUT_MAX	1500
#define CONF_RETRIES		10

#define TFTP_RRQ		1
#define TFTP_DATA		3
#define TFTP_ACK		4
#define TFTP_ERROR		5

#define TFTP_STATE_RRQ		0
#define TFTP_STATE_ACK		1
#define TFTP_STATE_COMPLETE	2

typedef unsigned int	u32;
typedef unsigned short	u16;
typedef unsigned char	u8;
typedef enum {
	media_autodetect,
	media_coax,
	media_utp,
	media_aui
} media_t;

struct bootp_pkt {		/* BOOTP packet format */
	u8 op;			/* 1=request, 2=reply */
	u8 htype;		/* HW address type */
	u8 hlen;		/* HW address length */
	u8 hops;		/* Used only by gateways */
	u32 xid;		/* Transaction ID */
	u16 secs;		/* Seconds since we started */
	u16 flags;		/* Just what it says */
	u32 client_ip;		/* Client's IP address if known */
	u32 your_ip;		/* Assigned IP address */
	u32 server_ip;		/* Server's IP address */
	u32 relay_ip;		/* IP address of BOOTP relay */
	u8 hw_addr[16];		/* Client's HW address */
	u8 serv_name[64];	/* Server host name */
	u8 boot_file[128];	/* Name of boot file */
	u8 vendor_area[128];	/* Area for extensions */
};

struct buflist {
	struct buflist	*next;
	void		*data;
	int		 size;
};

struct netdev {
	const		char *name;

	struct netdev	*next;
	u32		io_base;
	u8		hw_addr[6];
	u8		hw_broadcast[6];
	u8		hw_addr_len;

	u32		ip_addr;

	media_t		media;
	int		up:1;
};

struct sin {
	u32	sin_addr;
	u16	sin_port;
};

typedef struct {
    unsigned int    ip_addr;
    enet_addr_t     enet_addr;
} ip_route_lib_t;

#define MAX_IP_ROUTE_ENTRY	10
static int cur_ip_route_entry=0;
static ip_route_lib_t ip_route_lib[MAX_IP_ROUTE_ENTRY];
static void add_entry_to_ip_route_lib(unsigned int ip_addr, ip_route_t dest);
static int get_mac_from_ip_route_lib(unsigned int ip_addr, 
    enet_addr_t eth_addr);

extern int __arp_request(ip_addr_t ip_addr, enet_addr_t eth_addr);

struct tftp_rrq {
	u16	cmd;
	u8	filename[100];
};

struct tftp_ack {
	u16	cmd;
	u16	block;
};

struct tftp_data {
	u16	cmd;
	u16	block;
	u8	data[512];
};

extern void _init_i82559();
extern void disable_i82559(); 
static struct bootp_pkt	 ic_bootp;
static struct netdev	*ic_netdev;
static struct sin	 tftp_me, tftp_serv;

#ifdef CYGMON
extern		 ip_addr_t __local_ip_addr;
ip_route_t       dest_route;
unsigned         server_src_port;
unsigned         did_rcv_buff = 0;
int              bytes_received;
void             *rcv_buffer;
unsigned	 centisecs=0;	//DC initialized to 0
u32              load_addr;

int udp_send(struct netdev *ic_netdev,
             struct sin *tftp_me,
             struct sin *tftp_serv,
             struct buflist *bl)
{

    memcpy(((char *)dest_route.ip_addr), &tftp_serv->sin_addr, sizeof(u32));

    __udp_send((char*) bl->data,  bl->size,
               &dest_route,
               tftp_serv->sin_port,
               tftp_me->sin_port);
/*  bsp_printf("udp_send done\n"); */
    return (0);		// Return no errors
}


int udp_recv(struct netdev *ic_netdev,
             struct sin *from,
             struct sin *to,
	     void   *buffer,
             int    sizeof_data)
{
    did_rcv_buff = 0;
    rcv_buffer = buffer;
    bytes_received = 0;

    tftp_me.sin_port = 0x040a;

    // Keep polling till the packet arrives
    while (!did_rcv_buff)
       __enet_poll();

    return (bytes_received);
}


static void
tftp_rcv_handler(udp_socket_t *skt, char *buf, int len,
              ip_route_t *src_route, word src_port)
{

#if DEBUG_TFTP_RCV
    bsp_printf("tftp_rcv_handler: buf=%x, rcv_buf=%x, len=%d, src_port=%d\n",
             buf, rcv_buffer, len, src_port);
#endif // DEBUG_TFTP_RCV

    memcpy(rcv_buffer, buf, len);

    server_src_port = src_port;
    bytes_received = len;
    did_rcv_buff = 1;
}


int ftp_setup(unsigned server_ip, unsigned local_ip,
               char *file_name, unsigned char *aload_addr)
{
    int  result = 0;

    strcpy(ic_bootp.boot_file, file_name);
    server_ip = htonl(server_ip);
    ic_bootp.server_ip = server_ip;
    load_addr = (u32) aload_addr;

#ifdef DISPLAY_LOCAL_IP
    __local_ip_addr[0]= local_ip & 0xFF;
    __local_ip_addr[1]= (local_ip>>8) & 0xFF;
    __local_ip_addr[2]= (local_ip>>16) & 0xFF;
    __local_ip_addr[3]= (local_ip>>24) & 0xFF;
bsp_printf("ftp_setup: local_ip %d.%d.%d.%d\n",
    (unsigned) __local_ip_addr[0],
    (unsigned) __local_ip_addr[1],
    (unsigned) __local_ip_addr[2],
    (unsigned) __local_ip_addr[3]);
#endif /* #ifdef DISPLAY_LOCAL_IP */


    local_ip = htonl(local_ip);
    memcpy(((char *)__local_ip_addr), &local_ip, sizeof(u32));

    //DC following re-initialiaze i82559 for following tftp
    //operation
    _init_i82559(); 
#ifdef BSP_PRINTOUT_DEBUG_INFO
bsp_printf("Before ARP querry\n");
#endif  /* #ifdef BSP_PRINTOUT_DEBUG_INFO */
    //first get server mac address from current ip_route_lib 
    if (get_mac_from_ip_route_lib(server_ip, &dest_route.enet_addr))
    {
#ifdef BSP_PRINTOUT_DEBUG_INFO
        bsp_printf(" get from lib tmac=%2.2X.%2.2X.%2.2X.%2.2X.%2.2X.%2.2X\n",
               dest_route.enet_addr[0],
               dest_route.enet_addr[1],
               dest_route.enet_addr[2],
               dest_route.enet_addr[3],
               dest_route.enet_addr[4],
               dest_route.enet_addr[5]);
#endif  //#ifdef BSP_PRINTOUT_DEBUG_INFO
        result = 1;
    }
    else 
    {
        if (__arp_request(&server_ip, &dest_route.enet_addr))
        {
            result = 1;
            add_entry_to_ip_route_lib(server_ip, dest_route);
#ifdef BSP_PRINTOUT_DEBUG_INFO
            bsp_printf(" get from arp tmac=%2.2X.%2.2X.%2.2X.%2.2X.%2.2X.%2.2X\n",
               dest_route.enet_addr[0],
               dest_route.enet_addr[1],
               dest_route.enet_addr[2],
               dest_route.enet_addr[3],
               dest_route.enet_addr[4],
               dest_route.enet_addr[5]);
#endif  /* #ifdef BSP_PRINTOUT_DEBUG_INFO */
        }
        else
        {
//            disable_i82559(); //DC disable i82559, prevent unexpected 
                              // garbage packets transmission, will 
                              //reinitialize it, when we need it
            result=0;
#ifdef BSP_PRINTOUT_DEBUG_INFO 
            bsp_printf("Failed to resolve the destination MAC addr\n");
#endif  /* #ifdef BSP_PRINTOUT_DEBUG_INFO */
        }
    }
#ifdef BSP_PRINTOUT_DEBUG_INFO 
        bsp_printf("ARP querry done\n");
        bsp_printf("ftp_setup: file_name=%s, serv_addr=%x, local_addr=%x,\
            load_addr=%x\n",
           file_name, ic_bootp.server_ip, local_ip, aload_addr);
#endif  /* #ifdef BSP_PRINTOUT_DEBUG_INFO */
    return result;
}

#endif // CYGMON


static int tftp_send_rrq(void)
{
	struct tftp_rrq rrq;
	struct buflist bl;
        int i=0, len=0;
        u8 octet_str[5]="octet";

	tftp_me.sin_addr = ic_netdev->ip_addr;
	tftp_me.sin_port = 0x040a;

	tftp_serv.sin_addr = ic_bootp.server_ip;
	tftp_serv.sin_port = 0x45;

#if DEBUG_TFTP_SEND
        bsp_printf("tftp_send_rrq: serv_addr=%x, serv_port=%x, my_port=%x\n",
             tftp_me.sin_addr, tftp_me.sin_port, tftp_me.sin_port);
#endif // DEBUG_TFTP_SEND

#ifdef CYGMON
	memset(&rrq, 0, sizeof(rrq));
#else
	memzero(&rrq, sizeof(rrq));
#endif // CYGMON
	rrq.cmd = htons(TFTP_RRQ);

        len=strlen(ic_bootp.boot_file);
        for(i=0; i < len; i++)
            rrq.filename[i]=ic_bootp.boot_file[i];
        rrq.filename[len]=0;
        len++;
        for(i=0; i < 5; i++)
            rrq.filename[len+i]=octet_str[i];
        rrq.filename[len+5]=0;  

#ifdef BSP_PRINTOUT_DEBUG_INFO
        bsp_printf("rrq_fn=%s\n", rrq.filename);
#endif  /* #ifdef BSP_PRINTOUT_DEBUG_INFO */

	bl.data = &rrq;
	bl.size = strlen(rrq.filename) + strlen("octet") + 4;  
	bl.next = NULL;

#ifdef BSP_PRINTOUT_DEBUG_INFO
       bsp_printf("tftp_send_rrq: netdev %x %x %x %x %x %x\n",
             ic_netdev->hw_addr[0],
             ic_netdev->hw_addr[1],
             ic_netdev->hw_addr[2],
             ic_netdev->hw_addr[3],
             ic_netdev->hw_addr[4],
             ic_netdev->hw_addr[5]);
#endif  /* #ifdef BSP_PRINTOUT_DEBUG_INFO */

	return udp_send(ic_netdev, &tftp_me, &tftp_serv, &bl);
}

static int tftp_send_ack(int block)
{
	struct tftp_ack ack;
	struct buflist bl;

	ack.cmd   = htons(TFTP_ACK);
	ack.block = htons(block);

	bl.data = &ack;
	bl.size = 102;
	bl.next = NULL;

	tftp_serv.sin_port = server_src_port;

#if DEBUG_TFTP_SEND
    bsp_printf("tftp_send_ack:\n");
#endif // DEBUG_TFTP_SEND

	return udp_send(ic_netdev, &tftp_me, &tftp_serv, &bl);
}


static int tftp_recv_data(int block, void *buffer)
{
	struct tftp_data data;
	int bytes, recvd_block;

	if (block == 1)
		tftp_serv.sin_port = 0;

#if DEBUG_TFTP_RCV
        bsp_printf("tftp_recv_data: calling udp_recv\n"); 
#endif // DEBUG_TFTP_RCV

	bytes = udp_recv(ic_netdev, &tftp_serv, &tftp_me, &data, sizeof(data));

#if DEBUG_TFTP_RCV
        bsp_printf("tftp_recv_data: udp_recv done, size=%d\n", bytes); 
#endif // DEBUG_TFTP_RCV

	if (!bytes)
		return 0;

	if (data.cmd != htons(TFTP_DATA))
		return 0;

	recvd_block = htons(data.block);

	if (recvd_block <= block)
		tftp_send_ack(recvd_block);

	if (recvd_block != block)
		return 0;

#if DEBUG_TFTP_RCV
        bsp_printf("tftp_recv_data: copying data to=%x, =%d\n",
		    buffer, bytes - 4); 
#endif // DEBUG_TFTP_RCV
	memcpy(buffer, data.data, bytes - 4);

	return bytes - 4;
}

#define TWIDDLE_IDX(x)	  (((x) >> 16) & 3)
#define TWIDDLE_UPDATE(x) (((x) & 0xffff) == 0)

udp_socket_t udp_skt;
//unsigned     tftp_listen_port = 0x45;
unsigned     tftp_listen_port = 1033;
unsigned     tftp_listen_data_port = 1034;

int do_tftp(void)
{
	static char twiddle[] = { '|', '/', '-', '\\' };
	int timeout, targ, retries, error, block = 1, bytes = 0;
	int total_bytes = 0;
	unsigned char *data = (unsigned char *)load_addr;


#ifdef CYGMON
	bsp_printf("TFTPing %s... \n", ic_bootp.boot_file);

        /* setup a socket listener for ftpd replies */
        __udp_install_listener(&udp_skt, tftp_listen_data_port,
				tftp_rcv_handler);
#else
	printf("TFTPing %s... ", ic_bootp.boot_file);
#endif // CYGMON

	timeout = CONF_TIMEOUT_MAJOR_BASE;
	retries = CONF_RETRIES;

	do {
		if (block == 1)
                {
#if DEBUG_TFTP_SEND
                     bsp_printf("do_tftp: calling tftp_send_rrq\n");
#endif // DEBUG_TFTP_SEND

		     error = tftp_send_rrq();

#if DEBUG_TFTP_SEND
                     bsp_printf("do_tftp: tftp_send_rrq done\n");
#endif // DEBUG_TFTP_SEND
                }
		else
			error = tftp_send_ack(block);

#if DEBUG_TFTP_SEND
                bsp_printf("do_tftp: tftp_send_ack result=%d \n", error);
#endif // DEBUG_TFTP_SEND
		if (error)
			break;

		targ = centisecs + timeout;

		while (centisecs < targ) {
			bytes = tftp_recv_data(block, data + total_bytes);
			if (bytes) {
#if DEBUG_TFTP_RCV
		bsp_printf("c=%8.8x, t=%8.8x\n", centisecs, timeout);  //DC test
#endif //#if DEBUG_TFTP_RCV
				retries = CONF_RETRIES;
				timeout = CONF_TIMEOUT_MINOR_BASE;
				targ = centisecs + timeout;

				if (TWIDDLE_UPDATE(total_bytes))
				    bsp_printf("%c\010",
                                          twiddle[TWIDDLE_IDX(total_bytes)]);
				total_bytes += bytes;
#if DEBUG_TFTP_RCV
        bsp_printf("ftp_recv_data done, block=%d size=%d total_size\n",
                                    block, bytes, total_bytes);
#endif // DEBUG_TFTP_RCV
				if (bytes == 512)
					block ++;
				else
					break;
			}
		}

		if (timeout == CONF_TIMEOUT_MINOR_BASE)
			timeout = CONF_TIMEOUT_MAJOR_BASE;

		if (bytes)
			break;

		if (!--retries) {
			bsp_printf("\010 timed out\n");
			error = 1;
			break;
		}

		timeout = timeout * CONF_TIMEOUT_MULT;
		if (timeout > CONF_TIMEOUT_MAX)
			timeout = CONF_TIMEOUT_MAX;
#if DEBUG_TFTP_RCV
		bsp_printf("c=%8.8x, t=%8.8x\n", centisecs, timeout);  
#endif //#if DEBUG_TFTP_RCV
	} while (1);

        bsp_printf(" file transfer complete - total %d bytes\n", total_bytes);

#ifdef CYGMON
        __udp_remove_listener(tftp_listen_data_port);
#endif // CYGMON

	if (error) {
		bsp_printf("\010 Error\n");
		return 1;
	}

#ifdef BSP_PRINTOUT_DEBUG_INFO
#ifdef CYGMON
	bsp_printf("\010 Ok - %d KB\n", total_bytes / 1024);
#else
	printf("\010 Ok - %dKB\n", total_bytes / 1024);
#endif // CYGMON
#endif  /* #ifdef BSP_PRINTOUT_DEBUG_INFO */

	return 0;
}


static void add_entry_to_ip_route_lib(unsigned int ip_addr, ip_route_t dest)
{
 int i=0;
    if(cur_ip_route_entry < MAX_IP_ROUTE_ENTRY)
    {
        ip_route_lib[cur_ip_route_entry].ip_addr=ip_addr;
        for(i=0; i < 6; i++)
            ip_route_lib[cur_ip_route_entry].enet_addr[i]=
                dest.enet_addr[i];
        cur_ip_route_entry++;
    }
}

static int get_mac_from_ip_route_lib(unsigned int ip_addr, 
    enet_addr_t eth_addr)
{
 int i=0, j=0, result=0;
    for(i=0; i < cur_ip_route_entry; i++)
    {
        if(ip_route_lib[i].ip_addr==ip_addr)
        {
            for(j=0; j < 6; j++)
                eth_addr[j]=ip_route_lib[i].enet_addr[j];
            result=1;
            return result; 
         }
    }
    return result;
}

