/* libopcodes support functions for the minimal BFD header
 *
 * Copyright (c) 1997, 1998, 1999 Cygnus Solutions
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */
#include <stdio.h>
#include <stdlib.h>
#include <bfd.h>
#include <monitor.h>
#include <dis-asm.h>

#ifdef HAVE_BSP
#include "cpu_info.h"
#include <bsp/bsp.h>
#endif

#include <board.h>

#include "fmt_util.h"
#include "tservice.h"

#define FPRINTF xfprintf
#define FFLUSH(x)

int
xfprintf(FILE *f, const char *fmt, ...)
{
  va_list ap;

  va_start (ap, fmt);
  xvprintf (fmt, ap);
  va_end (ap);

  return 0;
}

static mem_addr_t address;

int
getdata (bfd_vma addr, bfd_byte *dst, int length,
	 disassemble_info *d)
{
  mem_addr_t res;

  ADD_OFFSET(&address, &res, addr);
  return read_memory (&res, 1, length, dst);
}

static void
memerr (int status, bfd_vma memaddr, disassemble_info *i)
{
  FPRINTF (stderr, "error of %d\n", status);
}

static void 
paddrf (bfd_vma addr, disassemble_info *info)
{
  char buf[30];
  mem_addr_t res;

  ADD_OFFSET(&address, &res, addr);
  addr2str (&res, buf);
  info->fprintf_func (info->stream, "0x%s", buf);
}

void
sprintf_vma(void *s, bfd_vma x)
{
  mem_addr_t res;

  ADD_OFFSET(&address, &res, x);
  addr2str (&res, s);
}

mem_addr_t
do_dis(mem_addr_t *addr)
{
  static disassemble_info inf;
  static char buf[512];
  static int initted = 0;
  int offset = 0;
  mem_addr_t res;

  if (!initted)
    {
      initted = 1;
      inf.fprintf_func = (fprintf_ftype)FPRINTF;
      inf.stream = stdout;
      inf.flavour = bfd_target_unknown_flavour;
      inf.endian = BFD_ENDIAN_BIG;
      inf.symbols = NULL;
      inf.num_symbols = 0;
      inf.read_memory_func = getdata;
      inf.memory_error_func = memerr;
      inf.print_address_func = paddrf;
      inf.buffer = buf;
      inf.buffer_vma = 0;
      inf.mach = BFD_MACH;
    }
  address = *addr;
  inf.print_address_func (offset, &inf);
  inf.fprintf_func (inf.stream, " ");
  FFLUSH (inf.stream);
  offset = PRINT_INSN (offset, &inf);
  FFLUSH (inf.stream);
  ADD_OFFSET (addr, &res, offset);
  xprintf("\n") ;
  return res;
}

unsigned long
bfd_getb32(const unsigned char *p)
{
  return (((unsigned long) p[0]) << 24) | (((unsigned long) p[1]) <<16)
	  | (((unsigned long ) p[2]) <<8) | (unsigned long) p[3];
}

unsigned long
bfd_getl32(const unsigned char *p)
{
  return (((unsigned long) p[3]) << 24) | (((unsigned long) p[2])<<16)
	  | (((unsigned long) p[1])<<8) | (unsigned long ) p[0];
}

unsigned long
bfd_getb16(const unsigned char *p)
{
  return ((p[0] << 8) | p[1]) & 0xffff;
}

unsigned long
bfd_getl16(const unsigned char *p)
{
  return ((p[1] << 8) | p[0]) & 0xffff;
}


void
bfd_putb32 (unsigned long data, unsigned char *addr)
{
  addr[0] = (bfd_byte)(data >> 24);
  addr[1] = (bfd_byte)(data >> 16);
  addr[2] = (bfd_byte)(data >>  8);
  addr[3] = (bfd_byte)data;
}

void
bfd_putl32 (unsigned long data, unsigned char *addr)
{
  addr[0] = (bfd_byte)data;
  addr[1] = (bfd_byte)(data >>  8);
  addr[2] = (bfd_byte)(data >> 16);
  addr[3] = (bfd_byte)(data >> 24);
}

void
bfd_putb16 (unsigned long data, unsigned char *addr)
{
  addr[0] = (bfd_byte)(data >> 8);
  addr[1] = (bfd_byte )data;
}

void
bfd_putl16 (unsigned long data, unsigned char *addr)
{
  addr[0] = (bfd_byte )data;
  addr[1] = (bfd_byte)(data >> 8);
}

char *
xmalloc(int amt)
{
#ifdef NO_MALLOC
  xprintf("xmalloc(%d)!\n", amt);
  return NULL;
#else
  return malloc(amt);
#endif
}

void
flush_dis ()
{
  FFLUSH(stdout);
}
