
/* generic_bp32.c - Default function for setting and clearing breakpoints */

/*
 * Copyright (c) 1998, 1999 Cygnus Solutions
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

/* In the absence of more specific formula for setting and clearing
   breakpoints.  The logic here provides a default for processors with
   32 bit inttructions and which use a trap instruction to set or clear
   breakpoints.
   Selection of this file takes place in the configure.in file
   based on the target triple.
   Originally cloned from mn10300, but also used by mips.
 */
#ifdef HAVE_BSP
#include <bsp/bsp.h>
#include <bsp/cpu.h>
#endif
#include "monitor.h"
#include "tservice.h"
#include "stub-tservice.h"

#ifdef HAVE_BSP
#define _breakinst bsp_breakinsn
#else
extern unsigned char _breakinst[];
#endif

/* Set to 1 to enable debugging output.  */

#define DEBUG 0

/* ----- SET_BREAKPOINT --------------------------------------- */
/* Write a trap instruction to memory and fillin the structure
   used in managing breakpoints. protect from writes to inimplemented memory
   which might cause a bus error or trap.
*/


/* Breakpoint support.  */

#if DEBUG
void
print_value (char *prefix, unsigned long n)
{
  xprintf("%s = 0x%x\n", prefix, n);
}
#endif

void
set_breakpoint (struct bp *bp)
{
  if (! bp->in_memory)
    {
      /* Don't set a breakpoint at the current PC location. */
      if (bp->address.addr != get_pc())
	{
	  read_memory (&bp->address, sizeof (bp_inst_t), 1,
		       (char *)(&bp->old_inst));
	  bp->in_memory = 1;
	  write_memory (&bp->address, sizeof (bp_inst_t), 1,
		        (char *) _breakinst);
#if DEBUG
	  print_value ("set_breakpoint: address", bp->address.addr);
	  print_value ("set_breakpoint: value", *((bp_inst_t *)_breakinst));
#endif
#ifdef HAVE_BSP
	  bsp_flush_dcache((void *)bp->address.addr, sizeof (bp_inst_t));
	  bsp_flush_icache((void *)bp->address.addr, sizeof (bp_inst_t));
#else
	  __instruction_cache(CACHE_FLUSH);
#endif
	}
    }
}


void
clear_breakpoint (struct bp *bp)
{
  if (bp->in_memory)
    {
#if DEBUG
      print_value ("clear_breakpoint: address", bp->address.addr);
#endif
      write_memory (&bp->address, sizeof (bp_inst_t), 1,
		    (char *)(&bp->old_inst));
      bp->in_memory = 0;
#ifdef HAVE_BSP
      bsp_flush_dcache((void *)bp->address.addr, sizeof (bp_inst_t));
      bsp_flush_icache((void *)bp->address.addr, sizeof (bp_inst_t));
#else
      __instruction_cache(CACHE_FLUSH);
#endif
    }
}
