
/* generic_mem.c
  It may be appropriate to relay directly into the stubs
   implementation of read memory, without board specific
   considerations such as checking the allowed ranges of
   addresses. Perhaps there needs to be some consideration of address spaces or,
   masking addresses.
   
   This file implements a default version of reading and writing
   memory when none of this is an issue */

/*
 *
 * Copyright (c) 1998, 1999 Cygnus Solutions
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */
#ifdef HAVE_BSP
#include <bsp/bsp.h>
#include "cpu_info.h"
#endif
#include "board.h"
#include "monitor.h"
#include "tservice.h"

#ifndef HAVE_BSP
#include "generic-stub.h"
#endif


int
read_memory (mem_addr_t *src, int size, int amt, char *dst)
{
#ifdef HAVE_BSP
    return (bsp_memory_read((unsigned char *)src->addr, MEM_ADDR_ASI(src),
			    size << 3, amt, dst) != amt);
#else
  int totamt = size * amt;
  return (totamt != __read_mem_safe (dst, src->addr, totamt));
#endif
}

int
write_memory (mem_addr_t *dst, int size, int amt, char *src)
{
#ifdef HAVE_BSP
    return (bsp_memory_write((unsigned char *)dst->addr, MEM_ADDR_ASI(dst),
			     size << 3, amt, src) != amt);
#else
  int totamt = size * amt;
  return (totamt != __write_mem_safe (src, dst->addr, totamt));
#endif
}


   
