/* Monitor command definitions for the CygMON ROM monitor
 *
 * Copyright (c) 1997, 1998, 1999 Cygnus Solutions
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

#ifndef MONITOR_CMD_H
#define MONITOR_CMD_H

#ifndef ASM
typedef enum {
  INVOCATION, USAGE, SHORT_HELP, LONG_HELP
} cmdmode_t;

struct cmdentry
{
  char *alias;
  char *cmd;
  int (*function) (cmdmode_t);
};

extern struct cmdentry cmdtab [];

extern int monitor_loop (void);

extern char inbuf[];

extern char **argvect;


#ifdef USE_HELP
extern void usage (char *string);
extern void short_help (char *string);
extern void long_help (char *string);
extern void example (char *string);
#else
#define usage(x) no_help_usage ()
#define short_help(x) no_help()
#define long_help(x) no_help()
#define example(x)
extern void no_help (void);
extern void no_help_usage (void);
#endif
extern int help_cmd (cmdmode_t mode);
extern int mem_cmd (cmdmode_t mode);

extern int dump_cmd (cmdmode_t mode);
extern int ethaddr_cmd (cmdmode_t mode);
extern int ipaddr_cmd (cmdmode_t mode);
extern int tcpport_cmd (cmdmode_t mode);
extern int load_cmd (cmdmode_t mode);
extern int reg_cmd (cmdmode_t mode);
extern int go_cmd (cmdmode_t mode);
extern int othernames_cmd (cmdmode_t mode);
extern int step_cmd (cmdmode_t mode);
extern int transfer_cmd (cmdmode_t mode);
extern int timer_cmd (cmdmode_t mode);
extern int disassemble_cmd (cmdmode_t mode);
extern int breakpoint_cmd (cmdmode_t mode);
extern int clear_breakpoint_cmd (cmdmode_t mode);
extern int memusage_cmd (cmdmode_t mode);
extern int set_serial_port_cmd (cmdmode_t mode);
extern int set_serial_speed_cmd (cmdmode_t mode);
extern int version_cmd (cmdmode_t mode);
extern int cache_cmd (cmdmode_t mode);
extern int set_term_cmd (cmdmode_t mode);
extern int reset_cmd (cmdmode_t mode);
extern int copy_cmd (cmdmode_t mode);
extern int fill_cmd (cmdmode_t mode);
extern int set_program_args_cmd (cmdmode_t mode);
extern int swapmem_cmd (cmdmode_t mode);
extern int checksumcmd (cmdmode_t mode);
extern int int_cmd (cmdmode_t mode);

/* Load the S record in RECORD into memory. If successful, a positive
   value will be returned. If zero is returned, RECORD represents a
   termination record. If a negative value is returned, the record
   was defective in some way. */
extern int load_one_srecord (char *record);
#endif

#endif
