


/* This function copies data from rom to ram. it needs to be called
   before we do anything which depends in initialized data.
   it is possable to call this in main and avoid coding it in assembler
   but the crt0 stuff which preceedes main had better not test any of those
   variables.
   If you must call this from crt0, the stack must be setup already.
   Worse case: disassemble this and paste it into crt0 */

/* This function expects the linker script to setup the data segment twice,
   Once in ROM and once in ram. The loader loads it on rom but we link to the
   copy in ram.
   */
/* These symbols are established by the linker script. They are NOT
   real variables */
 
extern char _fdata ;  /* Beginning of data in ram */
extern char _edata ;  /* end of initialized data in rom */
extern char _etext ;


void relocate_data_from_rom(char * src) /* SRC is etext */
{
  char * dest =  &_fdata ;
  char * limit = &_edata ;
  /* src = & _etext ;  Doesnt work-  See note A below */
  if (src != dest)
  while (dest < limit)
      *dest++ = *src++ ;
}


/* NOTE A:
  This is what I would rather do than pass '_etext' from assembler
   But the linker  gives the following warning.
     relocate.o: In function `relocate_data_from_rom':
../../../../devo/cygmon/relocate.c(.text+0x8): relocation truncated to fit: R_MIPS_GPREL16 _etext
collect2: ld returned 1 exit status
  I suspect this is because addresses like this are resolved as offsets from
  the global pointer. In the case of etext, thats too big is etext is in
  rom. So, the assembler i used has 'la' load address instruction which
  has no such limitations. I coule use inline assembler here but
  than this fragment would become processor dependent.
*/



/* The linker allows us to setup the two data segments as follows.

   The linker considers two addresses: the address where things are
   located in memory at program runtime(VMA) and, the address where
   things are located in the generated executable (LMA). They can be
   different and, for code on ROM, the data section is generally placed
   in the ROM image immediatly after the text section,but is considered to
   live at the start of wherever ram section is defined.

   This fragment of the linker script looks like:

   ALIGN(4);
   msdata = SIZEOF(.rodata) + ADDR(.rodata);
   _msdata = SIZEOF(.rodata) + ADDR(.rodata);
   .data : AT(msdata) {
      sdata = .;
      _sdata = .;
      *(.data)
      ALIGN(4) ;
      edata = . ;
   } > ram

   The msdata symbol is set to the address at the end of the .rodata
   section. The AT(msdata)directive tells the linker to set the LMA of
   the data section to the value of msdata; thus, the data section
   will be initially loaded into ROM while the program that uses it
   sees addresses of variables in RAM. The "> ram" directive at the
   end tells the linker that sections VMA is in the ram area.
   */


