/*
 * Definitions for generic port
 *
 * Copyright (c) 1997, 1998 Cygnus Solutions
 *
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

#ifndef TEMPLATE_MON_H
#define TEMPLATE_MON_H
#include <board.h>

#ifndef ASM

/* The type of an address used for read_memory() and write_memory (). */
typedef struct mem_addr {
  target_register_t addr;
  /* !!! Any additional definitions would go here */
} mem_addr_t;

/* !!! Change to fit the definition of mem_addr_t as necessary */
#define MAKE_STD_ADDR(SRC, DST) ((DST)->addr = (SRC))
#define ADD_OFFSET(SRC,DST,OFFSET)  ((DST)->addr = (SRC)->addr + (OFFSET))
#define ADD_ALIGN(SRC,DST,ALIGN) ((DST)->addr = (SRC)->addr - ((SRC)->addr % (ALIGN)))
#define MEM_ADDR_EQ_P(A, B) ((A).addr == (B).addr)
#define MEM_ADDR_DIFF(A, B) ((A).addr - (B).addr)

/* Read AMT number of SIZE-sized quantities from SRC, and store them
   in DST. */
extern int read_memory (mem_addr_t *src, int size, int amt, char *dst);

/* Write AMT number of SIZE-sized quantities from SRC to DST. */
extern int write_memory (mem_addr_t *dst, int size, int amt, char *src);

/* Convert the address in STRING to a mem_addr_t. */
extern int str2addr (char *string, mem_addr_t *res);

/* Given ADDR, store an equivalent string representation in DEST. */
extern void addr2str (mem_addr_t *addr, char *dest);

/* !!! Type of one breakpoint instruction. On some machines this may have
   to be a struct that includes the length of the saved breakpoint 
   instruction. */
typedef long bp_inst_t;

/* We'd like to give these proper prototypes, but unfortunately this would
   be ugly. */
void set_breakpoint (/* struct bp *breakpoint */);
void clear_breakpoint (/* struct bp *breakpoint */);

/* !!! Set things up so that the next user resume will execute one instruction.
   This may be done by setting breakpoints or setting a "single step" flag
   in the saved user registers, for example. */
void single_step (void);

/* !!! Clear the single-step state. */
void clear_single_step (void);
#endif

#endif
